@@ -1,6 +1,6 @@
 #include "ast/ast.h"
 
-#include <iostream>
+#include <algorithm>
 
 #include "ast/visitors.h"
 #include "log.h"
@@ -41,9 +41,12 @@ MAKE_ACCEPT(AttachPoint)
 MAKE_ACCEPT(If)
 MAKE_ACCEPT(Unroll)
 MAKE_ACCEPT(While)
+MAKE_ACCEPT(For)
 MAKE_ACCEPT(Config)
 MAKE_ACCEPT(Jump)
 MAKE_ACCEPT(Probe)
+MAKE_ACCEPT(SubprogArg)
+MAKE_ACCEPT(Subprog)
 MAKE_ACCEPT(Program)
 
 #undef MAKE_ACCEPT
@@ -179,6 +182,12 @@ Unroll::~Unroll()
   delete stmts;
   stmts = nullptr;
 }
+Jump::~Jump()
+{
+  if (return_value)
+    delete return_value;
+  return_value = nullptr;
+}
 Predicate::~Predicate()
 {
   delete expr;
@@ -202,6 +211,15 @@ While::~While()
   delete stmts;
 }
 
+For::~For()
+{
+  delete decl;
+  delete expr;
+  for (auto *stmt : *stmts)
+    delete stmt;
+  delete stmts;
+}
+
 Config::~Config()
 {
   for (auto *stmt : *stmts)
@@ -209,6 +227,15 @@ Config::~Config()
   delete stmts;
 }
 
+Scope::~Scope()
+{
+  if (stmts)
+    for (auto *stmt : *stmts)
+      delete stmt;
+  delete stmts;
+  stmts = nullptr;
+}
+
 Probe::~Probe()
 {
   if (attach_points)
@@ -219,16 +246,22 @@ Probe::~Probe()
 
   delete pred;
   pred = nullptr;
+}
 
-  if (stmts)
-    for (Statement *s : *stmts)
-      delete s;
-  delete stmts;
-  stmts = nullptr;
+Subprog::~Subprog()
+{
+  if (args)
+    for (SubprogArg *a : *args)
+      delete a;
+  delete args;
 }
 
 Program::~Program()
 {
+  if (functions)
+    for (Subprog *s : *functions)
+      delete s;
+  delete functions;
   if (probes)
     for (Probe *p : *probes)
       delete p;
@@ -248,26 +281,22 @@ String::String(const std::string &str, location loc) : Expression(loc), str(str)
   is_literal = true;
 }
 
-
 StackMode::StackMode(const std::string &mode, location loc)
     : Expression(loc), mode(mode)
 {
   is_literal = true;
 }
 
-
 Builtin::Builtin(const std::string &ident, location loc)
     : Expression(loc), ident(is_deprecated(ident))
 {
 }
 
-
 Identifier::Identifier(const std::string &ident, location loc)
     : Expression(loc), ident(ident)
 {
 }
 
-
 PositionalParameter::PositionalParameter(PositionalParameterType ptype,
                                          long n,
                                          location loc)
@@ -276,7 +305,6 @@ PositionalParameter::PositionalParameter(PositionalParameterType ptype,
   is_literal = true;
 }
 
-
 Call::Call(const std::string &func, location loc)
     : Expression(loc), func(is_deprecated(func)), vargs(nullptr)
 {
@@ -316,8 +344,7 @@ Map::Map(const std::string &ident, ExpressionList *vargs, location loc)
     : Expression(loc), ident(ident), vargs(vargs)
 {
   is_map = true;
-  for (auto expr : *vargs)
-  {
+  for (auto expr : *vargs) {
     expr->key_for_map = this;
   }
 }
@@ -343,7 +370,6 @@ Unop::Unop(Operator op, Expression *expr, bool is_post_op, location loc)
 {
 }
 
-
 Ternary::Ternary(Expression *cond,
                  Expression *left,
                  Expression *right,
@@ -352,7 +378,6 @@ Ternary::Ternary(Expression *cond,
 {
 }
 
-
 FieldAccess::FieldAccess(Expression *expr,
                          const std::string &field,
                          location loc)
@@ -365,7 +390,6 @@ FieldAccess::FieldAccess(Expression *expr, ssize_t index, location loc)
 {
 }
 
-
 ArrayAccess::ArrayAccess(Expression *expr, Expression *indexpr, location loc)
     : Expression(loc), expr(expr), indexpr(indexpr)
 {
@@ -377,13 +401,11 @@ Cast::Cast(SizedType cast_type, Expression *expr, location loc)
   type = cast_type;
 }
 
-
 Tuple::Tuple(ExpressionList *elems, location loc)
     : Expression(loc), elems(elems)
 {
 }
 
-
 ExprStatement::ExprStatement(Expression *expr, location loc)
     : Statement(loc), expr(expr)
 {
@@ -419,13 +441,11 @@ Predicate::Predicate(Expression *expr, location loc) : Node(loc), expr(expr)
 {
 }
 
-
 AttachPoint::AttachPoint(const std::string &raw_input, location loc)
     : Node(loc), raw_input(raw_input)
 {
 }
 
-
 If::If(Expression *cond, StatementList *stmts) : cond(cond), stmts(stmts)
 {
 }
@@ -435,30 +455,57 @@ If::If(Expression *cond, StatementList *stmts, StatementList *else_stmts)
 {
 }
 
-
 Unroll::Unroll(Expression *expr, StatementList *stmts, location loc)
     : Statement(loc), expr(expr), stmts(stmts)
 {
 }
 
+Scope::Scope(StatementList *stmts) : stmts(stmts)
+{
+}
+
 Probe::Probe(AttachPointList *attach_points,
              Predicate *pred,
              StatementList *stmts)
-    : attach_points(attach_points), pred(pred), stmts(stmts)
+    : Scope(stmts), attach_points(attach_points), pred(pred)
+{
+}
+
+SubprogArg::SubprogArg(std::string name, SizedType type)
+    : type(std::move(type)), name_(std::move(name))
+{
+}
+
+std::string SubprogArg::name() const
+{
+  return name_;
+}
+
+Subprog::Subprog(std::string name,
+                 SizedType return_type,
+                 SubprogArgList *args,
+                 StatementList *stmts)
+    : Scope(stmts),
+      args(args),
+      return_type(std::move(return_type)),
+      name_(std::move(name))
 {
 }
 
 Program::Program(const std::string &c_definitions,
                  Config *config,
+                 SubprogList *functions,
                  ProbeList *probes)
-    : c_definitions(c_definitions), config(config), probes(probes)
+    : c_definitions(c_definitions),
+      config(config),
+      functions(functions),
+      probes(probes)
 {
 }
 
-std::string opstr(Jump &jump)
+std::string opstr(const Jump &jump)
 {
-  switch (jump.ident)
-  {
+  switch (jump.ident) {
     case JumpType::RETURN:
       return "return";
     case JumpType::BREAK:
@@ -472,7 +519,7 @@ std::string opstr(Jump &jump)
   return {}; // unreached
 }
 
-std::string opstr(Binop &binop)
+std::string opstr(const Binop &binop)
 {
   switch (binop.op) {
     case Operator::EQ:
@@ -518,7 +565,7 @@ std::string opstr(Binop &binop)
   return {}; // unreached
 }
 
-std::string opstr(Unop &unop)
+std::string opstr(const Unop &unop)
 {
   switch (unop.op) {
     case Operator::LNOT:
@@ -540,19 +587,69 @@ std::string opstr(Unop &unop)
   return {}; // unreached
 }
 
-std::string AttachPoint::name(const std::string &attach_target,
-                              const std::string &attach_point) const
+AttachPoint AttachPoint::create_expansion_copy(const std::string &match) const
+{
+  AttachPoint ap = *this; // copy here
+  switch (probetype(ap.provider)) {
+    case ProbeType::kprobe:
+    case ProbeType::kretprobe:
+      ap.func = match;
+      if (match.find(":") != std::string::npos)
+        ap.target = erase_prefix(ap.func);
+      break;
+    case ProbeType::uprobe:
+    case ProbeType::uretprobe:
+    case ProbeType::kfunc:
+    case ProbeType::kretfunc:
+    case ProbeType::tracepoint:
+      // Tracepoint, uprobe, and k(ret)func probes specify both a target
+      // (category for tracepoints, binary for uprobes, and kernel module
+      // for k(ret)func) and a function name.
+      ap.func = match;
+      ap.target = erase_prefix(ap.func);
+      break;
+    case ProbeType::usdt:
+      // USDT probes specify a target binary path, a provider, and a function
+      // name.
+      ap.func = match;
+      ap.target = erase_prefix(ap.func);
+      ap.ns = erase_prefix(ap.func);
+      break;
+    case ProbeType::watchpoint:
+    case ProbeType::asyncwatchpoint:
+      // Watchpoint probes come with target prefix. Strip the target to get the
+      // function
+      ap.func = match;
+      erase_prefix(ap.func);
+      break;
+    case ProbeType::rawtracepoint:
+      ap.func = match;
+      break;
+    case ProbeType::software:
+    case ProbeType::hardware:
+    case ProbeType::interval:
+    case ProbeType::profile:
+    case ProbeType::special:
+    case ProbeType::iter:
+    case ProbeType::invalid:
+      break;
+    default:
+      LOG(BUG) << "Unknown probe type";
+  }
+  return ap;
+}
+
+std::string AttachPoint::name() const
 {
   std::string n = provider;
-  if (attach_target != "")
-    n += ":" + attach_target;
+  if (target != "")
+    n += ":" + target;
   if (lang != "")
     n += ":" + lang;
   if (ns != "")
     n += ":" + ns;
-  if (attach_point != "")
-  {
-    n += ":" + attach_point;
+  if (func != "") {
+    n += ":" + func;
     if (func_offset != 0)
       n += "+" + std::to_string(func_offset);
   }
@@ -567,11 +664,6 @@ std::string AttachPoint::name(const std::string &attach_target,
   return n;
 }
 
-std::string AttachPoint::name(const std::string &attach_point) const
-{
-  return name(target, attach_point);
-}
-
 int AttachPoint::index() const
 {
   return index_;
@@ -584,28 +676,12 @@ void AttachPoint::set_index(int index)
 
 std::string Probe::name() const
 {
-  std::string n;
-  for (auto &attach_point : *attach_points)
-  {
-    if (!n.empty())
-      n += ',';
-    n += attach_point->provider;
-    if (attach_point->target != "")
-      n += ":" + attach_point->target;
-    if (attach_point->ns != "")
-      n += ":" + attach_point->ns;
-    if (attach_point->func != "")
-    {
-      n += ":" + attach_point->func;
-      if (attach_point->func_offset != 0)
-        n += "+" + std::to_string(attach_point->func_offset);
-    }
-    if (attach_point->address != 0)
-      n += ":" + std::to_string(attach_point->address);
-    if (attach_point->freq != 0)
-      n += ":" + std::to_string(attach_point->freq);
-  }
-  return n;
+  std::vector<std::string> ap_names;
+  std::transform(attach_points->begin(),
+                 attach_points->end(),
+                 std::back_inserter(ap_names),
+                 [](const AttachPoint *ap) { return ap->name(); });
+  return str_join(ap_names, ",");
 }
 
 std::string Probe::args_typename() const
@@ -618,7 +694,8 @@ int Probe::index() const
   return index_;
 }
 
-void Probe::set_index(int index) {
+void Probe::set_index(int index)
+{
   index_ = index;
 }
 
@@ -686,19 +763,34 @@ Cast::Cast(const Cast &other) : Expression(other)
 {
 }
 
-Probe::Probe(const Probe &other) : Node(other)
+SubprogArg::SubprogArg(const SubprogArg &other) : Node(other)
+{
+  name_ = other.name_;
+  type = other.type;
+}
+
+Subprog::Subprog(const Subprog &other) : Scope(other)
+{
+  name_ = other.name_;
+}
+
+Probe::Probe(const Probe &other) : Scope(static_cast<const Scope &>(other))
 {
   need_expansion = other.need_expansion;
   tp_args_structs_level = other.tp_args_structs_level;
   index_ = other.index_;
 }
 
+std::string Subprog::name() const
+{
+  return name_;
+}
+
 bool Probe::has_ap_of_probetype(ProbeType probe_type)
 {
   if (!attach_points)
     return false;
-  for (auto ap : *attach_points)
-  {
+  for (auto ap : *attach_points) {
     if (probetype(ap->provider) == probe_type)
       return true;
   }
@@ -709,6 +801,10 @@ While::While(const While &other) : Statement(other)
 {
 }
 
+For::For(const For &other) : Statement(other)
+{
+}
+
 Tuple::Tuple(const Tuple &other) : Expression(other)
 {
 }
