@@ -1,10 +1,13 @@
 #include "elf_parser.h"
 #include <elf.h>
+#ifndef EM_BPF     // EM_BPF may not be defined in older containers, e.g. RHEL7
+#define EM_BPF 247 // Could be in linux/elf.h, but it can cause conflicts
+#endif
 
 namespace bpftrace {
 namespace elf {
 
-BpfBytecode parseBpfBytecodeFromElfObject(void *const elf)
+BpfBytecode parseBpfBytecodeFromElfObject(void *const elf, size_t elf_size)
 {
   char *fileptr = (char *)elf;
   Elf64_Ehdr *ehdr = (Elf64_Ehdr *)elf;
@@ -28,18 +31,16 @@ BpfBytecode parseBpfBytecodeFromElfObject(void *const elf)
   assert(strtable_shdr->sh_type == SHT_STRTAB);
   char *strtable = fileptr + strtable_shdr->sh_offset;
 
-  BpfBytecode result;
+  BpfBytecode result(elf, elf_size);
 
-  for (int i = 0; i < ehdr->e_shnum; ++i)
-  {
+  for (int i = 0; i < ehdr->e_shnum; ++i) {
     Elf64_Shdr *shdr = &shdrs[i];
 
     char *name = strtable + shdr->sh_name;
     std::vector<uint8_t> data;
     data.resize(shdr->sh_size);
 
-    if (shdr->sh_size && shdr->sh_type != SHT_NOBITS)
-    {
+    if (shdr->sh_size && shdr->sh_type != SHT_NOBITS) {
       // NOBITS sections occupy no size on disk but take up size in
       // memory. Copy the file data for all other sections.
 
@@ -52,4 +53,4 @@ BpfBytecode parseBpfBytecodeFromElfObject(void *const elf)
 }
 
 } // namespace elf
-} // namespace bpftrace
\ No newline at end of file
+} // namespace bpftrace
