@@ -30,14 +30,6 @@
 #error Unsupported LLVM version
 #endif
 
-#if LLVM_VERSION_MAJOR >= 10
-#define CREATE_MEMSET(ptr, val, size, align)                                   \
-  CreateMemSet((ptr), (val), (size), MaybeAlign((align)))
-#else
-#define CREATE_MEMSET(ptr, val, size, align)                                   \
-  CreateMemSet((ptr), (val), (size), (align))
-#endif
-
 #if LLVM_VERSION_MAJOR >= 13
 #define CREATE_ATOMIC_RMW(op, ptr, val, align, order)                          \
   CreateAtomicRMW((op), (ptr), (val), MaybeAlign((align)), (order))
@@ -46,29 +38,40 @@
   CreateAtomicRMW((op), (ptr), (val), (order))
 #endif
 
+#if LLVM_VERSION_MAJOR >= 15
+#define GET_PTR_TY() getPtrTy()
+#else
+#define GET_PTR_TY() getInt8PtrTy()
+#endif
+
 namespace bpftrace {
 namespace ast {
 
 using namespace llvm;
 
-class IRBuilderBPF : public IRBuilder<>
-{
+class IRBuilderBPF : public IRBuilder<> {
 public:
   IRBuilderBPF(LLVMContext &context, Module &module, BPFtrace &bpftrace);
 
-  AllocaInst *CreateAllocaBPF(llvm::Type *ty, const std::string &name="");
-  AllocaInst *CreateAllocaBPF(const SizedType &stype, const std::string &name="");
-  AllocaInst *CreateAllocaBPFInit(const SizedType &stype, const std::string &name);
-  AllocaInst *CreateAllocaBPF(llvm::Type *ty, llvm::Value *arraysize, const std::string &name="");
-  AllocaInst *CreateAllocaBPF(const SizedType &stype, llvm::Value *arraysize, const std::string &name="");
-  AllocaInst *CreateAllocaBPF(int bytes, const std::string &name="");
+  AllocaInst *CreateAllocaBPF(llvm::Type *ty, const std::string &name = "");
+  AllocaInst *CreateAllocaBPF(const SizedType &stype,
+                              const std::string &name = "");
+  AllocaInst *CreateAllocaBPFInit(const SizedType &stype,
+                                  const std::string &name);
+  AllocaInst *CreateAllocaBPF(llvm::Type *ty,
+                              llvm::Value *arraysize,
+                              const std::string &name = "");
+  AllocaInst *CreateAllocaBPF(const SizedType &stype,
+                              llvm::Value *arraysize,
+                              const std::string &name = "");
+  AllocaInst *CreateAllocaBPF(int bytes, const std::string &name = "");
+  void CreateMemsetBPF(Value *ptr, Value *val, uint32_t size);
   llvm::Type *GetType(const SizedType &stype);
+  llvm::Type *GetMapValueType(const SizedType &stype);
   llvm::ConstantInt *GetIntSameSize(uint64_t C, llvm::Value *expr);
   llvm::ConstantInt *GetIntSameSize(uint64_t C, llvm::Type *ty);
-  CallInst *CreateBpfPseudoCallId(int mapid);
-  CallInst *CreateBpfPseudoCallId(Map &map);
-  CallInst *CreateBpfPseudoCallValue(int mapid);
-  CallInst *CreateBpfPseudoCallValue(Map &map);
+  Value *GetMapVar(const std::string &map_name);
+  Value *GetNull();
   CallInst *CreateMapLookup(Map &map,
                             Value *key,
                             const std::string &name = "lookup_elem");
@@ -77,12 +80,18 @@ public:
                              Value *key,
                              const location &loc);
   Value *CreateMapLookupElem(Value *ctx,
-                             int mapid,
+                             const std::string &map_name,
                              Value *key,
                              SizedType &type,
                              const location &loc);
+  Value *CreatePerCpuMapAggElems(Value *ctx,
+                                 Map &map,
+                                 Value *key,
+                                 const SizedType &type,
+                                 const location &loc,
+                                 bool is_aot);
   void CreateMapUpdateElem(Value *ctx,
-                           Map &map,
+                           const std::string &map_ident,
                            Value *key,
                            Value *val,
                            const location &loc,
@@ -142,12 +151,7 @@ public:
                                 pid_t pid,
                                 AddrSpace as,
                                 const location &loc);
-  Value *CreateStrncmp(Value *str1,
-                       uint64_t str1_size,
-                       Value *str2,
-                       uint64_t str2_size,
-                       uint64_t n,
-                       bool inverse);
+  Value *CreateStrncmp(Value *str1, Value *str2, uint64_t n, bool inverse);
   Value *CreateStrcontains(Value *val1,
                            uint64_t str1_size,
                            Value *val2,
@@ -177,9 +181,21 @@ public:
   CallInst *CreateGetCpuId(const location &loc);
   CallInst *CreateGetCurrentTask(const location &loc);
   CallInst *CreateGetRandom(const location &loc);
-  CallInst   *CreateGetStackId(Value *ctx, bool ustack, StackType stack_type, const location& loc);
-  CallInst *CreateGetFuncIp(const location &loc);
+  CallInst *CreateGetStack(Value *ctx,
+                           bool ustack,
+                           Value *buf,
+                           StackType stack_type,
+                           const location &loc);
+  CallInst *CreateGetFuncIp(Value *ctx, const location &loc);
   CallInst *CreateGetJoinMap(BasicBlock *failure_callback, const location &loc);
+  CallInst *CreateGetStackScratchMap(StackType stack_type,
+                                     BasicBlock *failure_callback,
+                                     const location &loc);
+  CallInst *CreateGetStrScratchMap(int idx,
+                                   BasicBlock *failure_callback,
+                                   const location &loc);
+  void CreateCheckSetRecursion(const location &loc, int early_exit_ret);
+  void CreateUnSetRecursion(const location &loc);
   CallInst *CreateHelperCall(libbpf::bpf_func_id func_id,
                              FunctionType *helper_type,
                              ArrayRef<Value *> args,
@@ -189,12 +205,15 @@ public:
                        Value *callee,
                        ArrayRef<Value *> args,
                        const Twine &Name);
-  void        CreateGetCurrentComm(Value *ctx, AllocaInst *buf, size_t size, const location& loc);
+  void CreateGetCurrentComm(Value *ctx,
+                            AllocaInst *buf,
+                            size_t size,
+                            const location &loc);
   void CreateOutput(Value *ctx,
                     Value *data,
                     size_t size,
                     const location *loc = nullptr);
-  void CreateAtomicIncCounter(int mapfd, uint32_t idx);
+  void CreateAtomicIncCounter(const std::string &map_name, uint32_t idx);
   void CreateMapElemInit(Value *ctx,
                          Map &map,
                          Value *key,
@@ -212,15 +231,24 @@ public:
                          Value *fmt_size,
                          const std::vector<Value *> &values,
                          const location &loc);
-  void        CreateSignal(Value *ctx, Value *sig, const location &loc);
-  void        CreateOverrideReturn(Value *ctx, Value *rc);
-  void        CreateHelperError(Value *ctx, Value *return_value, libbpf::bpf_func_id func_id, const location& loc);
-  void        CreateHelperErrorCond(Value *ctx, Value *return_value, libbpf::bpf_func_id func_id, const location& loc, bool compare_zero=false);
-  StructType *GetStructType(std::string name, const std::vector<llvm::Type *> & elements, bool packed = false);
+  void CreateSignal(Value *ctx, Value *sig, const location &loc);
+  void CreateOverrideReturn(Value *ctx, Value *rc);
+  void CreateHelperError(Value *ctx,
+                         Value *return_value,
+                         libbpf::bpf_func_id func_id,
+                         const location &loc);
+  void CreateHelperErrorCond(Value *ctx,
+                             Value *return_value,
+                             libbpf::bpf_func_id func_id,
+                             const location &loc,
+                             bool compare_zero = false);
+  StructType *GetStructType(std::string name,
+                            const std::vector<llvm::Type *> &elements,
+                            bool packed = false);
   AllocaInst *CreateUSym(llvm::Value *val, int probe_id, const location &loc);
   Value *CreateRegisterRead(Value *ctx, const std::string &builtin);
   Value *CreateRegisterRead(Value *ctx, int offset, const std::string &name);
-  Value      *CreatKFuncArg(Value *ctx, SizedType& type, std::string& name);
+  Value *CreatKFuncArg(Value *ctx, SizedType &type, std::string &name);
   Value *CreateRawTracepointArg(Value *ctx, const std::string &builtin);
   Value *CreateUprobeArgsRecord(Value *ctx, const SizedType &args_type);
   llvm::Type *UprobeArgsType(const SizedType &args_type);
@@ -228,10 +256,7 @@ public:
                             Value *len,
                             AllocaInst *data,
                             size_t size);
-  void CreatePath(Value *ctx,
-                  AllocaInst *buf,
-                  Value *path,
-                  const location &loc);
+  void CreatePath(Value *ctx, Value *buf, Value *path, const location &loc);
   void CreateSeqPrintf(Value *ctx,
                        Value *fmt,
                        Value *fmt_size,
@@ -269,14 +294,25 @@ private:
                                 Builtin &builtin,
                                 AddrSpace as,
                                 const location &loc);
-  CallInst *createMapLookup(int mapid,
+  CallInst *createMapLookup(const std::string &map_name,
                             Value *key,
                             const std::string &name = "lookup_elem");
-  CallInst *createMapLookup(int mapid,
+  CallInst *createMapLookup(const std::string &map_name,
                             Value *key,
                             PointerType *val_ptr_ty,
                             const std::string &name = "lookup_elem");
-  CallInst *createGetScratchMap(int mapid,
+  CallInst *createPerCpuMapLookup(
+      const std::string &map_name,
+      Value *key,
+      Value *cpu,
+      const std::string &name = "lookup_percpu_elem");
+  CallInst *createPerCpuMapLookup(
+      const std::string &map_name,
+      Value *key,
+      Value *cpu,
+      PointerType *val_ptr_ty,
+      const std::string &name = "lookup_percpu_elem");
+  CallInst *createGetScratchMap(const std::string &map_name,
                                 const std::string &name,
                                 PointerType *val_ptr_ty,
                                 const location &loc,
@@ -294,6 +330,12 @@ private:
                              size_t size,
                              const location *loc = nullptr);
 
+  void createPerCpuSum(AllocaInst *ret, CallInst *call, const SizedType &type);
+  void createPerCpuMinMax(AllocaInst *ret,
+                          AllocaInst *is_ret_set,
+                          CallInst *call,
+                          const SizedType &type);
+
   std::map<std::string, StructType *> structs_;
 };
 
