@@ -13,7 +13,7 @@ void print(Node *root, const std::string &name, std::ostream &out)
 {
   out << "\nAST after: " << name << std::endl;
   out << "-------------------\n";
-  ast::Printer printer(out, true);
+  ast::Printer printer(out);
   printer.print(root);
   out << std::endl;
 }
@@ -29,14 +29,12 @@ PassResult PassManager::Run(std::unique_ptr<Node> node, PassContext &ctx)
   Node *root = node.release();
   if (bt_debug != DebugLevel::kNone)
     print(root, "parser", std::cout);
-  for (auto &pass : passes_)
-  {
+  for (auto &pass : passes_) {
     auto result = pass.Run(*root, ctx);
     if (!result.Ok())
       return result;
 
-    if (result.Root())
-    {
+    if (result.Root()) {
       delete root;
       root = result.Root();
     }
