@@ -17,7 +17,7 @@
 #include "bpftrace.h"
 #include "format_string.h"
 #include "location.hh"
-#include "map.h"
+#include "required_resources.h"
 
 namespace bpftrace {
 namespace ast {
@@ -26,17 +26,20 @@ using namespace llvm;
 
 using CallArgs = std::vector<std::tuple<FormatString, std::vector<Field>>>;
 
-class CodegenLLVM : public Visitor
-{
+class CodegenLLVM : public Visitor {
 public:
-  explicit CodegenLLVM(Node *root, BPFtrace &bpftrace);
+  explicit CodegenLLVM(Node *root, BPFtrace &bpftrace, bool is_aot = false);
+  explicit CodegenLLVM(Node *root,
+                       BPFtrace &bpftrace,
+                       bool is_aot,
+                       std::unique_ptr<USDTHelper> usdt_helper);
 
   void visit(Integer &integer) override;
   void visit(PositionalParameter &param) override;
   void visit(String &string) override;
   void visit(Identifier &identifier) override;
   void visit(Builtin &builtin) override;
-  void visit(StackMode &) override { };
+  void visit(StackMode &) override{};
   void visit(Call &call) override;
   void visit(Sizeof &szof) override;
   void visit(Offsetof &ofof) override;
@@ -55,27 +58,46 @@ public:
   void visit(If &if_block) override;
   void visit(Unroll &unroll) override;
   void visit(While &while_block) override;
+  void visit(For &f) override;
   void visit(Jump &jump) override;
   void visit(Predicate &pred) override;
   void visit(AttachPoint &ap) override;
   void visit(Probe &probe) override;
+  void visit(Subprog &subprog) override;
   void visit(Program &program) override;
+
   AllocaInst *getHistMapKey(Map &map, Value *log2);
   int getNextIndexForProbe();
-  Value      *createLogicalAnd(Binop &binop);
-  Value      *createLogicalOr(Binop &binop);
+  Value *createLogicalAnd(Binop &binop);
+  Value *createLogicalOr(Binop &binop);
 
   // Exists to make calling from a debugger easier
   void DumpIR(void);
   void DumpIR(std::ostream &out);
   void DumpIR(const std::string filename);
-  void createFormatStringCall(Call &call, int &id, CallArgs &call_args,
-                              const std::string &call_name, AsyncAction async_action);
+  void createFormatStringCall(Call &call,
+                              int &id,
+                              CallArgs &call_args,
+                              const std::string &call_name,
+                              AsyncAction async_action);
 
   void createPrintMapCall(Call &call);
   void createPrintNonMapCall(Call &call, int &id);
 
+  void createMapDefinition(const std::string &name,
+                           libbpf::bpf_map_type map_type,
+                           uint64_t max_entries,
+                           const MapKey &key,
+                           const SizedType &value_type);
+  AllocaInst *createTuple(
+      const SizedType &tuple_type,
+      const std::vector<std::pair<llvm::Value *, const location *>> &vals,
+      const std::string &name);
+
   void generate_ir(void);
+  libbpf::bpf_map_type get_map_type(const SizedType &val_type,
+                                    const MapKey &key);
+  void generate_maps(const RequiredResources &resources);
   void optimize(void);
   bool verify(void);
   BpfBytecode emit(void);
@@ -86,8 +108,7 @@ public:
 
 private:
   static constexpr char LLVMTargetTriple[] = "bpf-pc-linux";
-  class ScopedExprDeleter
-  {
+  class ScopedExprDeleter {
   public:
     explicit ScopedExprDeleter(std::function<void()> deleter)
     {
@@ -132,12 +153,17 @@ private:
   // invalid probes that still need to be visited.
   void generateProbe(Probe &probe,
                      const std::string &full_func_id,
-                     const std::string &section_name,
+                     const std::string &name,
                      FunctionType *func_type,
-                     bool expansion,
                      std::optional<int> usdt_location_index = std::nullopt,
                      bool dummy = false);
 
+  // Generate a probe and register it to the BPFtrace class.
+  void add_probe(AttachPoint &ap,
+                 Probe &probe,
+                 const std::string &name,
+                 FunctionType *func_type);
+
   [[nodiscard]] ScopedExprDeleter accept(Node *node);
   [[nodiscard]] std::tuple<Value *, ScopedExprDeleter> getMapKey(Map &map);
   AllocaInst *getMultiMapKey(Map &map, const std::vector<Value *> &extra_keys);
@@ -167,8 +193,9 @@ private:
 
   // Create return instruction
   //
-  // If null, return value will depend on current attach point
+  // If null, return value will depend on current attach point (void in subprog)
   void createRet(Value *value = nullptr);
+  int getReturnValueForProbe(ProbeType probe_type);
 
   // Every time we see a watchpoint that specifies a function + arg pair, we
   // generate a special "setup" probe that:
@@ -206,6 +233,10 @@ private:
   void createIncDec(Unop &unop);
 
   Function *createMapLenCallback();
+  Function *createForEachMapCallback(Map &map,
+                                     const Variable &decl,
+                                     const std::vector<Statement *> &stmts);
+  Function *createMurmurHash2Func();
 
   // Return a lambda that has captured-by-value CodegenLLVM's async id state
   // (ie `printf_id_`, `mapped_printf_id_`, etc.).  Running the returned lambda
@@ -213,15 +244,19 @@ private:
   // first called.
   std::function<void()> create_reset_ids();
 
+  bool canAggPerCpuMapElems(const SizedType &val_type, const MapKey &key);
+
   Node *root_ = nullptr;
 
   BPFtrace &bpftrace_;
+  std::unique_ptr<USDTHelper> usdt_helper_;
   std::unique_ptr<LLVMContext> context_;
   std::unique_ptr<TargetMachine> target_machine_;
   std::unique_ptr<Module> module_;
   IRBuilderBPF b_;
 
   DIBuilderBPF debug_;
+  bool is_aot_;
 
   const DataLayout &datalayout() const
   {
@@ -240,8 +275,12 @@ private:
   int next_probe_index_ = 1;
   // Used if there are duplicate USDT entries
   int current_usdt_location_index_{ 0 };
+  bool inside_subprog_ = false;
+  bool need_recursion_check_ = false;
 
   std::map<std::string, AllocaInst *> variables_;
+
+  // NB: ensure all IDs are saved/restored in create_reset_ids()
   int printf_id_ = 0;
   int mapped_printf_id_ = 0;
   int time_id_ = 0;
@@ -253,9 +292,14 @@ private:
   uint64_t watchpoint_id_ = 0;
   int cgroup_path_id_ = 0;
   int skb_output_id_ = 0;
+  int str_id_ = 0;
+
+  std::unordered_map<std::string, libbpf::bpf_map_type> map_types_;
 
   Function *linear_func_ = nullptr;
   Function *log2_func_ = nullptr;
+  Function *murmur_hash_2_func_ = nullptr;
+  Function *map_len_func_ = nullptr;
   MDNode *loop_metadata_ = nullptr;
 
   size_t getStructSize(StructType *s)
@@ -266,8 +310,7 @@ private:
   std::vector<std::tuple<BasicBlock *, BasicBlock *>> loops_;
   std::unordered_map<std::string, bool> probe_names_;
 
-  enum class State
-  {
+  enum class State {
     INIT,
     IR,
     OPT,
