@@ -3,8 +3,8 @@
 #include <iostream>
 #include <sstream>
 
-#include "pass_manager.h"
-#include "visitors.h"
+#include "ast/pass_manager.h"
+#include "ast/visitors.h"
 
 namespace bpftrace {
 namespace ast {
@@ -14,8 +14,7 @@ namespace ast {
 //
 // Over time, we expect to relax these restrictions as AOT supports more
 // features.
-class PortabilityAnalyser : public Visitor
-{
+class PortabilityAnalyser : public Visitor {
 public:
   PortabilityAnalyser(Node *root, std::ostream &out = std::cerr);
   int analyse();
