@@ -12,20 +12,22 @@ namespace ast {
 
 void Printer::print(Node *root)
 {
-  depth_ = 0;
+  ++depth_;
   Visit(*root);
+  --depth_;
 }
 
 std::string Printer::type(const SizedType &ty)
 {
+  if (ty.IsNoneTy())
+    return "";
   std::stringstream buf;
-  if (print_types)
-  {
-    buf << " :: type[" << ty << ", ctx: " << ty.IsCtxAccess();
-    if (ty.GetAS() != AddrSpace::none)
-      buf << ", AS(" << ty.GetAS() << ")";
-    buf << "]";
-  }
+  buf << " :: [" << ty;
+  if (ty.IsCtxAccess())
+    buf << ", ctx: 1";
+  if (ty.GetAS() != AddrSpace::none)
+    buf << ", AS(" << ty.GetAS() << ")";
+  buf << "]";
   return buf.str();
 }
 
@@ -56,21 +58,17 @@ void Printer::visit(String &string)
   std::string indent(depth_, ' ');
   std::stringstream ss;
 
-  for (char c: string.str)
-  {
+  for (char c : string.str) {
     // the argument of isprint() must be an unsigned char or EOF
     int code = static_cast<unsigned char>(c);
-    if (std::isprint(code))
-    {
+    if (std::isprint(code)) {
       if (c == '\\')
         ss << "\\\\";
       else if (c == '"')
         ss << "\\\"";
       else
         ss << c;
-    }
-    else
-    {
+    } else {
       if (c == '\n')
         ss << "\\n";
       else if (c == '\t')
@@ -78,8 +76,7 @@ void Printer::visit(String &string)
       else if (c == '\r')
         ss << "\\r";
       else
-        ss << "\\x" << std::setfill('0') << std::setw(2)
-           << std::hex << code;
+        ss << "\\x" << std::setfill('0') << std::setw(2) << std::hex << code;
     }
   }
 
@@ -137,8 +134,16 @@ void Printer::visit(Offsetof &ofof)
   out_ << indent << "offsetof: " << type(ofof.type) << std::endl;
 
   ++depth_;
-  if (ofof.expr)
+  std::string indentParam(depth_, ' ');
+
+  // Print the args
+  if (ofof.expr) {
     ofof.expr->accept(*this);
+  } else {
+    out_ << indentParam << ofof.record << std::endl;
+  }
+
+  out_ << indentParam << ofof.field << std::endl;
   --depth_;
 }
 
@@ -175,15 +180,12 @@ void Printer::visit(Binop &binop)
 
 void Printer::visit(Unop &unop)
 {
-  if (unop.is_post_op)
-  {
+  if (unop.is_post_op) {
     std::string indent(depth_ + 1, ' ');
 
     unop.expr->accept(*this);
     out_ << indent << opstr(unop) << std::endl;
-  }
-  else
-  {
+  } else {
     std::string indent(depth_, ' ');
     out_ << indent << opstr(unop) << std::endl;
 
@@ -191,7 +193,6 @@ void Printer::visit(Unop &unop)
     unop.expr->accept(*this);
     --depth_;
   }
-
 }
 
 void Printer::visit(Ternary &ternary)
@@ -345,12 +346,29 @@ void Printer::visit(While &while_block)
   ++depth_;
   out_ << indent << " )" << std::endl;
 
-  for (Statement *stmt : *while_block.stmts)
-  {
+  for (Statement *stmt : *while_block.stmts) {
     stmt->accept(*this);
   }
 }
 
+void Printer::visit(For &for_loop)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "for" << std::endl;
+
+  ++depth_;
+  out_ << indent << " decl\n";
+  print(for_loop.decl);
+  out_ << indent << " expr\n";
+  print(for_loop.expr);
+
+  out_ << indent << " stmts\n";
+  for (Statement *stmt : *for_loop.stmts) {
+    print(stmt);
+  }
+  --depth_;
+}
+
 void Printer::visit(Config &config)
 {
   std::string indent(depth_, ' ');
@@ -358,8 +376,7 @@ void Printer::visit(Config &config)
   out_ << indent << "config" << std::endl;
 
   ++depth_;
-  for (Statement *stmt : *config.stmts)
-  {
+  for (Statement *stmt : *config.stmts) {
     stmt->accept(*this);
   }
   --depth_;
@@ -369,6 +386,11 @@ void Printer::visit(Jump &jump)
 {
   std::string indent(depth_, ' ');
   out_ << indent << opstr(jump) << std::endl;
+  if (jump.return_value) {
+    ++depth_;
+    jump.return_value->accept(*this);
+    --depth_;
+  }
 }
 
 void Printer::visit(Predicate &pred)
@@ -384,7 +406,7 @@ void Printer::visit(Predicate &pred)
 void Printer::visit(AttachPoint &ap)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << ap.name(ap.func) << std::endl;
+  out_ << indent << ap.name() << std::endl;
 }
 
 void Printer::visit(Probe &probe)
@@ -403,6 +425,27 @@ void Printer::visit(Probe &probe)
   --depth_;
 }
 
+void Printer::visit(Subprog &subprog)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << subprog.name() << ": " << subprog.return_type;
+
+  out_ << "(";
+  for (size_t i = 0; i < subprog.args->size(); i++) {
+    auto &arg = subprog.args->at(i);
+    out_ << arg->name() << " : " << arg->type;
+    if (i < subprog.args->size() - 1)
+      out_ << ", ";
+  }
+  out_ << ")" << std::endl;
+
+  ++depth_;
+  for (Statement *stmt : *subprog.stmts) {
+    stmt->accept(*this);
+  }
+  --depth_;
+}
+
 void Printer::visit(Program &program)
 {
   if (program.c_definitions.size() > 0)
@@ -411,14 +454,15 @@ void Printer::visit(Program &program)
   std::string indent(depth_, ' ');
   out_ << indent << "Program" << std::endl;
 
-  if (program.config)
-  {
+  if (program.config) {
     ++depth_;
     program.config->accept(*this);
     --depth_;
   }
 
   ++depth_;
+  for (Subprog *subprog : *program.functions)
+    subprog->accept(*this);
   for (Probe *probe : *program.probes)
     probe->accept(*this);
   --depth_;
