@@ -19,8 +19,7 @@ namespace ast {
 // TODO(danobi): Note that while complete resource collection in this pass is
 // the goal, there are still places where the goal is not yet realized. For
 // example the helper error metadata is still being collected during codegen.
-class ResourceAnalyser : public Visitor
-{
+class ResourceAnalyser : public Visitor {
 public:
   ResourceAnalyser(Node *root, std::ostream &out = std::cerr);
 
@@ -28,6 +27,7 @@ public:
 
 private:
   void visit(Probe &probe) override;
+  void visit(Subprog &subprog) override;
   void visit(Builtin &map) override;
   void visit(Call &call) override;
   void visit(Map &map) override;
@@ -47,6 +47,8 @@ private:
   std::ostringstream err_;
   // Current probe we're analysing
   Probe *probe_;
+
+  int next_map_id_ = 0;
 };
 
 Pass CreateResourcePass();
