@@ -31,10 +31,8 @@ void Visitor::visit(Identifier &identifier __attribute__((__unused__)))
 
 void Visitor::visit(Call &call)
 {
-  if (call.vargs)
-  {
-    for (Expression *expr : *call.vargs)
-    {
+  if (call.vargs) {
+    for (Expression *expr : *call.vargs) {
       Visit(*expr);
     }
   }
@@ -54,10 +52,8 @@ void Visitor::visit(Offsetof &ofof)
 
 void Visitor::visit(Map &map)
 {
-  if (map.vargs)
-  {
-    for (Expression *expr : *map.vargs)
-    {
+  if (map.vargs) {
+    for (Expression *expr : *map.vargs) {
       Visit(*expr);
     }
   }
@@ -133,15 +129,12 @@ void Visitor::visit(If &if_block)
 {
   Visit(*if_block.cond);
 
-  for (Statement *stmt : *if_block.stmts)
-  {
+  for (Statement *stmt : *if_block.stmts) {
     Visit(*stmt);
   }
 
-  if (if_block.else_stmts)
-  {
-    for (Statement *stmt : *if_block.else_stmts)
-    {
+  if (if_block.else_stmts) {
+    for (Statement *stmt : *if_block.else_stmts) {
       Visit(*stmt);
     }
   }
@@ -150,8 +143,7 @@ void Visitor::visit(If &if_block)
 void Visitor::visit(Unroll &unroll)
 {
   Visit(*unroll.expr);
-  for (Statement *stmt : *unroll.stmts)
-  {
+  for (Statement *stmt : *unroll.stmts) {
     Visit(*stmt);
   }
 }
@@ -160,14 +152,25 @@ void Visitor::visit(While &while_block)
 {
   Visit(*while_block.cond);
 
-  for (Statement *stmt : *while_block.stmts)
-  {
+  for (Statement *stmt : *while_block.stmts) {
     Visit(*stmt);
   }
 }
 
-void Visitor::visit(Jump &jump __attribute__((__unused__)))
+void Visitor::visit(For &for_loop)
 {
+  Visit(*for_loop.decl);
+  Visit(*for_loop.expr);
+
+  for (Statement *stmt : *for_loop.stmts) {
+    Visit(*stmt);
+  }
+}
+
+void Visitor::visit(Jump &jump)
+{
+  if (jump.return_value)
+    Visit(*jump.return_value);
 }
 
 void Visitor::visit(Predicate &pred)
@@ -181,31 +184,44 @@ void Visitor::visit(AttachPoint &ap __attribute__((__unused__)))
 
 void Visitor::visit(Probe &probe)
 {
-  for (AttachPoint *ap : *probe.attach_points)
-  {
+  for (AttachPoint *ap : *probe.attach_points) {
     Visit(*ap);
   }
 
-  if (probe.pred)
-  {
+  if (probe.pred) {
     Visit(*probe.pred);
   }
-  for (Statement *stmt : *probe.stmts)
-  {
+  for (Statement *stmt : *probe.stmts) {
     Visit(*stmt);
   }
 }
 
 void Visitor::visit(Config &config)
 {
-  for (Statement *stmt : *config.stmts)
-  {
+  for (Statement *stmt : *config.stmts) {
+    Visit(*stmt);
+  }
+}
+
+void Visitor::visit(SubprogArg &subprog_arg __attribute__((__unused__)))
+{
+}
+
+void Visitor::visit(Subprog &subprog)
+{
+  for (SubprogArg *arg : *subprog.args) {
+    Visit(*arg);
+  }
+  for (Statement *stmt : *subprog.stmts) {
     Visit(*stmt);
   }
 }
 
 void Visitor::visit(Program &program)
 {
+  if (program.functions)
+    for (Subprog *subprog : *program.functions)
+      Visit(*subprog);
   for (Probe *probe : *program.probes)
     Visit(*probe);
   if (program.config)
@@ -279,8 +295,7 @@ Node *Mutator::visit(Offsetof &ofof)
 Node *Mutator::visit(Map &map)
 {
   auto m = map.leafcopy();
-  if (map.vargs)
-  {
+  if (map.vargs) {
     m->vargs = mutateExprList(map.vargs);
 
     for (auto expr : *m->vargs)
@@ -405,6 +420,16 @@ Node *Mutator::visit(While &while_block)
   return w;
 }
 
+Node *Mutator::visit(For &for_loop)
+{
+  auto f = for_loop.leafcopy();
+  f->decl = Value<Variable>(for_loop.decl);
+  f->expr = Value<Expression>(for_loop.expr);
+
+  f->stmts = mutateStmtList(for_loop.stmts);
+  return f;
+}
+
 Node *Mutator::visit(Probe &probe)
 {
   auto p = probe.leafcopy();
@@ -426,9 +451,21 @@ Node *Mutator::visit(Config &config)
   return c;
 }
 
+Node *Mutator::visit(Subprog &subprog)
+{
+  auto s = subprog.leafcopy();
+
+  s->stmts = mutateStmtList(subprog.stmts);
+  return s;
+}
+
 Node *Mutator::visit(Program &program)
 {
   auto p = program.leafcopy();
+  p->functions = new SubprogList;
+  if (program.functions)
+    for (Subprog *subprog : *program.functions)
+      p->functions->push_back(Value<Subprog>(subprog));
   p->probes = new ProbeList;
   for (Probe *probe : *program.probes)
     p->probes->push_back(Value<Probe>(probe));
