@@ -1,6 +1,5 @@
 #include "bpffeature.h"
 
-#include <bcc/bcc_syms.h>
 #include <bcc/libbpf.h>
 #include <bpf/bpf.h>
 #include <bpf/btf.h>
@@ -24,18 +23,12 @@ namespace bpftrace {
 
 int BPFnofeature::parse(const char* str)
 {
-  for (auto feat : split_string(str, ','))
-  {
-    if (feat == "kprobe_multi")
-    {
+  for (auto feat : split_string(str, ',')) {
+    if (feat == "kprobe_multi") {
       kprobe_multi_ = true;
-    }
-    else if (feat == "uprobe_multi")
-    {
+    } else if (feat == "uprobe_multi") {
       uprobe_multi_ = true;
-    }
-    else
-    {
+    } else {
       return -1;
     }
   }
@@ -53,11 +46,12 @@ static bool try_load_(const char* name,
                       size_t logbuf_size,
                       int* outfd = nullptr)
 {
-  for (int attempt = 0; attempt < 3; attempt++)
-  {
-    auto version = kernel_version(attempt);
-    if (version == 0 && attempt > 0)
-    {
+  const KernelVersionMethod methods[] = { vDSO, UTS, File };
+
+  for (KernelVersionMethod method : methods) {
+    auto version = kernel_version(method);
+
+    if (method != vDSO && !version) {
       // Recent kernels don't check the version so we should try to call
       // bpf_prog_load during first iteration even if we failed to determine
       // the version. We should not do that in subsequent iterations to avoid
@@ -70,8 +64,7 @@ static bool try_load_(const char* name,
     opts.log_size = logbuf_size;
     opts.log_level = loglevel;
     opts.kern_version = version;
-    if (attach_type.has_value())
-    {
+    if (attach_type.has_value()) {
       opts.expected_attach_type = static_cast<::bpf_attach_type>(
           attach_type.value());
     }
@@ -84,8 +77,7 @@ static bool try_load_(const char* name,
                             insns,
                             insns_cnt,
                             &opts);
-    if (ret >= 0)
-    {
+    if (ret >= 0) {
       if (outfd)
         *outfd = ret;
       else
@@ -109,16 +101,14 @@ bool BPFfeature::try_load(enum libbpf::bpf_prog_type prog_type,
   char logbuf[log_size] = {};
 
   std::optional<unsigned> btf_id;
-  if (prog_type == libbpf::BPF_PROG_TYPE_TRACING && has_btf())
-  {
+  if (prog_type == libbpf::BPF_PROG_TYPE_TRACING && has_btf()) {
     auto id_fd = btf_.get_btf_id_fd(name, "vmlinux");
     btf_id = id_fd.first;
     if (id_fd.second >= 0)
       close(id_fd.second);
   }
 
-  if (prog_type == libbpf::BPF_PROG_TYPE_TRACING)
-  {
+  if (prog_type == libbpf::BPF_PROG_TYPE_TRACING) {
     // List of available functions must be readable
     std::ifstream traceable_funcs(tracefs::available_filter_functions());
     if (!traceable_funcs.good())
@@ -148,8 +138,7 @@ bool BPFfeature::try_load_btf(const void* btf_data, size_t btf_size)
               .log_size = log_size, );
 
   int fd = bpf_btf_load(btf_data, btf_size, &btf_opts);
-  if (fd >= 0)
-  {
+  if (fd >= 0) {
     close(fd);
     return true;
   }
@@ -192,7 +181,8 @@ bool BPFfeature::detect_helper(enum libbpf::bpf_func_id func_id,
     return false;
 
   return (strstr(buf, "invalid func ") == nullptr) &&
-         (strstr(buf, "unknown func ") == nullptr);
+         (strstr(buf, "unknown func ") == nullptr) &&
+         (strstr(buf, "program of this type cannot use helper ") == nullptr);
 }
 
 bool BPFfeature::detect_prog_type(
@@ -214,8 +204,7 @@ bool BPFfeature::detect_map(enum libbpf::bpf_map_type map_type)
   int flags = 0;
   int map_fd = 0;
 
-  switch (map_type)
-  {
+  switch (map_type) {
     case libbpf::BPF_MAP_TYPE_STACK_TRACE:
       value_size = 8;
       break;
@@ -319,16 +308,14 @@ int BPFfeature::instruction_limit(void)
   // processed 2 insns (limit 131072), stack depth 0
   std::string log(logbuf, logsize);
   std::size_t line_start = log.find("processed 2 insns");
-  if (line_start == std::string::npos)
-  {
+  if (line_start == std::string::npos) {
     insns_limit_ = std::make_optional<int>(-1);
     return *insns_limit_;
   }
 
   // Old kernels don't have the instruction limit in the verifier output
   auto begin = log.find("limit", line_start);
-  if (begin == std::string::npos)
-  {
+  if (begin == std::string::npos) {
     insns_limit_ = std::make_optional<int>(-1);
     return *insns_limit_;
   }
@@ -406,10 +393,11 @@ bool BPFfeature::has_uprobe_refcnt()
   if (has_uprobe_refcnt_.has_value())
     return *has_uprobe_refcnt_;
 
-  struct stat sb;
-  has_uprobe_refcnt_ =
-      ::stat("/sys/bus/event_source/devices/uprobe/format/ref_ctr_offset",
-             &sb) == 0;
+  std::error_code ec;
+  std_filesystem::path path{
+    "/sys/bus/event_source/devices/uprobe/format/ref_ctr_offset"
+  };
+  has_uprobe_refcnt_ = std_filesystem::exists(path, ec);
 
   return *has_uprobe_refcnt_;
 }
@@ -419,8 +407,7 @@ bool BPFfeature::has_kprobe_multi()
   if (has_kprobe_multi_.has_value())
     return *has_kprobe_multi_;
 
-  if (no_feature_.kprobe_multi_)
-  {
+  if (no_feature_.kprobe_multi_) {
     has_kprobe_multi_ = false;
     return *has_kprobe_multi_;
   }
@@ -449,8 +436,7 @@ bool BPFfeature::has_kprobe_multi()
                          ARRAY_SIZE(insns),
                          &load_opts);
 
-  if (progfd >= 0)
-  {
+  if (progfd >= 0) {
     linkfd = bpf_link_create(progfd,
                              0,
                              static_cast<enum ::bpf_attach_type>(
@@ -460,12 +446,10 @@ bool BPFfeature::has_kprobe_multi()
 
   has_kprobe_multi_ = linkfd >= 0;
 
-  if (linkfd >= 0)
-  {
+  if (linkfd >= 0) {
     close(linkfd);
   }
-  if (progfd >= 0)
-  {
+  if (progfd >= 0) {
     close(progfd);
   }
   return *has_kprobe_multi_;
@@ -477,8 +461,7 @@ bool BPFfeature::has_uprobe_multi()
     return *has_uprobe_multi_;
 
 #if defined(HAVE_LIBBPF_UPROBE_MULTI)
-  if (no_feature_.uprobe_multi_)
-  {
+  if (no_feature_.uprobe_multi_) {
     has_uprobe_multi_ = false;
     return *has_uprobe_multi_;
   }
@@ -503,8 +486,7 @@ bool BPFfeature::has_uprobe_multi()
                          ARRAY_SIZE(insns),
                          &load_opts);
 
-  if (progfd >= 0)
-  {
+  if (progfd >= 0) {
     LIBBPF_OPTS(bpf_link_create_opts, link_opts);
     const unsigned long offset = 0;
 
@@ -522,12 +504,10 @@ bool BPFfeature::has_uprobe_multi()
 
   has_uprobe_multi_ = linkfd < 0 && err == -EBADF;
 
-  if (linkfd >= 0)
-  {
+  if (linkfd >= 0) {
     close(linkfd);
   }
-  if (progfd >= 0)
-  {
+  if (progfd >= 0) {
     close(progfd);
   }
 #else
@@ -599,14 +579,12 @@ bool BPFfeature::has_raw_tp_special()
                ARRAY_SIZE(insns),
                nullptr,
                std::nullopt,
-               &fd))
-  {
+               &fd)) {
     struct bpf_test_run_opts opts = {};
     opts.sz = sizeof(opts);
     has_raw_tp_special_ = !::bpf_prog_test_run_opts(fd, &opts);
     close(fd);
-  }
-  else
+  } else
     has_raw_tp_special_ = false;
 
   return *has_raw_tp_special_;
@@ -615,10 +593,7 @@ bool BPFfeature::has_raw_tp_special()
 std::string BPFfeature::report(void)
 {
   std::stringstream buf;
-  auto to_str = [](bool f) -> auto
-  {
-    return f ? "yes\n" : "no\n";
-  };
+  auto to_str = [](bool f) -> auto { return f ? "yes\n" : "no\n"; };
 
   buf << "Kernel helpers" << std::endl
       << "  probe_read: " << to_str(has_helper_probe_read())
@@ -638,6 +613,7 @@ std::string BPFfeature::report(void)
       << "  get_tai_ns: " << to_str(has_helper_ktime_get_tai_ns())
       << "  get_func_ip: " << to_str(has_helper_get_func_ip())
       << "  jiffies64: " << to_str(has_helper_jiffies64())
+      << "  for_each_map_elem: " << to_str(has_helper_for_each_map_elem())
 
       << std::endl;
 
@@ -674,8 +650,7 @@ std::string BPFfeature::report(void)
 
 bool BPFfeature::has_prog_kfunc()
 {
-  if (!has_prog_kfunc_.has_value())
-  {
+  if (!has_prog_kfunc_.has_value()) {
     int progfd;
     if (!detect_prog_type(libbpf::BPF_PROG_TYPE_TRACING,
                           "sched_fork",
