@@ -19,8 +19,6 @@
 #include "child.h"
 #include "config.h"
 #include "dwarf_parser.h"
-#include "map.h"
-#include "mapmanager.h"
 #include "output.h"
 #include "pcap_writer.h"
 #include "printf.h"
@@ -31,37 +29,36 @@
 #include "types.h"
 #include "utils.h"
 
+#include <bcc/bcc_syms.h>
+
 namespace bpftrace {
 
 const int timeout_ms = 100;
 
-struct symbol
-{
+struct symbol {
   std::string name;
   uint64_t start;
   uint64_t size;
   uint64_t address;
 };
 
+struct stack_key {
+  int64_t stackid;
+  uint32_t nr_stack_frames;
+};
+
 enum class DebugLevel;
 
 // globals
 extern DebugLevel bt_debug;
 extern bool bt_quiet;
 extern bool bt_verbose;
-extern bool bt_verbose2;
 
-enum class DebugLevel
-{
-  kNone,
-  kDebug,
-  kFullDebug
-};
+enum class DebugLevel { kNone, kDebug, kFullDebug };
 
 inline DebugLevel operator++(DebugLevel &level, int)
 {
-  switch (level)
-  {
+  switch (level) {
     case DebugLevel::kNone:
       level = DebugLevel::kDebug;
       break;
@@ -78,10 +75,11 @@ inline DebugLevel operator++(DebugLevel &level, int)
   return level;
 }
 
-class WildcardException : public std::exception
-{
+class WildcardException : public std::exception {
 public:
-  WildcardException(const std::string &msg) : msg_(msg) {}
+  WildcardException(const std::string &msg) : msg_(msg)
+  {
+  }
 
   const char *what() const noexcept override
   {
@@ -92,8 +90,7 @@ private:
   std::string msg_;
 };
 
-class BPFtrace
-{
+class BPFtrace {
 public:
   BPFtrace(std::unique_ptr<Output> o = std::make_unique<TextOutput>(std::cout),
            BPFnofeature no_feature = BPFnofeature(),
@@ -106,9 +103,10 @@ public:
   {
   }
   virtual ~BPFtrace();
-  virtual int add_probe(ast::Probe &p);
-  Probe generateWatchpointSetupProbe(const std::string &func,
-                                     const ast::AttachPoint &ap,
+  virtual int add_probe(const ast::AttachPoint &ap,
+                        const ast::Probe &p,
+                        int usdt_location_idx = 0);
+  Probe generateWatchpointSetupProbe(const ast::AttachPoint &ap,
                                      const ast::Probe &probe);
   int num_probes() const;
   int prerun() const;
@@ -118,10 +116,11 @@ public:
       const BpfBytecode &bytecode);
   int run_iter();
   int print_maps();
-  int clear_map(IMap &map);
-  int zero_map(IMap &map);
-  int print_map(IMap &map, uint32_t top, uint32_t div);
+  int clear_map(const BpfMap &map);
+  int zero_map(const BpfMap &map);
+  int print_map(const BpfMap &map, uint32_t top, uint32_t div);
   std::string get_stack(int64_t stackid,
+                        uint32_t nr_stack_frames,
                         int pid,
                         int probe_id,
                         bool ustack,
@@ -146,10 +145,11 @@ public:
   std::string resolve_mac_address(const uint8_t *mac_addr) const;
   std::string resolve_cgroup_path(uint64_t cgroup_path_id,
                                   uint64_t cgroup_id) const;
-  virtual std::string extract_func_symbols_from_path(const std::string &path) const;
   std::string resolve_probe(uint64_t probe_id) const;
   uint64_t resolve_cgroupid(const std::string &path) const;
-  std::vector<std::unique_ptr<IPrintable>> get_arg_values(const std::vector<Field> &args, uint8_t* arg_data);
+  std::vector<std::unique_ptr<IPrintable>> get_arg_values(
+      const std::vector<Field> &args,
+      uint8_t *arg_data);
   void add_param(const std::string &param);
   std::string get_param(size_t index, bool is_str) const;
   size_t num_params() const;
@@ -158,6 +158,7 @@ public:
   std::optional<int64_t> get_int_literal(const ast::Expression *expr) const;
   std::optional<std::string> get_watchpoint_binary_path() const;
   virtual bool is_traceable_func(const std::string &func_name) const;
+  virtual int get_num_possible_cpus() const;
   virtual std::unordered_set<std::string> get_func_modules(
       const std::string &func_name) const;
   int create_pcaps(void);
@@ -168,6 +169,7 @@ public:
   bool has_btf_data() const;
   Dwarf *get_dwarf(const std::string &filename);
   Dwarf *get_dwarf(const ast::AttachPoint &attachpoint);
+  void kfunc_recursion_check(ast::Program *prog);
 
   std::string cmd_;
   bool finalize_ = false;
@@ -176,7 +178,6 @@ public:
   static volatile sig_atomic_t sigusr1_recv;
 
   RequiredResources resources;
-  MapManager maps;
   BpfBytecode bytecode_;
   StructManager structs;
   std::map<std::string, std::string> macros_;
@@ -202,8 +203,9 @@ public:
   bool debug_output_ = false;
   std::optional<struct timespec> boottime_;
   std::optional<struct timespec> delta_taitime_;
-  static constexpr uint32_t rb_loss_cnt_key_ = 0;
-  static constexpr uint64_t rb_loss_cnt_val_ = 0;
+  static constexpr uint32_t event_loss_cnt_key_ = 0;
+  static constexpr uint64_t event_loss_cnt_val_ = 0;
+  bool need_recursion_check_ = false;
 
   static void sort_by_key(
       std::vector<SizedType> key_args,
@@ -228,7 +230,7 @@ private:
   int run_special_probe(std::string name,
                         const BpfBytecode &bytecode,
                         void (*trigger)(void));
-  void* ksyms_{nullptr};
+  void *ksyms_{ nullptr };
   // note: exe_sym_ is used when layout is same for all instances of program
   std::map<std::string, std::pair<int, void *>> exe_sym_; // exe -> (pid, cache)
   std::map<int, void *> pid_sym_;                         // pid -> cache
@@ -245,7 +247,8 @@ private:
       bool file_activation);
   int setup_output();
   int setup_perf_events();
-  int setup_ringbuf();
+  void setup_ringbuf();
+  int setup_event_loss();
   // when the ringbuf feature is available, enable ringbuf for built-ins like
   // printf, cat.
   bool is_ringbuf_enabled(void) const
@@ -261,15 +264,19 @@ private:
   void teardown_output();
   void poll_output(bool drain = false);
   int poll_perf_events();
-  void handle_ringbuf_loss();
-  int print_map_hist(IMap &map, uint32_t top, uint32_t div);
-  int print_map_stats(IMap &map, uint32_t top, uint32_t div);
+  void handle_event_loss();
+  int print_map_hist(const BpfMap &map, uint32_t top, uint32_t div);
+  int print_map_stats(const BpfMap &map, uint32_t top, uint32_t div);
   static uint64_t read_address_from_output(std::string output);
-  std::vector<uint8_t> find_empty_key(IMap &map, size_t size) const;
+  std::optional<std::vector<uint8_t>> find_empty_key(const BpfMap &map) const;
+  struct bcc_symbol_option &get_symbol_opts();
+  Probe generate_probe(const ast::AttachPoint &ap,
+                       const ast::Probe &p,
+                       int usdt_location_idx = 0);
   bool has_iter_ = false;
   int epollfd_ = -1;
   struct ring_buffer *ringbuf_ = nullptr;
-  uint64_t ringbuf_loss_count_ = 0;
+  uint64_t event_loss_count_ = 0;
 
   // Mapping traceable functions to modules (or "vmlinux") they appear in.
   // Needs to be mutable to allow lazy loading of the mapping from const lookup
