@@ -6,6 +6,7 @@
 #include "tracefs.h"
 #include "types.h"
 #include "utils.h"
+#include <algorithm>
 #include <cmath>
 #include <cstring>
 #include <fcntl.h>
@@ -45,19 +46,15 @@ BTF::BTF(const std::set<std::string> &modules) : state(NODATA)
 {
   // Try to get BTF file from BPFTRACE_BTF env
   char *path = std::getenv("BPFTRACE_BTF");
-  if (path)
-  {
+  if (path) {
     btf_objects.push_back(
         BTFObj{ .btf = btf__parse_raw(path), .id = 0, .name = "" });
     vmlinux_btf = btf_objects.back().btf;
-  }
-  else
+  } else
     load_kernel_btfs(modules);
 
-  if (btf_objects.empty())
-  {
-    if (bt_debug != DebugLevel::kNone)
-      LOG(ERROR) << "BTF: failed to find BTF data";
+  if (btf_objects.empty()) {
+    LOG(V1) << "BTF: failed to find BTF data";
     return;
   }
 
@@ -75,10 +72,8 @@ BTF::~BTF()
 void BTF::load_kernel_btfs(const std::set<std::string> &modules)
 {
   vmlinux_btf = btf__load_vmlinux_btf();
-  if (libbpf_get_error(vmlinux_btf))
-  {
-    if (bt_debug != DebugLevel::kNone)
-      LOG(ERROR) << "BTF: failed to find BTF data for vmlinux, errno " << errno;
+  if (libbpf_get_error(vmlinux_btf)) {
+    LOG(V1) << "BTF: failed to find BTF data for vmlinux, errno " << errno;
     return;
   }
   btf_objects.push_back(
@@ -90,22 +85,18 @@ void BTF::load_kernel_btfs(const std::set<std::string> &modules)
   // Note that we cannot parse BTFs from /sys/kernel/btf/ as we need BTF object
   // IDs, so the only way is to iterate through all loaded BTF objects
   __u32 id = 0;
-  while (true)
-  {
+  while (true) {
     int err = bpf_btf_get_next_id(id, &id);
-    if (err)
-    {
-      if (errno != ENOENT && bt_debug != DebugLevel::kNone)
-        LOG(WARNING) << "BTF: failed to iterate modules BTF objects";
+    if (err) {
+      if (errno != ENOENT)
+        LOG(V1) << "BTF: failed to iterate modules BTF objects";
       break;
     }
 
     // Get BTF object FD
     int fd = bpf_btf_get_fd_by_id(id);
-    if (fd < 0)
-    {
-      if (bt_debug != DebugLevel::kNone)
-        LOG(WARNING) << "BTF: failed to get FD for object with id " << id;
+    if (fd < 0) {
+      LOG(V1) << "BTF: failed to get FD for object with id " << id;
       continue;
     }
 
@@ -118,10 +109,8 @@ void BTF::load_kernel_btfs(const std::set<std::string> &modules)
     __u32 info_len = sizeof(info);
     err = bpf_obj_get_info_by_fd(fd, &info, &info_len);
     close(fd); // close the FD not to leave too many files open
-    if (err)
-    {
-      if (bt_debug != DebugLevel::kNone)
-        LOG(WARNING) << "BTF: failed to get info for object with id " << id;
+    if (err) {
+      LOG(V1) << "BTF: failed to get info for object with id " << id;
       continue;
     }
 
@@ -129,12 +118,9 @@ void BTF::load_kernel_btfs(const std::set<std::string> &modules)
     if (!info.kernel_btf)
       continue;
 
-    if (mod_name == "vmlinux")
-    {
+    if (mod_name == "vmlinux") {
       btf_objects.front().id = id;
-    }
-    else if (modules.find(mod_name) != modules.end())
-    {
+    } else if (modules.find(mod_name) != modules.end()) {
       btf_objects.push_back(
           BTFObj{ .btf = btf__load_from_kernel_by_id_split(id, vmlinux_btf),
                   .id = id,
@@ -145,7 +131,7 @@ void BTF::load_kernel_btfs(const std::set<std::string> &modules)
 
 static void dump_printf(void *ctx, const char *fmt, va_list args)
 {
-  std::string *ret = static_cast<std::string*>(ctx);
+  std::string *ret = static_cast<std::string *>(ctx);
   char *str;
 
   if (vasprintf(&str, fmt, args) < 0)
@@ -167,10 +153,11 @@ static const char *btf_str(const struct btf *btf, __u32 off)
   if (!off)
     return "(anon)";
 
-  return btf__name_by_offset(btf, off) ? : "(invalid)";
+  return btf__name_by_offset(btf, off) ?: "(invalid)";
 }
 
-static std::string full_type_str(const struct btf *btf, const struct btf_type *type)
+static std::string full_type_str(const struct btf *btf,
+                                 const struct btf_type *type)
 {
   const char *str = btf_str(btf, type->name_off);
 
@@ -186,7 +173,7 @@ static std::string full_type_str(const struct btf *btf, const struct btf_type *t
   return str;
 }
 
-static std::string btf_type_str(const std::string& type)
+static std::string btf_type_str(const std::string &type)
 {
   return std::regex_replace(type, std::regex("^(struct )|(union )"), "");
 }
@@ -202,8 +189,7 @@ std::string BTF::dump_defs_from_btf(
 
   dump = dump_new(btf, dump_printf, &ret);
   err = libbpf_get_error(dump);
-  if (err)
-  {
+  if (err) {
     libbpf_strerror(err, err_buf, sizeof(err_buf));
     LOG(ERROR) << "BTF: failed to initialize dump (" << err_buf << ")";
     return std::string("");
@@ -213,23 +199,19 @@ std::string BTF::dump_defs_from_btf(
 
   // note that we're always iterating from 1 here as we need to go through the
   // vmlinux BTF entries, too (even for kernel module BTFs)
-  for (id = 1; id <= max && !types.empty(); id++)
-  {
+  for (id = 1; id <= max && !types.empty(); id++) {
     const struct btf_type *t = btf__type_by_id(btf, id);
     if (!t)
       continue;
 
     // Allow users to reference enum values by name to pull in entire enum defs
-    if (btf_is_enum(t))
-    {
+    if (btf_is_enum(t)) {
       const struct btf_enum *p = btf_enum(t);
       uint16_t vlen = btf_vlen(t);
-      for (int e = 0; e < vlen; ++e, ++p)
-      {
+      for (int e = 0; e < vlen; ++e, ++p) {
         std::string str = btf_str(btf, p->name_off);
         auto it = types.find(str);
-        if (it != types.end())
-        {
+        if (it != types.end()) {
           btf_dump__dump_type(dump, id);
           types.erase(it);
           break;
@@ -240,8 +222,7 @@ std::string BTF::dump_defs_from_btf(
     std::string str = full_type_str(btf, t);
 
     auto it = types.find(str);
-    if (it != types.end())
-    {
+    if (it != types.end()) {
       btf_dump__dump_type(dump, id);
       types.erase(it);
     }
@@ -259,8 +240,7 @@ std::string BTF::c_def(const std::unordered_set<std::string> &set) const
   // Definition dumping from multiple modules would require to resolve type
   // conflicts, so we allow dumping from a single module or from vmlinux only.
   std::unordered_set<std::string> to_dump(set);
-  if (btf_objects.size() == 2)
-  {
+  if (btf_objects.size() == 2) {
     auto *mod_btf = btf_objects[0].btf == vmlinux_btf ? btf_objects[1].btf
                                                       : btf_objects[0].btf;
     return dump_defs_from_btf(mod_btf, to_dump);
@@ -290,9 +270,8 @@ std::string BTF::type_of(const BTFId &type_id, const std::string &field)
 
   const struct btf_type *type = btf__type_by_id(type_id.btf, type_id.id);
 
-  if (!type ||
-      (BTF_INFO_KIND(type->info) != BTF_KIND_STRUCT &&
-       BTF_INFO_KIND(type->info) != BTF_KIND_UNION))
+  if (!type || (BTF_INFO_KIND(type->info) != BTF_KIND_STRUCT &&
+                BTF_INFO_KIND(type->info) != BTF_KIND_UNION))
     return std::string("");
 
   // We need to walk through oaa the struct/union members
@@ -302,13 +281,11 @@ std::string BTF::type_of(const BTFId &type_id, const std::string &field)
   //  https://www.kernel.org/doc/html/latest/bpf/btf.html#btf-kind-union
   const struct btf_member *m = btf_members(type);
 
-  for (unsigned int i = 0; i < BTF_INFO_VLEN(type->info); i++)
-  {
+  for (unsigned int i = 0; i < BTF_INFO_VLEN(type->info); i++) {
     std::string m_name = btf__name_by_offset(type_id.btf, m[i].name_off);
 
     // anonymous struct/union
-    if (m_name == "")
-    {
+    if (m_name == "") {
       std::string type_name = type_of(
           BTFId{ .btf = type_id.btf, .id = m[i].type }, field);
       if (!type_name.empty())
@@ -327,8 +304,7 @@ std::string BTF::type_of(const BTFId &type_id, const std::string &field)
     while (BTF_INFO_KIND(f->info) == BTF_KIND_PTR ||
            BTF_INFO_KIND(f->info) == BTF_KIND_CONST ||
            BTF_INFO_KIND(f->info) == BTF_KIND_VOLATILE ||
-           BTF_INFO_KIND(f->info) == BTF_KIND_RESTRICT)
-    {
+           BTF_INFO_KIND(f->info) == BTF_KIND_RESTRICT) {
       f = btf__type_by_id(type_id.btf, f->type);
       if (!f)
         return std::string("");
@@ -351,8 +327,7 @@ static bool btf_type_is_modifier(const struct btf_type *t)
   // ptr does not fall into this bucket
   // because its size is always sizeof(void *).
 
-  switch (BTF_INFO_KIND(t->info))
-  {
+  switch (BTF_INFO_KIND(t->info)) {
     case BTF_KIND_TYPEDEF:
     case BTF_KIND_VOLATILE:
     case BTF_KIND_CONST:
@@ -366,14 +341,28 @@ static bool btf_type_is_modifier(const struct btf_type *t)
 const struct btf_type *BTF::btf_type_skip_modifiers(const struct btf_type *t,
                                                     const struct btf *btf)
 {
-  while (t && btf_type_is_modifier(t))
-  {
+  while (t && btf_type_is_modifier(t)) {
     t = btf__type_by_id(btf, t->type);
   }
 
   return t;
 }
 
+__u32 BTF::get_type_tags(std::unordered_set<std::string> &tags,
+                         const BTFId &btf_id) const
+{
+  __u32 id = btf_id.id;
+  const struct btf_type *t = btf__type_by_id(btf_id.btf, btf_id.id);
+
+  while (t && btf_is_type_tag(t)) {
+    tags.insert(btf_str(btf_id.btf, t->name_off));
+    id = t->type;
+    t = btf__type_by_id(btf_id.btf, t->type);
+  }
+
+  return id;
+}
+
 SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
 {
   const struct btf_type *t = btf__type_by_id(btf_id.btf, btf_id.id);
@@ -385,17 +374,12 @@ SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
 
   auto stype = CreateNone();
 
-  if (btf_is_int(t))
-  {
+  if (btf_is_int(t)) {
     stype = CreateInteger(btf_int_bits(t),
                           btf_int_encoding(t) & BTF_INT_SIGNED);
-  }
-  else if (btf_is_enum(t))
-  {
+  } else if (btf_is_enum(t)) {
     stype = CreateInteger(t->size * 8, false);
-  }
-  else if (btf_is_composite(t))
-  {
+  } else if (btf_is_composite(t)) {
     std::string cast = btf_str(btf_id.btf, t->name_off);
     if (cast.empty() || cast == "(anon)")
       return CreateNone();
@@ -405,24 +389,20 @@ SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
     stype = CreateRecord(name, bpftrace_->structs.LookupOrAdd(name, t->size));
     if (resolve_structs)
       resolve_fields(stype);
-  }
-  else if (btf_is_ptr(t))
-  {
-    // t->type is the pointee type
+  } else if (btf_is_ptr(t)) {
+    const BTFId pointee_btf_id = { .btf = btf_id.btf, .id = t->type };
+    std::unordered_set<std::string> tags;
+    auto id = get_type_tags(tags, pointee_btf_id);
     stype = CreatePointer(
-        get_stype(BTFId{ .btf = btf_id.btf, .id = t->type }, false));
-  }
-  else if (btf_is_array(t))
-  {
+        get_stype(BTFId{ .btf = btf_id.btf, .id = id }, false));
+    stype.SetBtfTypeTags(std::move(tags));
+  } else if (btf_is_array(t)) {
     auto *array = btf_array(t);
     const auto &elem_type = get_stype(
         BTFId{ .btf = btf_id.btf, .id = array->type });
-    if (elem_type.type == Type::integer && elem_type.GetSize() == 1)
-    {
+    if (elem_type.IsIntTy() && elem_type.GetSize() == 1) {
       stype = CreateString(array->nelems);
-    }
-    else
-    {
+    } else {
       stype = CreateArray(array->nelems, elem_type);
     }
   }
@@ -430,48 +410,57 @@ SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
   return stype;
 }
 
-Struct BTF::resolve_args(const std::string &func, bool ret)
+std::optional<Struct> BTF::resolve_args(const std::string &func,
+                                        bool ret,
+                                        std::string &err)
 {
-  if (!has_data())
-    throw std::runtime_error("BTF data not available");
+  if (!has_data()) {
+    err = "BTF data not available";
+    return std::nullopt;
+  }
 
   auto func_id = find_id(func, BTF_KIND_FUNC);
-  if (!func_id.btf)
-    throw std::runtime_error("no BTF data for " + func);
+  if (!func_id.btf) {
+    err = "no BTF data for " + func;
+    return std::nullopt;
+  }
 
   const struct btf_type *t = btf__type_by_id(func_id.btf, func_id.id);
   t = btf__type_by_id(func_id.btf, t->type);
-  if (!t || !btf_is_func_proto(t))
-    throw std::runtime_error(func + " is not a function");
+  if (!t || !btf_is_func_proto(t)) {
+    err = func + " is not a function";
+    return std::nullopt;
+  }
 
-  if (bpftrace_ && !bpftrace_->is_traceable_func(func))
-  {
-    if (bpftrace_->get_traceable_funcs().empty())
-      throw std::runtime_error("could not read traceable functions from " +
-                               tracefs::available_filter_functions() +
-                               " (is tracefs mounted?)");
-    else
-      throw std::runtime_error("function not traceable (probably it is "
-                               "inlined or marked as \"notrace\")");
+  if (bpftrace_ && !bpftrace_->is_traceable_func(func)) {
+    if (bpftrace_->get_traceable_funcs().empty()) {
+      err = "could not read traceable functions from " +
+            tracefs::available_filter_functions() + " (is tracefs mounted?)";
+    } else {
+      err = "function not traceable (probably it is "
+            "inlined or marked as \"notrace\")";
+    }
+    return std::nullopt;
   }
 
   const struct btf_param *p = btf_params(t);
   __u16 vlen = btf_vlen(t);
-  if (vlen > arch::max_arg() + 1)
-  {
-    throw std::runtime_error("functions with more than 6 parameters are "
-                             "not supported.");
+  if (vlen > arch::max_arg() + 1) {
+    err = "functions with more than 6 parameters are "
+          "not supported.";
+    return std::nullopt;
   }
 
   Struct args(0, false);
   int j = 0;
   int arg_idx = 0;
   __u32 type_size = 0;
-  for (; j < vlen; j++, p++)
-  {
+  for (; j < vlen; j++, p++) {
     const char *str = btf_str(func_id.btf, p->name_off);
-    if (!str)
-      throw std::runtime_error("failed to resolve arguments");
+    if (!str) {
+      err = "failed to resolve arguments";
+      return std::nullopt;
+    }
 
     SizedType stype = get_stype(BTFId{ .btf = func_id.btf, .id = p->type });
     stype.funcarg_idx = arg_idx;
@@ -485,8 +474,7 @@ Struct BTF::resolve_args(const std::string &func, bool ret)
     arg_idx += std::ceil((float)type_size / (float)8);
   }
 
-  if (ret)
-  {
+  if (ret) {
     SizedType stype = get_stype(BTFId{ .btf = func_id.btf, .id = t->type });
     stype.funcarg_idx = arg_idx;
     stype.is_funcarg = true;
@@ -502,8 +490,7 @@ std::string BTF::get_all_funcs_from_btf(const BTFObj &btf_obj) const
   std::string funcs;
   __s32 id, max = (__s32)type_cnt(btf_obj.btf);
 
-  for (id = start_id(btf_obj.btf); id <= max; id++)
-  {
+  for (id = start_id(btf_obj.btf); id <= max; id++) {
     const struct btf_type *t = btf__type_by_id(btf_obj.btf, id);
 
     if (!t)
@@ -516,11 +503,9 @@ std::string BTF::get_all_funcs_from_btf(const BTFObj &btf_obj) const
     std::string func_name = str;
 
     t = btf__type_by_id(btf_obj.btf, t->type);
-    if (!t || !btf_is_func_proto(t))
-    {
+    if (!t || !btf_is_func_proto(t)) {
       /* bad.. */
-      if (!bt_verbose)
-        LOG(ERROR) << func_name << " function does not have FUNC_PROTO record";
+      LOG(ERROR) << func_name << " function does not have FUNC_PROTO record";
       break;
     }
 
@@ -556,16 +541,14 @@ std::map<std::string, std::vector<std::string>> BTF::get_params_from_btf(
 
   dump = dump_new(btf_obj.btf, dump_printf, &type);
   err = libbpf_get_error(dump);
-  if (err)
-  {
+  if (err) {
     libbpf_strerror(err, err_buf, sizeof(err_buf));
     LOG(ERROR) << "BTF: failed to initialize dump (" << err_buf << ")";
     return {};
   }
 
   std::map<std::string, std::vector<std::string>> params;
-  for (id = start_id(btf_obj.btf); id <= max; id++)
-  {
+  for (id = start_id(btf_obj.btf); id <= max; id++) {
     const struct btf_type *t = btf__type_by_id(btf_obj.btf, id);
 
     if (!t)
@@ -596,15 +579,13 @@ std::map<std::string, std::vector<std::string>> BTF::get_params_from_btf(
         const struct btf_param *p;
     int j;
 
-    for (j = 0, p = btf_params(t); j < btf_vlen(t); j++, p++)
-    {
+    for (j = 0, p = btf_params(t); j < btf_vlen(t); j++, p++) {
       // set by dump_printf callback
       type = std::string("");
       const char *arg_name = btf__name_by_offset(btf_obj.btf, p->name_off);
 
       err = btf_dump__emit_type_decl(dump, p->type, &decl_opts);
-      if (err)
-      {
+      if (err) {
         LOG(ERROR) << "failed to dump argument: " << arg_name;
         break;
       }
@@ -619,8 +600,7 @@ std::map<std::string, std::vector<std::string>> BTF::get_params_from_btf(
     type = std::string("");
 
     err = btf_dump__emit_type_decl(dump, t->type, &decl_opts);
-    if (err)
-    {
+    if (err) {
       LOG(ERROR) << "failed to dump return type for: " << func_name;
       break;
     }
@@ -644,8 +624,7 @@ std::map<std::string, std::vector<std::string>> BTF::get_params(
     return params.find(f) != params.end();
   };
 
-  for (auto &btf_obj : btf_objects)
-  {
+  for (auto &btf_obj : btf_objects) {
     if (std::all_of(funcs.begin(), funcs.end(), all_resolved))
       break;
 
@@ -667,15 +646,13 @@ std::set<std::string> BTF::get_all_structs_from_btf(const struct btf *btf) const
 
   dump = dump_new(btf, dump_printf, &types);
   err = libbpf_get_error(dump);
-  if (err)
-  {
+  if (err) {
     libbpf_strerror(err, err_buf, sizeof(err_buf));
     LOG(ERROR) << "BTF: failed to initialize dump (" << err_buf << ")";
     return {};
   }
 
-  for (id = start_id(btf); id <= max; id++)
-  {
+  for (id = start_id(btf); id <= max; id++) {
     const struct btf_type *t = btf__type_by_id(btf, id);
 
     if (!t || !(btf_is_struct(t) || btf_is_union(t) || btf_is_enum(t)))
@@ -697,28 +674,22 @@ std::set<std::string> BTF::get_all_structs_from_btf(const struct btf *btf) const
 
   btf_dump__free(dump);
 
-  if (bt_verbose)
-  {
+  if (bt_verbose) {
     // BTF dump contains definitions of all types in a single string, here we
     // split it
     std::istringstream type_stream(types);
     std::string line, type;
     bool in_def = false;
-    while (std::getline(type_stream, line))
-    {
-      if (in_def)
-      {
+    while (std::getline(type_stream, line)) {
+      if (in_def) {
         type += line + "\n";
-        if (line == "};")
-        {
+        if (line == "};") {
           // end of type definition
           struct_set.insert(type);
           type.clear();
           in_def = false;
         }
-      }
-      else if (!line.empty() && line.back() == '{')
-      {
+      } else if (!line.empty() && line.back() == '{') {
         // start of type definition
         type += line + "\n";
         in_def = true;
@@ -732,8 +703,7 @@ std::set<std::string> BTF::get_all_structs_from_btf(const struct btf *btf) const
 std::set<std::string> BTF::get_all_structs() const
 {
   std::set<std::string> structs;
-  for (auto &btf_obj : btf_objects)
-  {
+  for (auto &btf_obj : btf_objects) {
     auto mod_structs = get_all_structs_from_btf(btf_obj.btf);
     structs.insert(mod_structs.begin(), mod_structs.end());
   }
@@ -749,8 +719,7 @@ std::unordered_set<std::string> BTF::get_all_iters_from_btf(
   // verifier.") add 'struct bpf_iter__task__safe_trusted'
   const std::string suffix___safe_trusted = "__safe_trusted";
   __s32 id, max = (__s32)type_cnt(btf);
-  for (id = start_id(btf); id <= max; id++)
-  {
+  for (id = start_id(btf); id <= max; id++) {
     const struct btf_type *t = btf__type_by_id(btf, id);
 
     if (!t || !(btf_is_struct(t)))
@@ -764,8 +733,7 @@ std::unordered_set<std::string> BTF::get_all_iters_from_btf(
             (name.length() - suffix___safe_trusted.length()))
       continue;
     if (name.size() > prefix.size() &&
-        name.compare(0, prefix.size(), prefix) == 0)
-    {
+        name.compare(0, prefix.size(), prefix) == 0) {
       iter_set.insert(name.substr(prefix.size()));
     }
   }
@@ -775,8 +743,7 @@ std::unordered_set<std::string> BTF::get_all_iters_from_btf(
 std::unordered_set<std::string> BTF::get_all_iters() const
 {
   std::unordered_set<std::string> iters;
-  for (auto &btf_obj : btf_objects)
-  {
+  for (auto &btf_obj : btf_objects) {
     auto mod_iters = get_all_iters_from_btf(btf_obj.btf);
     iters.insert(mod_iters.begin(), mod_iters.end());
   }
@@ -788,8 +755,7 @@ std::unordered_set<std::string> BTF::get_all_iters() const
 std::pair<int, int> BTF::get_btf_id_fd(const std::string &func,
                                        const std::string &mod) const
 {
-  for (auto &btf_obj : btf_objects)
-  {
+  for (auto &btf_obj : btf_objects) {
     if (!mod.empty() && mod != btf_obj.name)
       continue;
 
@@ -804,8 +770,7 @@ std::pair<int, int> BTF::get_btf_id_fd(const std::string &func,
 BTF::BTFId BTF::find_id(const std::string &name,
                         std::optional<__u32> kind) const
 {
-  for (auto &btf_obj : btf_objects)
-  {
+  for (auto &btf_obj : btf_objects) {
     __s32 id = kind ? btf__find_by_name_kind(btf_obj.btf, name.c_str(), *kind)
                     : btf__find_by_name(btf_obj.btf, name.c_str());
     if (id >= 0)
@@ -819,8 +784,7 @@ __s32 BTF::find_id_in_btf(struct btf *btf,
                           const std::string &name,
                           std::optional<__u32> kind) const
 {
-  for (__s32 id = start_id(btf), max = (__s32)type_cnt(btf); id <= max; ++id)
-  {
+  for (__s32 id = start_id(btf), max = (__s32)type_cnt(btf); id <= max; ++id) {
     const struct btf_type *t = btf__type_by_id(btf, id);
     if (!t)
       continue;
@@ -873,12 +837,10 @@ void BTF::resolve_fields(const BTFId &type_id,
   if (!btf_type)
     return;
   auto members = btf_members(btf_type);
-  for (__u32 i = 0; i < BTF_INFO_VLEN(btf_type->info); i++)
-  {
+  for (__u32 i = 0; i < BTF_INFO_VLEN(btf_type->info); i++) {
     BTFId field_id{ .btf = type_id.btf, .id = members[i].type };
     auto field_type = btf__type_by_id(field_id.btf, field_id.id);
-    if (!field_type)
-    {
+    if (!field_type) {
       LOG(ERROR) << "Inconsistent BTF data (no type found for id "
                  << members[i].type << ")";
       record->ClearFields();
@@ -891,10 +853,9 @@ void BTF::resolve_fields(const BTFId &type_id,
     __u32 field_offset = start_offset + btf_member_bit_offset(btf_type, i) / 8;
 
     if (btf_is_composite(field_type) &&
-        (field_name.empty() || field_name == "(anon)"))
-    {
+        (field_name.empty() || field_name == "(anon)")) {
       resolve_fields(field_id, record, field_offset);
-      return;
+      continue;
     }
 
     record->AddField(field_name,
