@@ -29,18 +29,15 @@ namespace bpftrace {
 
 class BPFtrace;
 
-class BTF
-{
-  enum state
-  {
+class BTF {
+  enum state {
     NODATA,
     OK,
   };
 
   // BTF object for vmlinux or a kernel module.
   // We're currently storing its name and BTF id.
-  struct BTFObj
-  {
+  struct BTFObj {
     struct btf* btf;
     __u32 id;
     std::string name;
@@ -48,8 +45,7 @@ class BTF
 
   // It is often necessary to store a BTF id along with the BTF data containing
   // its definition.
-  struct BTFId
-  {
+  struct BTFId {
     struct btf* btf;
     __u32 id;
   };
@@ -78,7 +74,9 @@ public:
   std::map<std::string, std::vector<std::string>> get_params(
       const std::set<std::string>& funcs) const;
 
-  Struct resolve_args(const std::string& func, bool ret);
+  std::optional<Struct> resolve_args(const std::string& func,
+                                     bool ret,
+                                     std::string& err);
   void resolve_fields(SizedType& type);
 
   std::pair<int, int> get_btf_id_fd(const std::string& func,
@@ -105,6 +103,13 @@ private:
   std::set<std::string> get_all_structs_from_btf(const struct btf* btf) const;
   std::unordered_set<std::string> get_all_iters_from_btf(
       const struct btf* btf) const;
+  /*
+   * Similar to btf_type_skip_modifiers this returns the id of the first
+   * type that is not a BTF_KIND_TYPE_TAG while also populating the tags set
+   * with the tag/attribute names from the BTF_KIND_TYPE_TAG types it finds.
+   */
+  __u32 get_type_tags(std::unordered_set<std::string>& tags,
+                      const BTFId& btf_id) const;
 
   __s32 start_id(const struct btf* btf) const;
 
