@@ -1,7 +1,10 @@
+#include <llvm/Config/llvm-config.h>
 #include <sstream>
 
 #include "build_info.h"
 
+#include "version.h"
+
 namespace bpftrace {
 
 std::string BuildInfo::report()
@@ -24,8 +27,14 @@ std::string BuildInfo::report()
 #else
       << "no" << std::endl;
 #endif
-  buf << "  libdw (DWARF support): "
-#ifdef HAVE_LIBDW
+  buf << "  liblldb (DWARF support): "
+#ifdef HAVE_LIBLLDB
+      << "yes" << std::endl;
+#else
+      << "no" << std::endl;
+#endif
+  buf << "  libsystemd (systemd notify support): "
+#ifdef HAVE_LIBSYSTEMD
       << "yes" << std::endl;
 #else
       << "no" << std::endl;
