@@ -141,8 +141,7 @@ static bool is_anonymous(CXCursor c)
   if (clang_Cursor_isNull(parent))
     return false;
 
-  struct AnonFinderState
-  {
+  struct AnonFinderState {
     CXCursor struct_to_check;
     bool is_anon;
     bool prev_was_definition;
@@ -154,11 +153,9 @@ static bool is_anonymous(CXCursor c)
 
   clang_visitChildren(
       parent,
-      [](CXCursor c2, CXCursor, CXClientData client_data)
-      {
-        auto state = static_cast<struct AnonFinderState*>(client_data);
-        if (state->prev_was_definition)
-        {
+      [](CXCursor c2, CXCursor, CXClientData client_data) {
+        auto state = static_cast<struct AnonFinderState *>(client_data);
+        if (state->prev_was_definition) {
           // This is the next child after the definition of the struct we're
           // interested in. If it is a field containing a record, we assume
           // that it must be the field for our struct, so our struct is not
@@ -166,15 +163,16 @@ static bool is_anonymous(CXCursor c)
           state->prev_was_definition = false;
           auto kind = clang_getCursorKind(c2);
           auto type = clang_getCanonicalType(clang_getCursorType(c2));
-          if (kind == CXCursor_FieldDecl && type.kind == CXType_Record)
-          {
+          if (kind == CXCursor_FieldDecl && type.kind == CXType_Record) {
             state->is_anon = false;
             return CXChildVisit_Break;
           }
         }
 
         // We've found the definition of the struct we're interested in
-        if (memcmp(c2.data, state->struct_to_check.data, 3*sizeof(uintptr_t)) == 0)
+        if (memcmp(c2.data,
+                   state->struct_to_check.data,
+                   3 * sizeof(uintptr_t)) == 0)
           state->prev_was_definition = true;
         return CXChildVisit_Continue;
       },
@@ -194,8 +192,7 @@ static CXCursor get_named_parent(CXCursor c)
 {
   CXCursor parent = clang_getCursorSemanticParent(c);
 
-  while (!clang_Cursor_isNull(parent) && is_anonymous(parent))
-  {
+  while (!clang_Cursor_isNull(parent) && is_anonymous(parent)) {
     parent = clang_getCursorSemanticParent(parent);
   }
 
@@ -212,28 +209,26 @@ static std::optional<Bitfield> getBitfield(CXCursor c)
                   clang_getFieldDeclBitWidth(c));
 }
 
-// NOTE(mmarchini): as suggested in http://clang-developers.42468.n3.nabble.com/Extracting-macro-information-using-libclang-the-C-Interface-to-Clang-td4042648.html#message4042666
-static bool translateMacro(CXCursor cursor, std::string &name, std::string &value)
+// NOTE(mmarchini): as suggested in
+// http://clang-developers.42468.n3.nabble.com/Extracting-macro-information-using-libclang-the-C-Interface-to-Clang-td4042648.html#message4042666
+static bool translateMacro(CXCursor cursor,
+                           std::string &name,
+                           std::string &value)
 {
-  CXToken* tokens    = nullptr;
+  CXToken *tokens = nullptr;
   unsigned numTokens = 0;
   CXTranslationUnit transUnit = clang_Cursor_getTranslationUnit(cursor);
-  CXSourceRange srcRange  = clang_getCursorExtent(cursor);
+  CXSourceRange srcRange = clang_getCursorExtent(cursor);
   clang_tokenize(transUnit, srcRange, &tokens, &numTokens);
-  for (unsigned n=0; n<numTokens; n++)
-  {
+  for (unsigned n = 0; n < numTokens; n++) {
     auto tokenText = clang_getTokenSpelling(transUnit, tokens[n]);
-    if (n == 0)
-    {
+    if (n == 0) {
       value.clear();
       name = clang_getCString(tokenText);
-    }
-    else
-    {
+    } else {
       CXTokenKind tokenKind = clang_getTokenKind(tokens[n]);
-      if (tokenKind != CXToken_Comment)
-      {
-        const char* text = clang_getCString(tokenText);
+      if (tokenKind != CXToken_Comment) {
+        const char *text = clang_getCString(tokenText);
         if (text)
           value += text;
       }
@@ -263,8 +258,7 @@ static SizedType get_sized_type(CXType clang_type, StructManager &structs)
   auto size = 8 * clang_Type_getSizeOf(clang_type);
   auto typestr = get_unqualified_type_name(clang_type);
 
-  switch (clang_type.kind)
-  {
+  switch (clang_type.kind) {
     case CXType_Bool:
     case CXType_Char_U:
     case CXType_UChar:
@@ -288,17 +282,14 @@ static SizedType get_sized_type(CXType clang_type, StructManager &structs)
       return CreateInt(size);
     case CXType_Enum:
       return CreateUInt(size);
-    case CXType_Pointer:
-    {
+    case CXType_Pointer: {
       auto pointee_type = clang_getPointeeType(clang_type);
       return CreatePointer(get_sized_type(pointee_type, structs));
     }
-    case CXType_ConstantArray:
-    {
+    case CXType_ConstantArray: {
       auto elem_type = clang_getArrayElementType(clang_type);
       auto size = clang_getNumElements(clang_type);
-      if (elem_type.kind == CXType_Char_S || elem_type.kind == CXType_Char_U)
-      {
+      if (elem_type.kind == CXType_Char_S || elem_type.kind == CXType_Char_U) {
         return CreateString(size);
       }
 
@@ -321,7 +312,8 @@ ClangParser::ClangParserHandler::~ClangParserHandler()
   clang_disposeIndex(index);
 }
 
-CXTranslationUnit ClangParser::ClangParserHandler::get_translation_unit() {
+CXTranslationUnit ClangParser::ClangParserHandler::get_translation_unit()
+{
   return translation_unit;
 }
 
@@ -336,46 +328,43 @@ CXErrorCode ClangParser::ClangParserHandler::parse_translation_unit(
   // Clean up previous translation unit to prevent resource leak
   clang_disposeTranslationUnit(translation_unit);
 
-  return clang_parseTranslationUnit2(
-      index,
-      source_filename,
-      command_line_args, num_command_line_args,
-      unsaved_files, num_unsaved_files,
-      options,
-      &translation_unit);
+  return clang_parseTranslationUnit2(index,
+                                     source_filename,
+                                     command_line_args,
+                                     num_command_line_args,
+                                     unsaved_files,
+                                     num_unsaved_files,
+                                     options,
+                                     &translation_unit);
 }
 
-bool ClangParser::ClangParserHandler::check_diagnostics(
-    const std::string &input,
-    bool bail_on_error)
+bool ClangParser::ClangParserHandler::check_diagnostics(bool bail_on_error)
 {
-  for (unsigned int i=0; i < clang_getNumDiagnostics(get_translation_unit()); i++) {
+  for (unsigned int i = 0; i < clang_getNumDiagnostics(get_translation_unit());
+       i++) {
     CXDiagnostic diag = clang_getDiagnostic(get_translation_unit(), i);
     CXDiagnosticSeverity severity = clang_getDiagnosticSeverity(diag);
     std::string msg = clang_getCString(clang_getDiagnosticSpelling(diag));
     error_msgs.push_back(msg);
 
     if ((bail_on_error && severity == CXDiagnostic_Error) ||
-        severity == CXDiagnostic_Fatal)
-    {
+        severity == CXDiagnostic_Fatal) {
       // Do not fail on "too many errors"
       if (!bail_on_error && msg == "too many errors emitted, stopping now")
         return true;
-      if (bt_debug >= DebugLevel::kDebug)
-        LOG(ERROR) << "Input (" << input.size() << "): " << input;
       return false;
     }
   }
   return true;
 }
 
-CXCursor ClangParser::ClangParserHandler::get_translation_unit_cursor() {
+CXCursor ClangParser::ClangParserHandler::get_translation_unit_cursor()
+{
   return clang_getTranslationUnitCursor(translation_unit);
 }
 
 bool ClangParser::ClangParserHandler::parse_file(
     const std::string &filename,
-    const std::string &input,
     const std::vector<const char *> &args,
     std::vector<CXUnsavedFile> &unsaved_files,
     bool bail_on_errors)
@@ -393,26 +382,23 @@ bool ClangParser::ClangParserHandler::parse_file(
       CXTranslationUnit_DetailedPreprocessingRecord);
 
   error_msgs.clear();
-  if (error)
-  {
-    if (bt_debug == DebugLevel::kFullDebug)
-      LOG(ERROR) << "Clang error while parsing C definitions: " << error;
+  if (error) {
+    LOG(V1) << "Clang error while parsing C definitions: " << error;
     return false;
   }
 
-  return check_diagnostics(input, bail_on_errors);
+  return check_diagnostics(bail_on_errors);
 }
 
-const std::vector<std::string>
-    &ClangParser::ClangParserHandler::get_error_messages()
+const std::vector<std::string> &ClangParser::ClangParserHandler::
+    get_error_messages()
 {
   return error_msgs;
 }
 
 bool ClangParser::ClangParserHandler::has_redefinition_error()
 {
-  for (auto &msg : error_msgs)
-  {
+  for (auto &msg : error_msgs) {
     if (msg.find("redefinition") != std::string::npos)
       return true;
   }
@@ -421,8 +407,7 @@ bool ClangParser::ClangParserHandler::has_redefinition_error()
 
 bool ClangParser::ClangParserHandler::has_unknown_type_error()
 {
-  for (auto &msg : error_msgs)
-  {
+  for (auto &msg : error_msgs) {
     if (ClangParser::get_unknown_type(msg).has_value())
       return true;
   }
@@ -436,30 +421,29 @@ std::optional<std::string> get_field_decl_annotation(CXCursor c)
   assert(clang_getCursorKind(c) == CXCursor_FieldDecl);
 
   std::optional<std::string> annotation;
-  clang_visitChildren(c,
-                      [](CXCursor c,
-                         CXCursor __attribute__((unused)) parent,
-                         CXClientData data) {
-                        // The header generation code can annotate some struct
-                        // fields with additional information for us to parse
-                        // here. The annotation looks like:
-                        //
-                        //    struct Foo {
-                        //      __attribute__((annotate("tp_data_loc"))) int
-                        //      name;
-                        //    };
-                        //
-                        // Currently only the TracepointFormatParser does this.
-                        if (clang_getCursorKind(c) == CXCursor_AnnotateAttr)
-                        {
-                          auto &a = *static_cast<std::optional<std::string> *>(
-                              data);
-                          a = get_clang_string(clang_getCursorSpelling(c));
-                        }
+  clang_visitChildren(
+      c,
+      [](CXCursor c,
+         CXCursor __attribute__((unused)) parent,
+         CXClientData data) {
+        // The header generation code can annotate some struct
+        // fields with additional information for us to parse
+        // here. The annotation looks like:
+        //
+        //    struct Foo {
+        //      __attribute__((annotate("tp_data_loc"))) int
+        //      name;
+        //    };
+        //
+        // Currently only the TracepointFormatParser does this.
+        if (clang_getCursorKind(c) == CXCursor_AnnotateAttr) {
+          auto &a = *static_cast<std::optional<std::string> *>(data);
+          a = get_clang_string(clang_getCursorSpelling(c));
+        }
 
-                        return CXChildVisit_Recurse;
-                      },
-                      &annotation);
+        return CXChildVisit_Recurse;
+      },
+      &annotation);
 
   return annotation;
 }
@@ -469,24 +453,21 @@ bool ClangParser::visit_children(CXCursor &cursor, BPFtrace &bpftrace)
 {
   int err = clang_visitChildren(
       cursor,
-      [](CXCursor c, CXCursor parent, CXClientData client_data)
-      {
-        if (clang_getCursorKind(c) == CXCursor_MacroDefinition)
-        {
+      [](CXCursor c, CXCursor parent, CXClientData client_data) {
+        if (clang_getCursorKind(c) == CXCursor_MacroDefinition) {
           std::string macro_name;
           std::string macro_value;
-          if (translateMacro(c, macro_name, macro_value))
-          {
-            auto &macros = static_cast<BPFtrace*>(client_data)->macros_;
+          if (translateMacro(c, macro_name, macro_value)) {
+            auto &macros = static_cast<BPFtrace *>(client_data)->macros_;
             macros[macro_name] = macro_value;
           }
           return CXChildVisit_Recurse;
         }
 
-        if (clang_getCursorKind(parent) == CXCursor_EnumDecl)
-        {
-          auto &enums = static_cast<BPFtrace*>(client_data)->enums_;
-          enums[get_clang_string(clang_getCursorSpelling(c))] = clang_getEnumConstantDeclValue(c);
+        if (clang_getCursorKind(parent) == CXCursor_EnumDecl) {
+          auto &enums = static_cast<BPFtrace *>(client_data)->enums_;
+          enums[get_clang_string(clang_getCursorSpelling(c))] =
+              clang_getEnumConstantDeclValue(c);
           return CXChildVisit_Recurse;
         }
 
@@ -494,12 +475,12 @@ bool ClangParser::visit_children(CXCursor &cursor, BPFtrace &bpftrace)
             clang_getCursorKind(parent) != CXCursor_UnionDecl)
           return CXChildVisit_Recurse;
 
-        if (clang_getCursorKind(c) == CXCursor_FieldDecl)
-        {
+        if (clang_getCursorKind(c) == CXCursor_FieldDecl) {
           auto &structs = static_cast<BPFtrace *>(client_data)->structs;
 
           auto named_parent = get_named_parent(c);
-          auto ptype = clang_getCanonicalType(clang_getCursorType(named_parent));
+          auto ptype = clang_getCanonicalType(
+              clang_getCursorType(named_parent));
           auto ptypestr = get_unqualified_type_name(ptype);
           auto ptypesize = clang_Type_getSizeOf(ptype);
 
@@ -512,10 +493,8 @@ bool ClangParser::visit_children(CXCursor &cursor, BPFtrace &bpftrace)
 
           // Process field annotations
           auto annotation = get_field_decl_annotation(c);
-          if (annotation)
-          {
-            if (*annotation == "tp_data_loc")
-            {
+          if (annotation) {
+            if (*annotation == "tp_data_loc") {
               // If the field is a tracepoint __data_loc, we need to rewrite the
               // type as a u64. The reason is that the tracepoint infrastructure
               // exports an encoded 32bit integer that tells us where to find
@@ -532,8 +511,7 @@ bool ClangParser::visit_children(CXCursor &cursor, BPFtrace &bpftrace)
             structs.Add(ptypestr, ptypesize, false);
 
           auto str = structs.Lookup(ptypestr).lock();
-          if (str->allow_override)
-          {
+          if (str->allow_override) {
             str->ClearFields();
             str->allow_override = false;
           }
@@ -562,11 +540,10 @@ std::unordered_set<std::string> ClangParser::get_incomplete_types()
   // Parse without failing on compilation errors (ie incomplete structs) because
   // our goal is to enumerate all such errors.
   ClangParserHandler handler;
-  if (!handler.parse_file("definitions.h", input, args, input_files, false))
+  if (!handler.parse_file("definitions.h", args, input_files, false))
     return {};
 
-  struct TypeData
-  {
+  struct TypeData {
     std::unordered_set<std::string> complete_types;
     std::unordered_set<std::string> incomplete_types;
   } type_data;
@@ -587,8 +564,7 @@ std::unordered_set<std::string> ClangParser::get_incomplete_types()
         if (clang_getCursorKind(parent) == CXCursor_EnumDecl ||
             (clang_getCursorKind(c) == CXCursor_FieldDecl &&
              (clang_getCursorKind(parent) == CXCursor_UnionDecl ||
-              clang_getCursorKind(parent) == CXCursor_StructDecl)))
-        {
+              clang_getCursorKind(parent) == CXCursor_StructDecl))) {
           auto parent_type = clang_getCanonicalType(
               clang_getCursorType(parent));
           data.complete_types.emplace(get_unqualified_type_name(parent_type));
@@ -597,8 +573,7 @@ std::unordered_set<std::string> ClangParser::get_incomplete_types()
           // We need layouts of pointee types because users could dereference
           if (cursor_type.kind == CXType_Pointer)
             cursor_type = clang_getPointeeType(cursor_type);
-          if (cursor_type.kind == CXType_Record)
-          {
+          if (cursor_type.kind == CXType_Record) {
             auto type_name = get_unqualified_type_name(cursor_type);
             if (data.complete_types.find(type_name) ==
                 data.complete_types.end())
@@ -630,8 +605,7 @@ void ClangParser::resolve_incomplete_types_from_btf(
       bpftrace.config_.get(ConfigKeyInt::max_type_res_iterations), field_lvl);
 
   bool check_incomplete_types = true;
-  for (unsigned i = 0; i < max_iterations && check_incomplete_types; i++)
-  {
+  for (unsigned i = 0; i < max_iterations && check_incomplete_types; i++) {
     // Collect incomplete types and retrieve their definitions from BTF.
     auto incomplete_types = get_incomplete_types();
     size_t types_cnt = bpftrace.btf_set_.size();
@@ -672,16 +646,16 @@ void ClangParser::resolve_incomplete_types_from_btf(
  * their definitions but once he redefines any kernel type, he must provide all
  * necessary definitions.
  */
-bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<std::string> extra_flags)
+bool ClangParser::parse(ast::Program *program,
+                        BPFtrace &bpftrace,
+                        std::vector<std::string> extra_flags)
 {
 #ifdef FUZZ
   StderrSilencer silencer;
   silencer.silence();
 #endif
-  if (program->c_definitions.empty() && bpftrace.btf_set_.empty())
-    return true;
-
-  input = "#include <__btf_generated_header.h>\n" + program->c_definitions;
+  input = "#include </bpftrace/include/__btf_generated_header.h>\n" +
+          program->c_definitions;
 
   input_files = getTranslationUnitFiles(CXUnsavedFile{
       .Filename = "definitions.h",
@@ -691,8 +665,7 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
 
   args = { "-isystem", "/bpftrace/include" };
   auto system_paths = system_include_paths();
-  for (auto &path : system_paths)
-  {
+  for (auto &path : system_paths) {
     args.push_back("-isystem");
     args.push_back(path.c_str());
   }
@@ -700,8 +673,7 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
   args.push_back("-isystem");
   args.push_back(arch_path.c_str());
 
-  for (auto &flag : extra_flags)
-  {
+  for (auto &flag : extra_flags) {
     args.push_back(flag.c_str());
   }
 
@@ -715,8 +687,7 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
 
   bool btf_conflict = false;
   ClangParserHandler handler;
-  if (bpftrace.has_btf_data())
-  {
+  if (bpftrace.has_btf_data()) {
     // We set these args early because some systems may not have <linux/types.h>
     // (containers) and fully rely on BTF.
 
@@ -730,33 +701,28 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
     // conditionally include headers if BTF isn't available.
     args.push_back("-DBPFTRACE_HAVE_BTF");
 
-    if (handler.parse_file("definitions.h", input, args, input_files, false) &&
+    if (handler.parse_file("definitions.h", args, input_files, false) &&
         handler.has_redefinition_error())
       btf_conflict = true;
 
-    if (!btf_conflict)
-    {
+    if (!btf_conflict) {
       resolve_incomplete_types_from_btf(bpftrace, program->probes);
 
-      if (handler.parse_file(
-              "definitions.h", input, args, input_files, false) &&
+      if (handler.parse_file("definitions.h", args, input_files, false) &&
           handler.has_redefinition_error())
         btf_conflict = true;
     }
 
-    if (!btf_conflict)
-    {
+    if (!btf_conflict) {
       resolve_unknown_typedefs_from_btf(bpftrace);
 
-      if (handler.parse_file(
-              "definitions.h", input, args, input_files, false) &&
+      if (handler.parse_file("definitions.h", args, input_files, false) &&
           handler.has_redefinition_error())
         btf_conflict = true;
     }
   }
 
-  if (btf_conflict)
-  {
+  if (btf_conflict) {
     // There is a conflict (redefinition) between user-supplied types and types
     // taken from BTF. We cannot use BTF in such a case.
     args.pop_back();
@@ -765,19 +731,14 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
     input_files.back() = get_empty_btf_generated_header();
   }
 
-  if (!handler.parse_file("definitions.h", input, args, input_files))
-  {
-    if (handler.has_redefinition_error())
-    {
+  if (!handler.parse_file("definitions.h", args, input_files)) {
+    if (handler.has_redefinition_error()) {
       LOG(WARNING) << "Cannot take type definitions from BTF since there is "
                       "a redefinition conflict with user-defined types.";
-    }
-    else if (handler.has_unknown_type_error())
-    {
+    } else if (handler.has_unknown_type_error()) {
       LOG(ERROR) << "Include headers with missing type definitions or install "
                     "BTF information to your system.";
-      if (bpftrace.btf_->objects_cnt() > 2)
-      {
+      if (bpftrace.btf_->objects_cnt() > 2) {
         LOG(WARNING)
             << "Trying to dump BTF from multiple kernel modules at once. "
             << "This is currently not possible, use probes from a single module"
@@ -803,10 +764,8 @@ std::optional<std::string> ClangParser::ClangParser::get_unknown_type(
   const std::vector<std::string> unknown_type_msgs = {
     "unknown type name \'", "use of undeclared identifier \'"
   };
-  for (auto &unknown_type_msg : unknown_type_msgs)
-  {
-    if (diagnostic_msg.find(unknown_type_msg) == 0)
-    {
+  for (auto &unknown_type_msg : unknown_type_msgs) {
+    if (diagnostic_msg.find(unknown_type_msg) == 0) {
       return diagnostic_msg.substr(unknown_type_msg.length(),
                                    diagnostic_msg.length() -
                                        unknown_type_msg.length() - 1);
@@ -820,7 +779,7 @@ std::unordered_set<std::string> ClangParser::get_unknown_typedefs()
   // Parse without failing on compilation errors (ie unknown types) because
   // our goal is to enumerate and analyse all such errors
   ClangParserHandler handler;
-  if (!handler.parse_file("definitions.h", input, args, input_files, false))
+  if (!handler.parse_file("definitions.h", args, input_files, false))
     return {};
 
   std::unordered_set<std::string> unknown_typedefs;
@@ -828,8 +787,7 @@ std::unordered_set<std::string> ClangParser::get_unknown_typedefs()
   //   unknown type name 'type_t'
   // that imply an unresolved typedef of type_t. This cannot be done in
   // clang_visitChildren since clang does not have the unknown type names.
-  for (const auto &msg : handler.get_error_messages())
-  {
+  for (const auto &msg : handler.get_error_messages()) {
     auto unknown_type = get_unknown_type(msg);
     if (unknown_type)
       unknown_typedefs.emplace(unknown_type.value());
@@ -840,8 +798,7 @@ std::unordered_set<std::string> ClangParser::get_unknown_typedefs()
 void ClangParser::resolve_unknown_typedefs_from_btf(BPFtrace &bpftrace)
 {
   bool check_unknown_types = true;
-  while (check_unknown_types)
-  {
+  while (check_unknown_types) {
     // Collect unknown typedefs and retrieve their definitions from BTF.
     // These must be resolved completely since any unknown typedef will cause
     // the parser to fail (even if that type is not used in the program).
@@ -884,25 +841,33 @@ std::string ClangParser::get_arch_include_path()
   return "/usr/include/" + std::string(utsname.machine) + "-linux-gnu";
 }
 
-std::vector<std::string> ClangParser::system_include_paths()
+static void query_clang_include_dirs(std::vector<std::string> &result)
 {
-  std::vector<std::string> result;
-  try
-  {
+  try {
     auto clang = "clang-" + std::to_string(LLVM_VERSION_MAJOR);
     auto cmd = clang + " -Wp,-v -x c -fsyntax-only /dev/null 2>&1";
     auto check = exec_system(cmd.c_str());
     std::istringstream lines(check);
     std::string line;
     while (std::getline(lines, line) &&
-           line != "#include <...> search starts here:")
-    {
+           line != "#include <...> search starts here:") {
     }
     while (std::getline(lines, line) && line != "End of search list.")
       result.push_back(trim(line));
+  } catch (std::runtime_error &) { // If exec_system fails, just ignore it
   }
-  catch (std::runtime_error &)
-  { // If exec_system fails, just ignore it
+}
+
+std::vector<std::string> ClangParser::system_include_paths()
+{
+  std::vector<std::string> result;
+  std::istringstream lines(SYSTEM_INCLUDE_PATHS);
+  std::string line;
+  while (std::getline(lines, line, ':')) {
+    if (line == "auto")
+      query_clang_include_dirs(result);
+    else
+      result.push_back(trim(line));
   }
 
   if (result.empty())
