@@ -5,23 +5,15 @@
 
 namespace bpftrace {
 
-enum class AlignState
-{
-  Ok,
-  Fail,
-  NotAlign,
-  NotSupp
-};
+enum class AlignState { Ok, Fail, NotAlign, NotSupp };
 
-class IDisasm
-{
+class IDisasm {
 public:
   virtual AlignState is_aligned(uint64_t offset, uint64_t pc) = 0;
   virtual ~IDisasm() = default;
 };
 
-class Disasm
-{
+class Disasm {
 public:
   Disasm(std::string &path);
 
