@@ -2,22 +2,22 @@
 
 #include <fstream>
 #include <memory>
+#include <string_view>
 
 #include "ast/ast.h"
 #include "bpftrace.h"
 
-typedef void* yyscan_t;
+typedef void *yyscan_t;
 
 namespace bpftrace {
 
-class Driver
-{
+class Driver {
 public:
   explicit Driver(BPFtrace &bpftrace, std::ostream &o = std::cerr);
 
   int parse();
-  int parse_str(std::string script);
-  void source(std::string, std::string);
+  int parse_str(std::string_view script);
+  void source(std::string_view, std::string_view);
   void error(std::ostream &, const location &, const std::string &);
   void error(const location &l, const std::string &m);
   void error(const std::string &m);
