@@ -1,6 +1,5 @@
 #include <array>
 #include <bpf/libbpf.h>
-#include <csignal>
 #include <cstdio>
 #include <cstring>
 #include <fstream>
@@ -22,6 +21,7 @@
 #include "ast/passes/node_counter.h"
 #include "ast/passes/portability_analyser.h"
 #include "ast/passes/resource_analyser.h"
+#include "ast/passes/return_path_analyser.h"
 #include "ast/passes/semantic_analyser.h"
 
 #include "bpffeature.h"
@@ -37,7 +37,10 @@
 #include "output.h"
 #include "probe_matcher.h"
 #include "procmon.h"
+#include "run_bpftrace.h"
 #include "tracepoint_format_parser.h"
+#include "utils.h"
+#include "version.h"
 
 using namespace bpftrace;
 
@@ -49,22 +52,19 @@ enum class OutputBufferConfig {
   NONE,
 };
 
-enum class TestMode
-{
+enum class TestMode {
   UNSET = 0,
   CODEGEN,
 };
 
-enum class BuildMode
-{
+enum class BuildMode {
   // Compile script and run immediately
   DYNAMIC = 0,
   // Compile script into portable executable
   AHEAD_OF_TIME,
 };
 
-enum Options
-{
+enum Options {
   INFO = 2000,
   NO_WARNING,
   TEST,
@@ -97,7 +97,8 @@ void usage()
   std::cerr << "    -h, --help     show this help message" << std::endl;
   std::cerr << "    -I DIR         add the directory to the include search path" << std::endl;
   std::cerr << "    --include FILE add an #include file before preprocessing" << std::endl;
-  std::cerr << "    -l [search]    list probes" << std::endl;
+  std::cerr << "    -l [search|filename]" << std::endl;
+  std::cerr << "                   list kernel probes or probes in a program" << std::endl;
   std::cerr << "    -p PID         enable USDT probes on PID" << std::endl;
   std::cerr << "    -c 'CMD'       run CMD and enable USDT probes on resulting process" << std::endl;
   std::cerr << "    --usdt-file-activation" << std::endl;
@@ -112,7 +113,6 @@ void usage()
   std::cerr << std::endl;
   std::cerr << "TROUBLESHOOTING OPTIONS:" << std::endl;
   std::cerr << "    -v                      verbose messages" << std::endl;
-  std::cerr << "    -vv                     more verbose messages (max 2)" << std::endl;
   std::cerr << "    -d                      (dry run) debug info" << std::endl;
   std::cerr << "    -dd                     (dry run) verbose debug info" << std::endl;
   std::cerr << "    --emit-elf FILE         (dry run) generate ELF file with bpf programs and write to FILE" << std::endl;
@@ -125,6 +125,7 @@ void usage()
   std::cerr << "    BPFTRACE_DEBUG_OUTPUT             [default: 0] enable bpftrace's internal debugging outputs" << std::endl;
   std::cerr << "    BPFTRACE_KERNEL_BUILD             [default: /lib/modules/$(uname -r)] kernel build directory" << std::endl;
   std::cerr << "    BPFTRACE_KERNEL_SOURCE            [default: /lib/modules/$(uname -r)] kernel headers directory" << std::endl;
+  std::cerr << "    BPFTRACE_LAZY_SYMBOLICATION       [default: 0] symbolicate lazily/on-demand" << std::endl;
   std::cerr << "    BPFTRACE_LOG_SIZE                 [default: 1000000] log size in bytes" << std::endl;
   std::cerr << "    BPFTRACE_MAX_BPF_PROGS            [default: 512] max number of generated BPF programs" << std::endl;
   std::cerr << "    BPFTRACE_MAX_CAT_BYTES            [default: 10k] maximum bytes read by cat builtin" << std::endl;
@@ -147,7 +148,8 @@ void usage()
   // clang-format on
 }
 
-static void enforce_infinite_rlimit() {
+static void enforce_infinite_rlimit()
+{
   struct rlimit rl = {};
   int err;
 
@@ -155,20 +157,17 @@ static void enforce_infinite_rlimit() {
   rl.rlim_cur = rl.rlim_max;
   err = setrlimit(RLIMIT_MEMLOCK, &rl);
   if (err)
-    LOG(ERROR) << std::strerror(err) << ": couldn't set RLIMIT_MEMLOCK for "
-               << "bpftrace. If your program is not loading, you can try "
-               << "\"ulimit -l 8192\" to fix the problem";
+    LOG(WARNING) << std::strerror(err) << ": couldn't set RLIMIT_MEMLOCK for "
+                 << "bpftrace. If your program is not loading, you can try "
+                 << "\"ulimit -l 8192\" to fix the problem";
 }
 
-bool is_root()
+void check_is_root()
 {
-  if (geteuid() != 0)
-  {
+  if (geteuid() != 0) {
     LOG(ERROR) << "bpftrace currently only supports running as the root user.";
-    return false;
+    exit(1);
   }
-  else
-    return true;
 }
 
 static void info(BPFnofeature no_feature)
@@ -196,8 +195,7 @@ static std::optional<struct timespec> get_delta_with_boottime(int clock_type)
   // Run the "triple vdso sandwich" 5 times, taking the result from the
   // iteration with the lowest delta between first and last clock_gettime()
   // calls.
-  for (int i = 0; i < 5; ++i)
-  {
+  for (int i = 0; i < 5; ++i) {
     struct timespec before, after, boottime;
     long delta;
 
@@ -224,17 +222,13 @@ static std::optional<struct timespec> get_delta_with_boottime(int clock_type)
       continue;
 
     // Lowest delta seen so far, compute boot realtime and store it
-    if (delta < lowest_delta)
-    {
+    if (delta < lowest_delta) {
       struct timespec delta_with_boottime;
       long nsec_avg = (before.tv_nsec + after.tv_nsec) / 2;
-      if (nsec_avg - boottime.tv_nsec < 0)
-      {
+      if (nsec_avg - boottime.tv_nsec < 0) {
         delta_with_boottime.tv_sec = after.tv_sec - boottime.tv_sec - 1;
         delta_with_boottime.tv_nsec = nsec_avg - boottime.tv_nsec + 1e9;
-      }
-      else
-      {
+      } else {
         delta_with_boottime.tv_sec = after.tv_sec - boottime.tv_sec;
         delta_with_boottime.tv_nsec = nsec_avg - boottime.tv_nsec;
       }
@@ -262,131 +256,96 @@ static std::optional<struct timespec> get_delta_taitime()
   return get_delta_with_boottime(CLOCK_TAI);
 }
 
-[[nodiscard]] static bool parse_env(BPFtrace& bpftrace)
+static void parse_env(BPFtrace& bpftrace)
 {
   ConfigSetter config_setter(bpftrace.config_, ConfigSource::env_var);
-  if (!get_uint64_env_var("BPFTRACE_MAX_STRLEN", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::max_strlen, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_MAX_STRLEN", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::max_strlen, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_STRLEN", [&](uint64_t x) {
-        LOG(WARNING) << "BPFTRACE_STRLEN is deprecated. Use "
-                        "BPFTRACE_MAX_STRLEN instead.";
-        config_setter.set(ConfigKeyInt::max_strlen, x);
-      }))
-    return false;
-
-  // in practice, the largest buffer I've seen fit into the BPF stack was 240
-  // bytes. I've set the bar lower, in case your program has a deeper stack than
-  // the one from my tests, in the hope that you'll get this instructive error
-  // instead of getting the BPF verifier's error.
-  uint64_t max_strlen = bpftrace.config_.get(ConfigKeyInt::max_strlen);
-  if (max_strlen > 200)
-  {
-    // the verifier errors you would encounter when attempting larger
-    // allocations would be: >240=  <Looks like the BPF stack limit of 512 bytes
-    // is exceeded. Please move large on stack variables into BPF per-cpu array
-    // map.> ~1024= <A call to built-in function 'memset' is not supported.>
-    LOG(ERROR) << "'BPFTRACE_MAX_STRLEN' " << max_strlen
-               << " exceeds the current maximum of 200 bytes.\n"
-               << "This limitation is because strings are currently stored on "
-                  "the 512 byte BPF stack.\n"
-               << "Long strings will be pursued in: "
-                  "https://github.com/iovisor/bpftrace/issues/305";
-    return false;
-  }
+  get_uint64_env_var("BPFTRACE_STRLEN", [&](uint64_t x) {
+    LOG(WARNING) << "BPFTRACE_STRLEN is deprecated. Use "
+                    "BPFTRACE_MAX_STRLEN instead.";
+    config_setter.set(ConfigKeyInt::max_strlen, x);
+  });
 
   if (const char* env_p = std::getenv("BPFTRACE_STR_TRUNC_TRAILER"))
     config_setter.set(ConfigKeyString::str_trunc_trailer, std::string(env_p));
 
-  if (!get_bool_env_var("BPFTRACE_CPP_DEMANGLE", [&](bool x) {
-        config_setter.set(ConfigKeyBool::cpp_demangle, x);
-      }))
-    return false;
+  get_bool_env_var("BPFTRACE_CPP_DEMANGLE", [&](bool x) {
+    config_setter.set(ConfigKeyBool::cpp_demangle, x);
+  });
 
-  if (!get_bool_env_var("BPFTRACE_DEBUG_OUTPUT",
-                        [&](bool x) { bpftrace.debug_output_ = x; }))
-    return false;
+  get_bool_env_var("BPFTRACE_DEBUG_OUTPUT",
+                   [&](bool x) { bpftrace.debug_output_ = x; });
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_MAP_KEYS", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::max_map_keys, x);
-      }))
-    return false;
+  get_bool_env_var("BPFTRACE_LAZY_SYMBOLICATION", [&](bool x) {
+    config_setter.set(ConfigKeyBool::lazy_symbolication, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_PROBES", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::max_probes, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_MAX_MAP_KEYS", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::max_map_keys, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_BPF_PROGS", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::max_bpf_progs, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_MAX_PROBES", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::max_probes, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_LOG_SIZE", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::log_size, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_MAX_BPF_PROGS", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::max_bpf_progs, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_PERF_RB_PAGES", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::perf_rb_pages, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_LOG_SIZE", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::log_size, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_TYPE_RES_ITERATIONS", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::max_type_res_iterations, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_PERF_RB_PAGES", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::perf_rb_pages, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_CAT_BYTES", [&](uint64_t x) {
-        config_setter.set(ConfigKeyInt::max_cat_bytes, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_MAX_TYPE_RES_ITERATIONS", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::max_type_res_iterations, x);
+  });
 
-  if (const char* env_p = std::getenv("BPFTRACE_CACHE_USER_SYMBOLS"))
-  {
+  get_uint64_env_var("BPFTRACE_MAX_CAT_BYTES", [&](uint64_t x) {
+    config_setter.set(ConfigKeyInt::max_cat_bytes, x);
+  });
+
+  if (const char* env_p = std::getenv("BPFTRACE_CACHE_USER_SYMBOLS")) {
     const std::string s(env_p);
     if (!config_setter.set_user_symbol_cache_type(s))
-      return false;
+      exit(1);
   }
 
   bpftrace.max_ast_nodes_ = std::numeric_limits<uint64_t>::max();
-  if (!get_uint64_env_var("BPFTRACE_MAX_AST_NODES",
-                          [&](uint64_t x) { bpftrace.max_ast_nodes_ = x; }))
-    return false;
+  get_uint64_env_var("BPFTRACE_MAX_AST_NODES",
+                     [&](uint64_t x) { bpftrace.max_ast_nodes_ = x; });
 
-  if (const char* stack_mode = std::getenv("BPFTRACE_STACK_MODE"))
-  {
+  if (const char* stack_mode = std::getenv("BPFTRACE_STACK_MODE")) {
     if (!config_setter.set_stack_mode(stack_mode))
-      return false;
+      exit(1);
   }
 
-  if (!get_bool_env_var("BPFTRACE_NO_CPP_DEMANGLE", [&](bool x) {
-        LOG(WARNING) << "BPFTRACE_NO_CPP_DEMANGLE is deprecated. Use "
-                        "BPFTRACE_CPP_DEMANGLE=0 instead.";
-        config_setter.set(ConfigKeyBool::cpp_demangle, !x);
-      }))
-    return false;
+  get_bool_env_var("BPFTRACE_NO_CPP_DEMANGLE", [&](bool x) {
+    LOG(WARNING) << "BPFTRACE_NO_CPP_DEMANGLE is deprecated. Use "
+                    "BPFTRACE_CPP_DEMANGLE=0 instead.";
+    config_setter.set(ConfigKeyBool::cpp_demangle, !x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_CAT_BYTES_MAX", [&](uint64_t x) {
-        LOG(WARNING) << "BPFTRACE_CAT_BYTES_MAX is deprecated. Use "
-                        "BPFTRACE_MAX_CAT_BYTES instead.";
-        config_setter.set(ConfigKeyInt::max_cat_bytes, x);
-      }))
-    return false;
+  get_uint64_env_var("BPFTRACE_CAT_BYTES_MAX", [&](uint64_t x) {
+    LOG(WARNING) << "BPFTRACE_CAT_BYTES_MAX is deprecated. Use "
+                    "BPFTRACE_MAX_CAT_BYTES instead.";
+    config_setter.set(ConfigKeyInt::max_cat_bytes, x);
+  });
 
-  if (!get_uint64_env_var("BPFTRACE_MAP_KEYS_MAX", [&](uint64_t x) {
-        LOG(WARNING) << "BPFTRACE_MAP_KEYS_MAX is deprecated. Use "
-                        "BPFTRACE_MAX_MAP_KEYS instead.";
-        config_setter.set(ConfigKeyInt::max_map_keys, x);
-      }))
-    return false;
-
-  return true;
+  get_uint64_env_var("BPFTRACE_MAP_KEYS_MAX", [&](uint64_t x) {
+    LOG(WARNING) << "BPFTRACE_MAP_KEYS_MAX is deprecated. Use "
+                    "BPFTRACE_MAX_MAP_KEYS instead.";
+    config_setter.set(ConfigKeyInt::max_map_keys, x);
+  });
 }
 
-[[nodiscard]] std::unique_ptr<ast::Node> parse(
+[[nodiscard]] std::unique_ptr<ast::Program> parse(
     BPFtrace& bpftrace,
     const std::string& name,
     const std::string& program,
@@ -411,34 +370,6 @@ static std::optional<struct timespec> get_delta_taitime()
   if (TracepointFormatParser::parse(driver.root.get(), bpftrace) == false)
     return nullptr;
 
-  ClangParser clang;
-  std::vector<std::string> extra_flags;
-  {
-    struct utsname utsname;
-    uname(&utsname);
-    std::string ksrc, kobj;
-    auto kdirs = get_kernel_dirs(utsname);
-    ksrc = std::get<0>(kdirs);
-    kobj = std::get<1>(kdirs);
-
-    if (ksrc != "")
-      extra_flags = get_kernel_cflags(
-          utsname.machine, ksrc, kobj, bpftrace.kconfig);
-  }
-  extra_flags.push_back("-include");
-  extra_flags.push_back(CLANG_WORKAROUNDS_H);
-
-  for (auto dir : include_dirs)
-  {
-    extra_flags.push_back("-I");
-    extra_flags.push_back(dir);
-  }
-  for (auto file : include_files)
-  {
-    extra_flags.push_back("-include");
-    extra_flags.push_back(file);
-  }
-
   // NOTE(mmarchini): if there are no C definitions, clang parser won't run to
   // avoid issues in some versions. Since we're including files in the command
   // line, we want to force parsing, so we make sure C definitions are not
@@ -446,8 +377,40 @@ static std::optional<struct timespec> get_delta_taitime()
   if (!include_files.empty() && driver.root->c_definitions.empty())
     driver.root->c_definitions = "#define __BPFTRACE_DUMMY__";
 
-  if (!clang.parse(driver.root.get(), bpftrace, extra_flags))
-    return nullptr;
+  bool should_clang_parse = !(driver.root.get()->c_definitions.empty() &&
+                              bpftrace.btf_set_.empty());
+
+  if (should_clang_parse) {
+    ClangParser clang;
+    std::vector<std::string> extra_flags;
+    {
+      struct utsname utsname;
+      uname(&utsname);
+      std::string ksrc, kobj;
+      auto kdirs = get_kernel_dirs(utsname);
+      ksrc = std::get<0>(kdirs);
+      kobj = std::get<1>(kdirs);
+
+      if (ksrc != "") {
+        extra_flags = get_kernel_cflags(
+            utsname.machine, ksrc, kobj, bpftrace.kconfig);
+      }
+    }
+    extra_flags.push_back("-include");
+    extra_flags.push_back("/bpftrace/include/" CLANG_WORKAROUNDS_H);
+
+    for (auto dir : include_dirs) {
+      extra_flags.push_back("-I");
+      extra_flags.push_back(dir);
+    }
+    for (auto file : include_files) {
+      extra_flags.push_back("-include");
+      extra_flags.push_back(file);
+    }
+
+    if (!clang.parse(driver.root.get(), bpftrace, extra_flags))
+      return nullptr;
+  }
 
   err = driver.parse();
   if (err)
@@ -463,6 +426,7 @@ ast::PassManager CreateDynamicPM()
   pm.AddPass(ast::CreateSemanticPass());
   pm.AddPass(ast::CreateCounterPass());
   pm.AddPass(ast::CreateResourcePass());
+  pm.AddPass(ast::CreateReturnPathPass());
 
   return pm;
 }
@@ -473,12 +437,12 @@ ast::PassManager CreateAotPM()
   pm.AddPass(ast::CreateSemanticPass());
   pm.AddPass(ast::CreatePortabilityPass());
   pm.AddPass(ast::CreateResourcePass());
+  pm.AddPass(ast::CreateReturnPathPass());
 
   return pm;
 }
 
-struct Args
-{
+struct Args {
   std::string pid_str;
   std::string cmd_str;
   bool listing = false;
@@ -526,18 +490,13 @@ Args parse_args(int argc, char* argv[])
   };
 
   int c;
-  while ((c = getopt_long(
-              argc, argv, short_options, long_options, nullptr)) != -1)
-  {
-    switch (c)
-    {
+  while ((c = getopt_long(argc, argv, short_options, long_options, nullptr)) !=
+         -1) {
+    switch (c) {
       case Options::INFO: // --info
-        if (is_root())
-        {
-          info(args.no_feature);
-          exit(0);
-        }
-        exit(1);
+        check_is_root();
+        info(args.no_feature);
+        exit(0);
         break;
       case Options::EMIT_ELF: // --emit-elf
         args.output_elf = optarg;
@@ -551,8 +510,7 @@ Args parse_args(int argc, char* argv[])
       case Options::TEST: // --test
         if (std::strcmp(optarg, "codegen") == 0)
           args.test_mode = TestMode::CODEGEN;
-        else
-        {
+        else {
           LOG(ERROR) << "USAGE: --test can only be 'codegen'.";
           exit(1);
         }
@@ -562,8 +520,7 @@ Args parse_args(int argc, char* argv[])
         args.build_mode = BuildMode::AHEAD_OF_TIME;
         break;
       case Options::NO_FEATURE: // --no-feature
-        if (args.no_feature.parse(optarg))
-        {
+        if (args.no_feature.parse(optarg)) {
           LOG(ERROR) << "USAGE: --no-feature can only have values "
                         "'kprobe_multi,uprobe_multi'.";
           exit(1);
@@ -583,10 +540,8 @@ Args parse_args(int argc, char* argv[])
         bt_quiet = true;
         break;
       case 'v':
-        if (bt_verbose)
-          bt_verbose2 = true;
-        else
-          bt_verbose = true;
+        ENABLE_LOG(V1);
+        bt_verbose = true;
         break;
       case 'B':
         if (std::strcmp(optarg, "line") == 0) {
@@ -640,8 +595,7 @@ Args parse_args(int argc, char* argv[])
         exit(0);
       case 'k':
         args.helper_check_level++;
-        if (args.helper_check_level >= 3)
-        {
+        if (args.helper_check_level >= 3) {
           usage();
           exit(1);
         }
@@ -657,69 +611,60 @@ Args parse_args(int argc, char* argv[])
     exit(1);
   }
 
-  if (bt_verbose && (bt_debug != DebugLevel::kNone))
-  {
+  if (bt_verbose && (bt_debug != DebugLevel::kNone)) {
     // TODO: allow both
     LOG(ERROR) << "USAGE: Use either -v or -d.";
     exit(1);
   }
 
-  if (!args.cmd_str.empty() && !args.pid_str.empty())
-  {
+  if (!args.cmd_str.empty() && !args.pid_str.empty()) {
     LOG(ERROR) << "USAGE: Cannot use both -c and -p.";
     usage();
     exit(1);
   }
 
   // Difficult to serialize flex generated types
-  if (args.helper_check_level && args.build_mode == BuildMode::AHEAD_OF_TIME)
-  {
+  if (args.helper_check_level && args.build_mode == BuildMode::AHEAD_OF_TIME) {
     LOG(ERROR) << "Cannot use -k[k] with --aot";
     exit(1);
   }
 
-  if (args.listing)
-  {
+  if (args.listing) {
     // Expect zero or one positional arguments
-    if (optind == argc)
-    {
+    if (optind == argc) {
       args.search = "*:*";
-    }
-    else if (optind == argc - 1)
-    {
-      args.search = argv[optind];
-      if (args.search == "*")
-      {
-        args.search = "*:*";
+    } else if (optind == argc - 1) {
+      std::string_view val(argv[optind]);
+      if (std_filesystem::exists(val)) {
+        args.filename = val;
+      } else {
+        if (val == "*") {
+          args.search = "*:*";
+        } else {
+          args.search = val;
+        }
       }
       optind++;
-    }
-    else
-    {
+    } else {
       usage();
       exit(1);
     }
-  }
-  else
-  {
+  } else {
     // Expect to find a script either through -e or filename
-    if (args.script.empty() && argv[optind] == nullptr)
-    {
+    if (args.script.empty() && argv[optind] == nullptr) {
       LOG(ERROR) << "USAGE: filename or -e 'program' required.";
       exit(1);
     }
 
     // If no script was specified with -e, then we expect to find a script file
-    if (args.script.empty())
-    {
+    if (args.script.empty()) {
       args.filename = argv[optind];
       optind++;
     }
 
     // Load positional parameters before driver runs so positional
     // parameters used inside attach point definitions can be resolved.
-    while (optind < argc)
-    {
+    while (optind < argc) {
       args.params.push_back(argv[optind]);
       optind++;
     }
@@ -728,66 +673,36 @@ Args parse_args(int argc, char* argv[])
   return args;
 }
 
-static const char* libbpf_print_level_string(enum libbpf_print_level level)
-{
-  switch (level)
-  {
-    case LIBBPF_WARN:
-      return "WARN";
-    case LIBBPF_INFO:
-      return "INFO";
-    default:
-      return "DEBUG";
-  }
-}
-
-static int libbpf_print(enum libbpf_print_level level,
-                        const char* msg,
-                        va_list ap)
-{
-  if (bt_debug == DebugLevel::kNone)
-    return 0;
-
-  fprintf(stderr, "[%s] ", libbpf_print_level_string(level));
-  return vfprintf(stderr, msg, ap);
-}
-
 int main(int argc, char* argv[])
 {
   int err;
 
   const Args args = parse_args(argc, argv);
 
-  std::ostream * os = &std::cout;
+  std::ostream* os = &std::cout;
   std::ofstream outputstream;
-  if (!args.output_file.empty())
-  {
+  if (!args.output_file.empty()) {
     outputstream.open(args.output_file);
     if (outputstream.fail()) {
       LOG(ERROR) << "Failed to open output file: \"" << args.output_file
                  << "\": " << strerror(errno);
-      return 1;
+      exit(1);
     }
     os = &outputstream;
   }
 
   std::unique_ptr<Output> output;
-  if (args.output_format.empty() || args.output_format == "text")
-  {
+  if (args.output_format.empty() || args.output_format == "text") {
     output = std::make_unique<TextOutput>(*os);
-  }
-  else if (args.output_format == "json")
-  {
+  } else if (args.output_format == "json") {
     output = std::make_unique<JsonOutput>(*os);
-  }
-  else {
+  } else {
     LOG(ERROR) << "Invalid output format \"" << args.output_format << "\"\n"
                << "Valid formats: 'text', 'json'";
-    return 1;
+    exit(1);
   }
 
-  switch (args.obc)
-  {
+  switch (args.obc) {
     case OutputBufferConfig::UNSET:
     case OutputBufferConfig::LINE:
       std::setvbuf(stdout, NULL, _IOLBF, BUFSIZ);
@@ -808,8 +723,7 @@ int main(int argc, char* argv[])
   if (!args.cmd_str.empty())
     bpftrace.cmd_ = args.cmd_str;
 
-  if (!parse_env(bpftrace))
-    return 1;
+  parse_env(bpftrace);
 
   bpftrace.usdt_file_activation_ = args.usdt_file_activation;
   bpftrace.safe_mode_ = args.safe_mode;
@@ -817,49 +731,50 @@ int main(int argc, char* argv[])
   bpftrace.boottime_ = get_boottime();
   bpftrace.delta_taitime_ = get_delta_taitime();
 
-  if (!args.pid_str.empty())
-  {
-    try
-    {
-      bpftrace.procmon_ = std::make_unique<ProcMon>(args.pid_str);
+  if (!args.pid_str.empty()) {
+    std::string errmsg;
+    auto maybe_pid = parse_pid(args.pid_str, errmsg);
+    if (!maybe_pid.has_value()) {
+      LOG(ERROR) << "Failed to parse pid: " + errmsg;
+      exit(1);
     }
-    catch (const std::exception& e)
-    {
+    try {
+      bpftrace.procmon_ = std::make_unique<ProcMon>(*maybe_pid);
+    } catch (const std::exception& e) {
       LOG(ERROR) << e.what();
-      return 1;
+      exit(1);
     }
   }
 
-  if (!args.cmd_str.empty())
-  {
+  if (!args.cmd_str.empty()) {
     bpftrace.cmd_ = args.cmd_str;
-    try
-    {
+    try {
       bpftrace.child_ = std::make_unique<ChildProc>(args.cmd_str);
-    }
-    catch (const std::runtime_error& e)
-    {
+    } catch (const std::runtime_error& e) {
       LOG(ERROR) << "Failed to fork child: " << e.what();
-      return -1;
+      exit(1);
     }
   }
 
-  // Listing probes
-  if (args.listing)
-  {
-    if (!is_root())
-      return 1;
+  // Listing probes when there is no program
+  if (args.listing && args.script.empty() && args.filename.empty()) {
+    check_is_root();
 
-    if (args.search.find(':') == std::string::npos &&
-        (args.search.find("struct") == 0 || args.search.find("union") == 0 ||
-         args.search.find("enum") == 0))
-    {
+    if (is_type_name(args.search)) {
       // Print structure definitions
       bpftrace.parse_btf({});
       bpftrace.probe_matcher_->list_structs(args.search);
       return 0;
     }
 
+    if (args.search.find(".") != std::string::npos &&
+        args.search.find_first_of(":*") == std::string::npos) {
+      LOG(WARNING)
+          << "It appears that \'" << args.search
+          << "\' is a filename but the file does not exist. Treating \'"
+          << args.search << "\' as a search pattern.";
+    }
+
     Driver driver(bpftrace);
     driver.listing_ = true;
     driver.source("stdin", args.search);
@@ -882,15 +797,12 @@ int main(int argc, char* argv[])
   std::string filename;
   std::string program;
 
-  if (!args.filename.empty())
-  {
+  if (!args.filename.empty()) {
     std::stringstream buf;
 
-    if (args.filename == "-")
-    {
+    if (args.filename == "-") {
       std::string line;
-      while (std::getline(std::cin, line))
-      {
+      while (std::getline(std::cin, line)) {
         // Note we may add an extra newline if the input doesn't end in a new
         // line. This should not matter because bpftrace (the language) is not
         // whitespace sensitive.
@@ -899,15 +811,12 @@ int main(int argc, char* argv[])
 
       filename = "stdin";
       program = buf.str();
-    }
-    else
-    {
+    } else {
       std::ifstream file(args.filename);
-      if (file.fail())
-      {
+      if (file.fail()) {
         LOG(ERROR) << "failed to open file '" << args.filename
                    << "': " << std::strerror(errno);
-        return -1;
+        exit(1);
       }
 
       filename = args.filename;
@@ -915,25 +824,20 @@ int main(int argc, char* argv[])
       buf << file.rdbuf();
       program = buf.str();
     }
-  }
-  else
-  {
+  } else {
     // Script is provided as a command line argument
     filename = "stdin";
     program = args.script;
   }
 
-  for (const auto& param : args.params)
-  {
+  for (const auto& param : args.params) {
     bpftrace.add_param(param);
   }
 
-  if (!is_root())
-    return 1;
+  check_is_root();
 
   auto lockdown_state = lockdown::detect();
-  if (lockdown_state == lockdown::LockdownState::Confidentiality)
-  {
+  if (lockdown_state == lockdown::LockdownState::Confidentiality) {
     lockdown::emit_warning(std::cerr);
     return 1;
   }
@@ -942,15 +846,19 @@ int main(int argc, char* argv[])
   // rlimit?
   enforce_infinite_rlimit();
 
-  auto ast_root = parse(
+  auto ast_prog = parse(
       bpftrace, filename, program, args.include_dirs, args.include_files);
-  if (!ast_root)
+  if (!ast_prog)
     return 1;
 
+  if (args.listing) {
+    bpftrace.probe_matcher_->list_probes(ast_prog.get());
+    return 0;
+  }
+
   ast::PassContext ctx(bpftrace);
   ast::PassManager pm;
-  switch (args.build_mode)
-  {
+  switch (args.build_mode) {
     case BuildMode::DYNAMIC:
       pm = CreateDynamicPM();
       break;
@@ -959,86 +867,80 @@ int main(int argc, char* argv[])
       break;
   }
 
-  auto pmresult = pm.Run(std::move(ast_root), ctx);
+  bpftrace.kfunc_recursion_check(ast_prog.get());
+
+  auto pmresult = pm.Run(std::move(ast_prog), ctx);
   if (!pmresult.Ok())
     return 1;
 
-  ast_root = std::unique_ptr<ast::Node>(pmresult.Root());
+  auto ast_root = std::unique_ptr<ast::Node>(pmresult.Root());
 
-  if (!bpftrace.cmd_.empty())
-  {
-    try
-    {
+  if (!bpftrace.cmd_.empty()) {
+    try {
       bpftrace.child_ = std::make_unique<ChildProc>(args.cmd_str);
-    }
-    catch (const std::runtime_error& e)
-    {
+    } catch (const std::runtime_error& e) {
       LOG(ERROR) << "Failed to fork child: " << e.what();
-      return -1;
+      exit(1);
     }
   }
 
   err = bpftrace.create_pcaps();
-  if (err)
-  {
+  if (err) {
     LOG(ERROR) << "Failed to create pcap file";
     return err;
   }
 
-  ast::CodegenLLVM llvm(&*ast_root, bpftrace);
+  ast::CodegenLLVM llvm(&*ast_root,
+                        bpftrace,
+                        args.build_mode == BuildMode::AHEAD_OF_TIME);
   BpfBytecode bytecode;
-  try
-  {
+  try {
     llvm.generate_ir();
-    if (bt_debug == DebugLevel::kFullDebug)
-    {
+    if (bt_debug == DebugLevel::kFullDebug) {
       std::cout << "Before optimization\n";
       std::cout << "-------------------\n\n";
       llvm.DumpIR();
     }
-    if (!args.output_llvm.empty())
-    {
+    if (!args.output_llvm.empty()) {
       llvm.DumpIR(args.output_llvm + ".original.ll");
     }
 
     bool verify_llvm_ir = false;
-    if (!get_bool_env_var("BPFTRACE_VERIFY_LLVM_IR",
-                          [&](bool x) { verify_llvm_ir = x; }))
-      return 1;
-    if (verify_llvm_ir && !llvm.verify())
-    {
+    get_bool_env_var("BPFTRACE_VERIFY_LLVM_IR",
+                     [&](bool x) { verify_llvm_ir = x; });
+    if (verify_llvm_ir && !llvm.verify()) {
       LOG(ERROR) << "Verification of generated LLVM IR failed";
-      return 1;
+      exit(1);
     }
 
     llvm.optimize();
-    if (bt_debug != DebugLevel::kNone)
-    {
-      if (bt_debug == DebugLevel::kFullDebug)
-      {
+    if (bt_debug != DebugLevel::kNone) {
+      if (bt_debug == DebugLevel::kFullDebug) {
         std::cout << "\nAfter optimization\n";
         std::cout << "------------------\n\n";
       }
       llvm.DumpIR();
     }
-    if (!args.output_llvm.empty())
-    {
+    if (!args.output_llvm.empty()) {
       llvm.DumpIR(args.output_llvm + ".optimized.ll");
     }
-    if (!args.output_elf.empty())
-    {
+    if (!args.output_elf.empty()) {
       llvm.emit_elf(args.output_elf);
       return 0;
     }
+    if (args.build_mode == BuildMode::AHEAD_OF_TIME) {
+      llvm::SmallVector<char, 0> aot_output;
+      llvm::raw_svector_ostream aot_os(aot_output);
+      llvm.emit(aot_os);
+
+      return aot::generate(
+          bpftrace.resources, args.aot, aot_output.data(), aot_output.size());
+    }
     bytecode = llvm.emit();
-  }
-  catch (const std::system_error& ex)
-  {
+  } catch (const std::system_error& ex) {
     LOG(ERROR) << "failed to write elf: " << ex.what();
     return 1;
-  }
-  catch (const std::exception& ex)
-  {
+  } catch (const std::exception& ex) {
     LOG(ERROR) << "Failed to compile: " << ex.what();
     return 1;
   }
@@ -1046,44 +948,5 @@ int main(int argc, char* argv[])
   if (bt_debug != DebugLevel::kNone || args.test_mode == TestMode::CODEGEN)
     return 0;
 
-  if (args.build_mode == BuildMode::AHEAD_OF_TIME)
-    return aot::generate(bpftrace.resources, bytecode, args.aot);
-
-  // Signal handler that lets us know an exit signal was received.
-  struct sigaction act = {};
-  act.sa_handler = [](int) { BPFtrace::exitsig_recv = true; };
-  sigaction(SIGINT, &act, NULL);
-  sigaction(SIGTERM, &act, NULL);
-
-  // Signal handler that prints all maps when SIGUSR1 was received.
-  act.sa_handler = [](int) { BPFtrace::sigusr1_recv = true; };
-  sigaction(SIGUSR1, &act, NULL);
-
-  err = bpftrace.run(std::move(bytecode));
-  if (err)
-    return err;
-
-  // We are now post-processing. If we receive another SIGINT,
-  // handle it normally (exit)
-  act.sa_handler = SIG_DFL;
-  sigaction(SIGINT, &act, NULL);
-
-  std::cout << "\n\n";
-
-  err = bpftrace.print_maps();
-
-  if (bt_verbose && bpftrace.child_)
-  {
-    auto val = 0;
-    if ((val = bpftrace.child_->term_signal()) > -1)
-      std::cout << "Child terminated by signal: " << val << std::endl;
-    if ((val = bpftrace.child_->exit_code()) > -1)
-      std::cout << "Child exited with code: " << val << std::endl;
-  }
-
-  if (err)
-    return err;
-
-  bpftrace.close_pcaps();
-  return 0;
+  return run_bpftrace(bpftrace, bytecode);
 }
