@@ -1,6 +1,6 @@
 #include <cstring>
 
-#include "async_event_types.h"
+#include "ast/async_event_types.h"
 #include "bpftrace.h"
 #include "log.h"
 #include "mapkey.h"
@@ -25,8 +25,7 @@ std::string MapKey::argument_type_list() const
 {
   std::ostringstream list;
   list << "[";
-  for (size_t i = 0; i < args_.size(); i++)
-  {
+  for (size_t i = 0; i < args_.size(); i++) {
     if (i)
       list << ", ";
     list << args_[i];
@@ -35,20 +34,21 @@ std::string MapKey::argument_type_list() const
   return list.str();
 }
 
-std::vector<std::string> MapKey::argument_value_list(BPFtrace &bpftrace,
+std::vector<std::string> MapKey::argument_value_list(
+    BPFtrace &bpftrace,
     const std::vector<uint8_t> &data) const
 {
   std::vector<std::string> list;
   int offset = 0;
-  for (const SizedType &arg : args_)
-  {
+  for (const SizedType &arg : args_) {
     list.push_back(argument_value(bpftrace, arg, &data[offset]));
     offset += arg.GetSize();
   }
   return list;
 }
 
-std::string MapKey::argument_value_list_str(BPFtrace &bpftrace,
+std::string MapKey::argument_value_list_str(
+    BPFtrace &bpftrace,
     const std::vector<uint8_t> &data) const
 {
   if (args_.empty())
@@ -57,16 +57,14 @@ std::string MapKey::argument_value_list_str(BPFtrace &bpftrace,
 }
 
 std::string MapKey::argument_value(BPFtrace &bpftrace,
-    const SizedType &arg,
-    const void *data)
+                                   const SizedType &arg,
+                                   const void *data)
 {
-  auto arg_data = static_cast<const uint8_t*>(data);
+  auto arg_data = static_cast<const uint8_t *>(data);
   std::ostringstream ptr;
-  switch (arg.type)
-  {
+  switch (arg.GetTy()) {
     case Type::integer:
-      switch (arg.GetSize())
-      {
+      switch (arg.GetSize()) {
         case 1:
           return std::to_string(read_data<int8_t>(data));
         case 2:
@@ -80,17 +78,22 @@ std::string MapKey::argument_value(BPFtrace &bpftrace,
       }
       break;
     case Type::kstack:
-      return bpftrace.get_stack(
-          read_data<int64_t>(data), -1, -1, false, arg.stack_type, 4);
+      return bpftrace.get_stack(read_data<int64_t>(data),
+                                read_data<uint32_t>(arg_data + 8),
+                                -1,
+                                -1,
+                                false,
+                                arg.stack_type,
+                                4);
     case Type::ustack:
       return bpftrace.get_stack(read_data<int64_t>(data),
-                                read_data<int32_t>(arg_data + 8),
+                                read_data<uint32_t>(arg_data + 8),
                                 read_data<int32_t>(arg_data + 12),
+                                read_data<int32_t>(arg_data + 16),
                                 true,
                                 arg.stack_type,
                                 4);
-    case Type::timestamp:
-    {
+    case Type::timestamp: {
       auto p = static_cast<const AsyncEvent::Strftime *>(data);
       return bpftrace.resolve_timestamp(p->mode, p->strftime_id, p->nsecs);
     }
@@ -107,24 +110,20 @@ std::string MapKey::argument_value(BPFtrace &bpftrace,
       return bpftrace.resolve_uid(read_data<uint64_t>(data));
     case Type::probe:
       return bpftrace.resources.probe_ids[read_data<uint64_t>(data)];
-    case Type::string:
-    {
+    case Type::string: {
       auto p = static_cast<const char *>(data);
       return std::string(p, strnlen(p, arg.GetSize()));
     }
-    case Type::buffer:
-    {
+    case Type::buffer: {
       auto p = static_cast<const char *>(data) + 1;
       return hex_format_buffer(p, arg.GetSize() - 1);
     }
-    case Type::pointer:
-    {
+    case Type::pointer: {
       // use case: show me these pointer values
       ptr << "0x" << std::hex << read_data<int64_t>(data);
       return ptr.str();
     }
-    case Type::array:
-    {
+    case Type::array: {
       std::vector<std::string> elems;
       for (size_t i = 0; i < arg.GetNumElements(); i++)
         elems.push_back(argument_value(bpftrace,
@@ -133,11 +132,9 @@ std::string MapKey::argument_value(BPFtrace &bpftrace,
                                            i * arg.GetElementTy()->GetSize()));
       return "[" + str_join(elems, ",") + "]";
     }
-    case Type::record:
-    {
+    case Type::record: {
       std::vector<std::string> elems;
-      for (auto &field : arg.GetFields())
-      {
+      for (auto &field : arg.GetFields()) {
         elems.push_back("." + field.name + "=" +
                         argument_value(bpftrace,
                                        field.type,
@@ -145,13 +142,12 @@ std::string MapKey::argument_value(BPFtrace &bpftrace,
       }
       return "{" + str_join(elems, ",") + "}";
     }
-    case Type::mac_address:
-    {
+    case Type::mac_address: {
       auto p = static_cast<const uint8_t *>(data);
       return bpftrace.resolve_mac_address(p);
     }
     default:
-      LOG(ERROR) << "invalid mapkey argument type";
+      LOG(BUG) << "invalid mapkey argument type";
   }
   abort();
 }
