@@ -1,8 +1,9 @@
 #include "output.h"
+
+#include "ast/async_event_types.h"
 #include "bpftrace.h"
 #include "log.h"
 #include "utils.h"
-#include <async_event_types.h>
 
 #include <bpf/libbpf.h>
 
@@ -17,8 +18,7 @@ namespace bpftrace {
 namespace {
 bool is_quoted_type(const SizedType &ty)
 {
-  switch (ty.type)
-  {
+  switch (ty.GetTy()) {
     case Type::buffer:
     case Type::cgroup_path:
     case Type::inet:
@@ -49,28 +49,51 @@ bool is_quoted_type(const SizedType &ty)
     case Type::timestamp:
     case Type::timestamp_mode:
     case Type::tuple:
+    case Type::voidtype:
       return false;
   }
   return false;
 }
 } // namespace
 
-std::ostream& operator<<(std::ostream& out, MessageType type) {
+std::ostream &operator<<(std::ostream &out, MessageType type)
+{
   switch (type) {
-    case MessageType::map: out << "map"; break;
+    case MessageType::map:
+      out << "map";
+      break;
     case MessageType::value:
       out << "value";
       break;
-    case MessageType::hist: out << "hist"; break;
-    case MessageType::stats: out << "stats"; break;
-    case MessageType::printf: out << "printf"; break;
-    case MessageType::time: out << "time"; break;
-    case MessageType::cat: out << "cat"; break;
-    case MessageType::join: out << "join"; break;
-    case MessageType::syscall: out << "syscall"; break;
-    case MessageType::attached_probes: out << "attached_probes"; break;
-    case MessageType::lost_events: out << "lost_events"; break;
-    default: out << "?";
+    case MessageType::hist:
+      out << "hist";
+      break;
+    case MessageType::stats:
+      out << "stats";
+      break;
+    case MessageType::printf:
+      out << "printf";
+      break;
+    case MessageType::time:
+      out << "time";
+      break;
+    case MessageType::cat:
+      out << "cat";
+      break;
+    case MessageType::join:
+      out << "join";
+      break;
+    case MessageType::syscall:
+      out << "syscall";
+      break;
+    case MessageType::attached_probes:
+      out << "attached_probes";
+      break;
+    case MessageType::lost_events:
+      out << "lost_events";
+      break;
+    default:
+      out << "?";
   }
   return out;
 }
@@ -104,41 +127,39 @@ std::string TextOutput::hist_index_label(uint32_t index, uint32_t k)
   return label.str();
 }
 
-std::string TextOutput::lhist_index_label(int number)
+std::string TextOutput::lhist_index_label(int number, int step)
 {
-  int kilo = 1024;
-  int mega = 1048576;
+  constexpr int kilo = 1024;
+  constexpr int mega = 1024 * 1024;
+
+  if (step % kilo != 0)
+    return std::to_string(number);
 
   std::ostringstream label;
 
-  if (number == 0)
-  {
+  if (number == 0) {
     label << number;
-  }
-  else if (number % mega == 0)
-  {
+  } else if (number % mega == 0) {
     label << number / mega << 'M';
-  }
-  else if (number % kilo == 0)
-  {
+  } else if (number % kilo == 0) {
     label << number / kilo << 'K';
-  }
-  else
-  {
+  } else {
     label << number;
   }
 
   return label.str();
 }
 
-void Output::hist_prepare(const std::vector<uint64_t> &values, int &min_index, int &max_index, int &max_value) const
+void Output::hist_prepare(const std::vector<uint64_t> &values,
+                          int &min_index,
+                          int &max_index,
+                          int &max_value) const
 {
   min_index = -1;
   max_index = -1;
   max_value = 0;
 
-  for (size_t i = 0; i < values.size(); i++)
-  {
+  for (size_t i = 0; i < values.size(); i++) {
     int v = values.at(i);
     if (v > 0) {
       if (min_index == -1)
@@ -150,14 +171,21 @@ void Output::hist_prepare(const std::vector<uint64_t> &values, int &min_index, i
   }
 }
 
-void Output::lhist_prepare(const std::vector<uint64_t> &values, int min, int max, int step, int &max_index, int &max_value, int &buckets, int &start_value, int &end_value) const
+void Output::lhist_prepare(const std::vector<uint64_t> &values,
+                           int min,
+                           int max,
+                           int step,
+                           int &max_index,
+                           int &max_value,
+                           int &buckets,
+                           int &start_value,
+                           int &end_value) const
 {
   max_index = -1;
   max_value = 0;
   buckets = (max - min) / step; // excluding lt and gt buckets
 
-  for (size_t i = 0; i < values.size(); i++)
-  {
+  for (size_t i = 0; i < values.size(); i++) {
     int v = values.at(i);
     if (v != 0)
       max_index = i;
@@ -172,8 +200,7 @@ void Output::lhist_prepare(const std::vector<uint64_t> &values, int min, int max
   start_value = -1;
   end_value = 0;
 
-  for (unsigned int i = 0; i <= static_cast<unsigned int>(buckets) + 1; i++)
-  {
+  for (unsigned int i = 0; i <= static_cast<unsigned int>(buckets) + 1; i++) {
     if (values.at(i) > 0) {
       if (start_value == -1) {
         start_value = i;
@@ -190,22 +217,17 @@ void Output::lhist_prepare(const std::vector<uint64_t> &values, int min, int max
 std::string Output::get_helper_error_msg(int func_id, int retcode) const
 {
   std::string msg;
-  if (func_id == libbpf::BPF_FUNC_map_update_elem && retcode == -E2BIG)
-  {
+  if (func_id == libbpf::BPF_FUNC_map_update_elem && retcode == -E2BIG) {
     msg = "Map full; can't update element. Try increasing max_map_keys config";
-  }
-  else if (func_id == libbpf::BPF_FUNC_map_delete_elem && retcode == -ENOENT)
-  {
+  } else if (func_id == libbpf::BPF_FUNC_map_delete_elem &&
+             retcode == -ENOENT) {
     msg = "Can't delete map element because it does not exist.";
   }
   // bpftrace sets the return code to 0 for map_lookup_elem failures
   // which is why we're not also checking the retcode
-  else if (func_id == libbpf::BPF_FUNC_map_lookup_elem)
-  {
+  else if (func_id == libbpf::BPF_FUNC_map_lookup_elem) {
     msg = "Can't lookup map element because it does not exist.";
-  }
-  else
-  {
+  } else {
     msg = strerror(-retcode);
   }
   return msg;
@@ -219,12 +241,18 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
 {
   uint32_t nvalues = is_per_cpu ? bpftrace.ncpus_ : 1;
   if (type.IsKstackTy())
-    return bpftrace.get_stack(
-        read_data<uint64_t>(value.data()), -1, -1, false, type.stack_type, 8);
+    return bpftrace.get_stack(read_data<uint64_t>(value.data()),
+                              read_data<uint32_t>(value.data() + 8),
+                              -1,
+                              -1,
+                              false,
+                              type.stack_type,
+                              8);
   else if (type.IsUstackTy())
     return bpftrace.get_stack(read_data<int64_t>(value.data()),
-                              read_data<int32_t>(value.data() + 8),
+                              read_data<uint32_t>(value.data() + 8),
                               read_data<int32_t>(value.data() + 12),
+                              read_data<int32_t>(value.data() + 16),
                               true,
                               type.stack_type,
                               8);
@@ -242,29 +270,22 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
   else if (type.IsBufferTy())
     return bpftrace.resolve_buf(reinterpret_cast<char *>(value.data() + 1),
                                 *reinterpret_cast<uint8_t *>(value.data()));
-  else if (type.IsStringTy())
-  {
+  else if (type.IsStringTy()) {
     auto p = reinterpret_cast<const char *>(value.data());
     return std::string(p, strnlen(p, type.GetSize()));
-  }
-  else if (type.IsArrayTy())
-  {
+  } else if (type.IsArrayTy()) {
     size_t elem_size = type.GetElementTy()->GetSize();
     std::vector<std::string> elems;
-    for (size_t i = 0; i < type.GetNumElements(); i++)
-    {
+    for (size_t i = 0; i < type.GetNumElements(); i++) {
       std::vector<uint8_t> elem_data(value.begin() + i * elem_size,
                                      value.begin() + (i + 1) * elem_size);
       elems.push_back(value_to_str(
           bpftrace, *type.GetElementTy(), elem_data, is_per_cpu, div));
     }
     return array_to_str(elems);
-  }
-  else if (type.IsRecordTy())
-  {
+  } else if (type.IsRecordTy()) {
     std::vector<std::string> elems;
-    for (auto &field : type.GetFields())
-    {
+    for (auto &field : type.GetFields()) {
       std::vector<uint8_t> elem_data(value.begin() + field.offset,
                                      value.begin() + field.offset +
                                          field.type.GetSize());
@@ -273,12 +294,9 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
           value_to_str(bpftrace, field.type, elem_data, is_per_cpu, div)));
     }
     return struct_to_str(elems);
-  }
-  else if (type.IsTupleTy())
-  {
+  } else if (type.IsTupleTy()) {
     std::vector<std::string> elems;
-    for (auto &field : type.GetFields())
-    {
+    for (auto &field : type.GetFields()) {
       std::vector<uint8_t> elem_data(value.begin() + field.offset,
                                      value.begin() + field.offset +
                                          field.type.GetSize());
@@ -286,14 +304,11 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
           value_to_str(bpftrace, field.type, elem_data, is_per_cpu, div));
     }
     return tuple_to_str(elems);
-  }
-  else if (type.IsCountTy())
+  } else if (type.IsCountTy())
     return std::to_string(reduce_value<uint64_t>(value, nvalues) / div);
-  else if (type.IsIntTy())
-  {
+  else if (type.IsIntTy()) {
     auto sign = type.IsSigned();
-    switch (type.GetIntBitWidth())
-    {
+    switch (type.GetIntBitWidth()) {
       // clang-format off
       case 64:
         if (sign)
@@ -316,24 +331,29 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
         return std::to_string(reduce_value<uint8_t>(value, nvalues) / div);
         // clang-format on
       default:
-        LOG(FATAL) << "value_to_str: Invalid int bitwidth: "
-                   << type.GetIntBitWidth() << "provided";
+        LOG(BUG) << "value_to_str: Invalid int bitwidth: "
+                 << type.GetIntBitWidth() << "provided";
         return {};
     }
-    // lgtm[cpp/missing-return]
-  }
-  else if (type.IsSumTy() || type.IsIntTy())
-  {
+  } else if (type.IsSumTy() || type.IsIntTy()) {
     if (type.IsSigned())
       return std::to_string(reduce_value<int64_t>(value, nvalues) / div);
 
     return std::to_string(reduce_value<uint64_t>(value, nvalues) / div);
-  }
-  else if (type.IsMinTy())
-    return std::to_string(min_value(value, nvalues) / div);
-  else if (type.IsMaxTy())
-    return std::to_string(max_value(value, nvalues) / div);
-  else if (type.IsProbeTy())
+  } else if (type.IsMinTy() || type.IsMaxTy()) {
+    if (is_per_cpu) {
+      if (type.IsSigned()) {
+        return std::to_string(
+            min_max_value<int64_t>(value, nvalues, type.IsMaxTy()) / div);
+      }
+      return std::to_string(
+          min_max_value<uint64_t>(value, nvalues, type.IsMaxTy()) / div);
+    }
+    if (type.IsSigned()) {
+      return std::to_string(read_data<int64_t>(value.data()) / div);
+    }
+    return std::to_string(read_data<uint64_t>(value.data()) / div);
+  } else if (type.IsProbeTy())
     return bpftrace.resolve_probe(read_data<uint64_t>(value.data()));
   else if (type.IsTimestampTy())
     return bpftrace.resolve_timestamp(
@@ -349,14 +369,11 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
         reinterpret_cast<AsyncEvent::CgroupPath *>(value.data())->cgroup_id);
   else if (type.IsStrerrorTy())
     return strerror(read_data<uint64_t>(value.data()));
-  else if (type.IsPtrTy())
-  {
+  else if (type.IsPtrTy()) {
     std::ostringstream res;
     res << "0x" << std::hex << read_data<uint64_t>(value.data());
     return res.str();
-  }
-  else
-  {
+  } else {
     assert(type.IsNoneTy());
     return "";
   }
@@ -374,7 +391,7 @@ std::string Output::struct_to_str(const std::vector<std::string> &elems) const
 
 std::string Output::map_to_str(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>>
@@ -383,29 +400,28 @@ std::string Output::map_to_str(
   std::vector<std::string> elems;
   uint32_t i = 0;
   size_t total = values_by_key.size();
-  for (auto &pair : values_by_key)
-  {
+  const auto &map_type = bpftrace.resources.maps_info.at(map.name()).value_type;
+  for (auto &pair : values_by_key) {
     auto key = pair.first;
     auto value = pair.second;
 
-    if (top)
-    {
+    if (top) {
       if (total > top && i++ < (total - top))
         continue;
     }
 
     auto key_str = map_key_to_str(bpftrace, map, key);
     auto value_str = value_to_str(
-        bpftrace, map.type_, value, map.is_per_cpu_type(), div);
-    elems.push_back(map_keyval_to_str(map, key_str, value_str));
+        bpftrace, map_type, value, map.is_per_cpu_type(), div);
+    elems.push_back(map_keyval_to_str(map_type, key_str, value_str));
   }
 
-  return str_join(elems, map_elem_delim_to_str(map));
+  return str_join(elems, map_elem_delim_to_str(map_type));
 }
 
 std::string Output::map_hist_to_str(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::map<std::vector<uint8_t>, std::vector<uint64_t>> &values_by_key,
@@ -414,8 +430,9 @@ std::string Output::map_hist_to_str(
 {
   std::vector<std::string> elems;
   uint32_t i = 0;
-  for (auto &key_count : total_counts_by_key)
-  {
+  const auto &map_info = bpftrace.resources.maps_info.at(map.name());
+  const auto &map_type = map_info.value_type;
+  for (auto &key_count : total_counts_by_key) {
     auto &key = key_count.first;
     auto &value = values_by_key.at(key);
 
@@ -423,18 +440,22 @@ std::string Output::map_hist_to_str(
       continue;
 
     auto key_str = map_key_to_str(bpftrace, map, key);
-    auto val_str = map.type_.IsHistTy()
-                       ? hist_to_str(value, div, map.bits())
-                       : lhist_to_str(value, map.lqmin, map.lqmax, map.lqstep);
+    std::string val_str;
+    if (map_type.IsHistTy()) {
+      val_str = hist_to_str(value, div, *map_info.hist_bits_arg);
+    } else {
+      auto &args = map_info.lhist_args;
+      val_str = lhist_to_str(value, args->min, args->max, args->step);
+    }
 
-    elems.push_back(map_keyval_to_str(map, key_str, val_str));
+    elems.push_back(map_keyval_to_str(map_type, key_str, val_str));
   }
-  return str_join(elems, map_elem_delim_to_str(map));
+  return str_join(elems, map_elem_delim_to_str(map_type));
 }
 
 std::string Output::map_stats_to_str(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::map<std::vector<uint8_t>, std::vector<int64_t>> &values_by_key,
@@ -442,13 +463,13 @@ std::string Output::map_stats_to_str(
         &total_counts_by_key) const
 {
   std::vector<std::string> elems;
+  const auto &map_type = bpftrace.resources.maps_info.at(map.name()).value_type;
   uint32_t i = 0;
-  for (auto &key_count : total_counts_by_key)
-  {
+  for (auto &key_count : total_counts_by_key) {
     auto &key = key_count.first;
     auto &value = values_by_key.at(key);
 
-    if (map.type_.IsAvgTy() && top && values_by_key.size() > top &&
+    if (map_type.IsAvgTy() && top && values_by_key.size() > top &&
         i++ < (values_by_key.size() - top))
       continue;
 
@@ -462,25 +483,23 @@ std::string Output::map_stats_to_str(
       average = total / count;
 
     std::string value_str;
-    if (map.type_.IsStatsTy())
-    {
+    if (map_type.IsStatsTy()) {
       std::vector<std::pair<std::string, int64_t>> stats = {
         { "count", count }, { "average", average }, { "total", total }
       };
       value_str = key_value_pairs_to_str(stats);
-    }
-    else
+    } else
       value_str = std::to_string(average / div);
 
-    elems.push_back(map_keyval_to_str(map, key_str, value_str));
+    elems.push_back(map_keyval_to_str(map_type, key_str, value_str));
   }
 
-  return str_join(elems, map_elem_delim_to_str(map));
+  return str_join(elems, map_elem_delim_to_str(map_type));
 }
 
 void TextOutput::map(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>>
@@ -500,8 +519,7 @@ std::string TextOutput::hist_to_str(const std::vector<uint64_t> &values,
     return "";
 
   std::ostringstream res;
-  for (int i = min_index; i <= max_index; i++)
-  {
+  for (int i = min_index; i <= max_index; i++) {
     std::ostringstream header;
 
     // Index 0 is for negative values. Following that, each sequence
@@ -517,23 +535,18 @@ std::string TextOutput::hist_to_str(const std::vector<uint64_t> &values,
     // Higher indexes contain multiple values and we use helpers to print
     // the range as open intervals.
 
-    if (i == 0)
-    {
+    if (i == 0) {
       header << "(..., 0)";
-    }
-    else if (i <= (2 << k))
-    {
+    } else if (i <= (2 << k)) {
       header << "[" << (i - 1) << "]";
-    }
-    else
-    {
+    } else {
       // Use a helper function to print the interval boundaries.
       header << "[" << hist_index_label(i - 1, k);
       header << ", " << hist_index_label(i, k) << ")";
     }
 
     int max_width = 52;
-    int bar_width = values.at(i)/(float)max_value*max_width;
+    int bar_width = values.at(i) / (float)max_value * max_width;
     std::string bar(bar_width, '@');
 
     res << std::setw(16) << std::left << header.str() << std::setw(8)
@@ -549,23 +562,30 @@ std::string TextOutput::lhist_to_str(const std::vector<uint64_t> &values,
                                      int step) const
 {
   int max_index, max_value, buckets, start_value, end_value;
-  lhist_prepare(values, min, max, step, max_index, max_value, buckets, start_value, end_value);
+  lhist_prepare(values,
+                min,
+                max,
+                step,
+                max_index,
+                max_value,
+                buckets,
+                start_value,
+                end_value);
   if (max_index == -1)
     return "";
 
   std::ostringstream res;
-  for (int i = start_value; i <= end_value; i++)
-  {
+  for (int i = start_value; i <= end_value; i++) {
     int max_width = 52;
-    int bar_width = values.at(i)/(float)max_value*max_width;
+    int bar_width = values.at(i) / (float)max_value * max_width;
     std::ostringstream header;
     if (i == 0) {
-      header << "(..., " << lhist_index_label(min) << ")";
+      header << "(..., " << lhist_index_label(min, step) << ")";
     } else if (i == (buckets + 1)) {
-      header << "[" << lhist_index_label(max) << ", ...)";
+      header << "[" << lhist_index_label(max, step) << ", ...)";
     } else {
-      header << "[" << lhist_index_label((i - 1) * step + min);
-      header << ", " << lhist_index_label(i * step + min) << ")";
+      header << "[" << lhist_index_label((i - 1) * step + min, step);
+      header << ", " << lhist_index_label(i * step + min, step) << ")";
     }
 
     std::string bar(bar_width, '@');
@@ -579,7 +599,7 @@ std::string TextOutput::lhist_to_str(const std::vector<uint64_t> &values,
 
 void TextOutput::map_hist(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::map<std::vector<uint8_t>, std::vector<uint64_t>> &values_by_key,
@@ -593,7 +613,7 @@ void TextOutput::map_hist(
 
 void TextOutput::map_stats(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::map<std::vector<uint8_t>, std::vector<int64_t>> &values_by_key,
@@ -612,7 +632,9 @@ void TextOutput::value(BPFtrace &bpftrace,
   out_ << value_to_str(bpftrace, ty, value, false, 1) << std::endl;
 }
 
-void TextOutput::message(MessageType type __attribute__((unused)), const std::string& msg, bool nl) const
+void TextOutput::message(MessageType type __attribute__((unused)),
+                         const std::string &msg,
+                         bool nl) const
 {
   out_ << msg;
   if (nl)
@@ -655,28 +677,28 @@ std::string TextOutput::tuple_to_str(
 }
 
 std::string TextOutput::map_key_to_str(BPFtrace &bpftrace,
-                                       IMap &map,
+                                       const BpfMap &map,
                                        const std::vector<uint8_t> &key) const
 {
-  return map.name_ + map.key_.argument_value_list_str(bpftrace, key);
+  const auto &map_key = bpftrace.resources.maps_info.at(map.name()).key;
+  return map.name() + map_key.argument_value_list_str(bpftrace, key);
 }
 
-std::string TextOutput::map_keyval_to_str(IMap &map,
+std::string TextOutput::map_keyval_to_str(const SizedType &map_type,
                                           const std::string &key,
                                           const std::string &val) const
 {
   std::string res = key + ": ";
-  if (map.type_.IsHistTy() || map.type_.IsLhistTy())
+  if (map_type.IsHistTy() || map_type.IsLhistTy())
     res += "\n";
   res += val;
   return res;
 }
 
-std::string TextOutput::map_elem_delim_to_str(IMap &map) const
+std::string TextOutput::map_elem_delim_to_str(const SizedType &map_type) const
 {
-  if (map.type_.type != Type::kstack && map.type_.type != Type::ustack &&
-      map.type_.type != Type::ksym && map.type_.type != Type::usym &&
-      map.type_.type != Type::inet)
+  if (!map_type.IsKstackTy() && !map_type.IsUstackTy() &&
+      !map_type.IsKsymTy() && !map_type.IsUsymTy() && !map_type.IsInetTy())
     return "\n";
 
   return "";
@@ -694,10 +716,8 @@ std::string TextOutput::key_value_pairs_to_str(
 std::string JsonOutput::json_escape(const std::string &str) const
 {
   std::ostringstream escaped;
-  for (const char &c : str)
-  {
-    switch (c)
-    {
+  for (const char &c : str) {
+    switch (c) {
       case '"':
         escaped << "\\\"";
         break;
@@ -720,12 +740,10 @@ std::string JsonOutput::json_escape(const std::string &str) const
 
       default:
         // c always >= '\x00'
-        if (c <= '\x1f')
-        {
-          escaped << "\\u" << std::hex << std::setw(4) << std::setfill('0') << (int)c;
-        }
-        else
-        {
+        if (c <= '\x1f') {
+          escaped << "\\u" << std::hex << std::setw(4) << std::setfill('0')
+                  << (int)c;
+        } else {
           escaped << c;
         }
     }
@@ -735,7 +753,7 @@ std::string JsonOutput::json_escape(const std::string &str) const
 
 void JsonOutput::map(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>>
@@ -744,14 +762,16 @@ void JsonOutput::map(
   if (values_by_key.empty())
     return;
 
+  const auto &map_key = bpftrace.resources.maps_info.at(map.name()).key;
+
   out_ << "{\"type\": \"" << MessageType::map << "\", \"data\": {";
-  out_ << "\"" << json_escape(map.name_) << "\": ";
-  if (map.key_.size() > 0) // check if this map has keys
+  out_ << "\"" << json_escape(map.name()) << "\": ";
+  if (map_key.size() > 0) // check if this map has keys
     out_ << "{";
 
   out_ << map_to_str(bpftrace, map, top, div, values_by_key);
 
-  if (map.key_.size() > 0)
+  if (map_key.size() > 0)
     out_ << "}";
   out_ << "}}" << std::endl;
 }
@@ -767,8 +787,7 @@ std::string JsonOutput::hist_to_str(const std::vector<uint64_t> &values,
 
   std::ostringstream res;
   res << "[";
-  for (int i = min_index; i <= max_index; i++)
-  {
+  for (int i = min_index; i <= max_index; i++) {
     if (i > min_index)
       res << ", ";
 
@@ -778,16 +797,11 @@ std::string JsonOutput::hist_to_str(const std::vector<uint64_t> &values,
     // N = 2^k indexes have one value each (equal to i -1)
     // and remaining sets of N indexes each cover one power of 2,
     // whose ranges are computed as described in hist_index_label()
-    if (i == 0)
-    {
+    if (i == 0) {
       res << "\"max\": -1, ";
-    }
-    else if (i <= (2 << k))
-    {
+    } else if (i <= (2 << k)) {
       res << "\"min\": " << i - 1 << ", \"max\": " << i - 1 << ", ";
-    }
-    else
-    {
+    } else {
       const uint32_t n = 1 << k;
       uint32_t power = ((i - 1) >> k) - 1, bucket = (i - 1) & (n - 1);
       const long low = (1ULL << power) * (n + bucket);
@@ -810,14 +824,21 @@ std::string JsonOutput::lhist_to_str(const std::vector<uint64_t> &values,
                                      int step) const
 {
   int max_index, max_value, buckets, start_value, end_value;
-  lhist_prepare(values, min, max, step, max_index, max_value, buckets, start_value, end_value);
+  lhist_prepare(values,
+                min,
+                max,
+                step,
+                max_index,
+                max_value,
+                buckets,
+                start_value,
+                end_value);
   if (max_index == -1)
     return "[]";
 
   std::ostringstream res;
   res << "[";
-  for (int i = start_value; i <= end_value; i++)
-  {
+  for (int i = start_value; i <= end_value; i++) {
     if (i > start_value)
       res << ", ";
 
@@ -841,7 +862,7 @@ std::string JsonOutput::lhist_to_str(const std::vector<uint64_t> &values,
 
 void JsonOutput::map_hist(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::map<std::vector<uint8_t>, std::vector<uint64_t>> &values_by_key,
@@ -851,22 +872,24 @@ void JsonOutput::map_hist(
   if (total_counts_by_key.empty())
     return;
 
+  const auto &map_key = bpftrace.resources.maps_info.at(map.name()).key;
+
   out_ << "{\"type\": \"" << MessageType::hist << "\", \"data\": {";
-  out_ << "\"" << json_escape(map.name_) << "\": ";
-  if (map.key_.size() > 0) // check if this map has keys
+  out_ << "\"" << json_escape(map.name()) << "\": ";
+  if (map_key.size() > 0) // check if this map has keys
     out_ << "{";
 
   out_ << map_hist_to_str(
       bpftrace, map, top, div, values_by_key, total_counts_by_key);
 
-  if (map.key_.size() > 0)
+  if (map_key.size() > 0)
     out_ << "}";
   out_ << "}}" << std::endl;
 }
 
 void JsonOutput::map_stats(
     BPFtrace &bpftrace,
-    IMap &map,
+    const BpfMap &map,
     uint32_t top,
     uint32_t div,
     const std::map<std::vector<uint8_t>, std::vector<int64_t>> &values_by_key,
@@ -876,15 +899,17 @@ void JsonOutput::map_stats(
   if (total_counts_by_key.empty())
     return;
 
+  const auto &map_key = bpftrace.resources.maps_info.at(map.name()).key;
+
   out_ << "{\"type\": \"" << MessageType::stats << "\", \"data\": {";
-  out_ << "\"" << json_escape(map.name_) << "\": ";
-  if (map.key_.size() > 0) // check if this map has keys
+  out_ << "\"" << json_escape(map.name()) << "\": ";
+  if (map_key.size() > 0) // check if this map has keys
     out_ << "{";
 
   out_ << map_stats_to_str(
       bpftrace, map, top, div, values_by_key, total_counts_by_key);
 
-  if (map.key_.size() > 0)
+  if (map_key.size() > 0)
     out_ << "}";
   out_ << "}}" << std::endl;
 }
@@ -898,15 +923,21 @@ void JsonOutput::value(BPFtrace &bpftrace,
        << std::endl;
 }
 
-void JsonOutput::message(MessageType type, const std::string& msg, bool nl __attribute__((unused))) const
+void JsonOutput::message(MessageType type,
+                         const std::string &msg,
+                         bool nl __attribute__((unused))) const
 {
-  out_ << "{\"type\": \"" << type << "\", \"data\": \"" << json_escape(msg) << "\"}" << std::endl;
+  out_ << "{\"type\": \"" << type << "\", \"data\": \"" << json_escape(msg)
+       << "\"}" << std::endl;
 }
 
-void JsonOutput::message(MessageType type, const std::string& field, uint64_t value) const
+void JsonOutput::message(MessageType type,
+                         const std::string &field,
+                         uint64_t value) const
 {
-  out_ << "{\"type\": \"" << type << "\", \"data\": " <<  "{\"" << field
-       << "\": " << value << "}" << "}" << std::endl;
+  out_ << "{\"type\": \"" << type << "\", \"data\": "
+       << "{\"" << field << "\": " << value << "}"
+       << "}" << std::endl;
 }
 
 void JsonOutput::lost_events(uint64_t lost) const
@@ -956,25 +987,26 @@ std::string JsonOutput::value_to_str(BPFtrace &bpftrace,
 }
 
 std::string JsonOutput::map_key_to_str(BPFtrace &bpftrace,
-                                       IMap &map,
+                                       const BpfMap &map,
                                        const std::vector<uint8_t> &key) const
 {
-  std::vector<std::string> args = map.key_.argument_value_list(bpftrace, key);
-  if (!args.empty())
-  {
+  const auto &map_key = bpftrace.resources.maps_info.at(map.name()).key;
+  std::vector<std::string> args = map_key.argument_value_list(bpftrace, key);
+  if (!args.empty()) {
     return "\"" + json_escape(str_join(args, ",")) + "\"";
   }
   return "";
 }
 
-std::string JsonOutput::map_keyval_to_str(IMap &map __attribute__((unused)),
+std::string JsonOutput::map_keyval_to_str(const SizedType &map_type
+                                          __attribute__((unused)),
                                           const std::string &key,
                                           const std::string &val) const
 {
   return key.empty() ? val : key + ": " + val;
 }
 
-std::string JsonOutput::map_elem_delim_to_str(IMap &map
+std::string JsonOutput::map_elem_delim_to_str(const SizedType &map
                                               __attribute__((unused))) const
 {
   return ", ";
