@@ -16,9 +16,8 @@ std::set<std::string> TracepointFormatParser::struct_list;
 
 bool TracepointFormatParser::parse(ast::Program *program, BPFtrace &bpftrace)
 {
-  std::vector<ast::Probe*> probes_with_tracepoint;
-  for (ast::Probe *probe : *program->probes)
-  {
+  std::vector<ast::Probe *> probes_with_tracepoint;
+  for (ast::Probe *probe : *program->probes) {
     if (probe->has_ap_of_probetype(ProbeType::tracepoint))
       probes_with_tracepoint.push_back(probe);
   }
@@ -29,50 +28,39 @@ bool TracepointFormatParser::parse(ast::Program *program, BPFtrace &bpftrace)
   ast::TracepointArgsVisitor n{};
   if (!bpftrace.has_btf_data())
     program->c_definitions += "#include <linux/types.h>\n";
-  for (ast::Probe *probe : probes_with_tracepoint)
-  {
+  for (ast::Probe *probe : probes_with_tracepoint) {
     n.visit(*probe);
 
-    for (ast::AttachPoint *ap : *probe->attach_points)
-    {
-      if (ap->provider == "tracepoint")
-      {
+    for (ast::AttachPoint *ap : *probe->attach_points) {
+      if (ap->provider == "tracepoint") {
         std::string &category = ap->target;
         std::string &event_name = ap->func;
         std::string format_file_path = tracefs::event_format_file(category,
                                                                   event_name);
         glob_t glob_result;
 
-        if (has_wildcard(category) || has_wildcard(event_name))
-        {
+        if (has_wildcard(category) || has_wildcard(event_name)) {
           // tracepoint wildcard expansion, part 1 of 3. struct definitions.
           memset(&glob_result, 0, sizeof(glob_result));
           int ret = glob(format_file_path.c_str(), 0, NULL, &glob_result);
-          if (ret != 0)
-          {
-            if (ret == GLOB_NOMATCH)
-            {
+          if (ret != 0) {
+            if (ret == GLOB_NOMATCH) {
               LOG(ERROR, ap->loc, std::cerr)
                   << "tracepoints not found: " << category << ":" << event_name;
               // helper message:
               if (category == "syscall")
                 LOG(ERROR, ap->loc, std::cerr)
                     << "Did you mean syscalls:" << event_name << "?";
-              if (bt_verbose) {
-                LOG(ERROR) << strerror(errno) << ": " << format_file_path;
-              }
+              LOG(V1) << strerror(errno) << ": " << format_file_path;
               return false;
-            }
-            else
-            {
+            } else {
               // unexpected error
               LOG(ERROR, ap->loc, std::cerr) << std::string(strerror(errno));
               return false;
             }
           }
 
-          if (probe->tp_args_structs_level <= 0)
-          {
+          if (probe->tp_args_structs_level <= 0) {
             globfree(&glob_result);
             continue;
           }
@@ -88,24 +76,21 @@ bool TracepointFormatParser::parse(ast::Program *program, BPFtrace &bpftrace)
             std::string real_event = filename.substr(
                 pos, filename.length() - std::string("/format").length() - pos);
 
-            // Check to avoid adding the same struct more than once to definitions
+            // Check to avoid adding the same struct more than once to
+            // definitions
             std::string struct_name = get_struct_name(real_category,
                                                       real_event);
-            if (!TracepointFormatParser::struct_list.count(struct_name))
-            {
+            if (!TracepointFormatParser::struct_list.count(struct_name)) {
               program->c_definitions += get_tracepoint_struct(
                   format_file, real_category, real_event, bpftrace);
               TracepointFormatParser::struct_list.insert(struct_name);
             }
           }
           globfree(&glob_result);
-        }
-        else
-        {
+        } else {
           // single tracepoint
           std::ifstream format_file(format_file_path.c_str());
-          if (format_file.fail())
-          {
+          if (format_file.fail()) {
             // Errno might get clobbered by LOG().
             int saved_errno = errno;
 
@@ -123,8 +108,7 @@ bool TracepointFormatParser::parse(ast::Program *program, BPFtrace &bpftrace)
               LOG(WARNING, ap->loc, std::cerr)
                   << "Did you mean syscalls:" << event_name << "?";
 
-            if (fail && bt_verbose)
-            {
+            if (fail && bt_verbose) {
               // Having the location info isn't really useful here, so no
               // bpftrace.error
               LOG(ERROR) << strerror(saved_errno) << ": " << format_file_path;
@@ -150,7 +134,9 @@ bool TracepointFormatParser::parse(ast::Program *program, BPFtrace &bpftrace)
   return true;
 }
 
-std::string TracepointFormatParser::get_struct_name(const std::string &category, const std::string &event_name)
+std::string TracepointFormatParser::get_struct_name(
+    const std::string &category,
+    const std::string &event_name)
 {
   return "struct _tracepoint_" + category + "_" + event_name;
 }
@@ -199,27 +185,25 @@ std::string TracepointFormatParser::parse_field(const std::string &line,
 
   // If there'a gap between last field and this one,
   // generate padding fields
-  if (offset && *last_offset)
-  {
+  if (offset && *last_offset) {
     int i, gap = offset - *last_offset;
 
-    for (i = 0; i < gap; i++)
-    {
+    for (i = 0; i < gap; i++) {
       extra += "  char __pad_" + std::to_string(offset - gap + i) + ";\n";
     }
   }
 
   *last_offset = offset + size;
 
-  std::string field = line.substr(field_pos + 6, field_semi_pos - field_pos - 6);
+  std::string field = line.substr(field_pos + 6,
+                                  field_semi_pos - field_pos - 6);
   auto field_type_end_pos = field.find_last_of("\t ");
   if (field_type_end_pos == std::string::npos)
     return "";
   std::string field_type = field.substr(0, field_type_end_pos);
-  std::string field_name = field.substr(field_type_end_pos+1);
+  std::string field_name = field.substr(field_type_end_pos + 1);
 
-  if (field_type.find("__data_loc") != std::string::npos)
-  {
+  if (field_type.find("__data_loc") != std::string::npos) {
     // Note that the type here (ie `int`) does not matter. Later during parse
     // time the parser will rewrite this field type to a u64 so that it can
     // hold the pointer to the actual location of the data.
@@ -236,8 +220,7 @@ std::string TracepointFormatParser::parse_field(const std::string &line,
   // <linux/types.h> and request all the types we need from BTF.
   bpftrace.btf_set_.emplace(field_type);
 
-  if (arr_size_pos != std::string::npos)
-  {
+  if (arr_size_pos != std::string::npos) {
     auto arr_size = field_name.substr(arr_size_pos + 1,
                                       arr_size_end_pos - arr_size_pos - 1);
     if (arr_size.find_first_not_of("0123456789") != std::string::npos)
@@ -247,17 +230,18 @@ std::string TracepointFormatParser::parse_field(const std::string &line,
   return extra + "  " + field_type + " " + field_name + ";\n";
 }
 
-std::string TracepointFormatParser::adjust_integer_types(const std::string &field_type, int size)
+std::string TracepointFormatParser::adjust_integer_types(
+    const std::string &field_type,
+    int size)
 {
   std::string new_type = field_type;
   // Adjust integer fields to correctly sized types
-  if (size == 8)
-  {
+  if (size == 8) {
     if (field_type == "int")
       new_type = "s64";
     if (field_type == "unsigned int" || field_type == "unsigned" ||
-        field_type == "u32" || field_type == "pid_t" ||
-        field_type == "uid_t" || field_type == "gid_t")
+        field_type == "u32" || field_type == "pid_t" || field_type == "uid_t" ||
+        field_type == "gid_t")
       new_type = "u64";
   }
 
@@ -273,8 +257,7 @@ std::string TracepointFormatParser::get_tracepoint_struct(
   std::string format_struct = get_struct_name(category, event_name) + "\n{\n";
   int last_offset = 0;
 
-  for (std::string line; getline(format_file, line); )
-  {
+  for (std::string line; getline(format_file, line);) {
     format_struct += parse_field(line, &last_offset, bpftrace);
   }
 
