@@ -42,7 +42,7 @@ namespace {
 
 std::vector<int> read_cpu_range(std::string path)
 {
-  std::ifstream cpus_range_stream { path };
+  std::ifstream cpus_range_stream{ path };
   std::vector<int> cpus;
   std::string cpu_range;
 
@@ -50,8 +50,7 @@ std::vector<int> read_cpu_range(std::string path)
     std::size_t rangeop = cpu_range.find('-');
     if (rangeop == std::string::npos) {
       cpus.push_back(std::stoi(cpu_range));
-    }
-    else {
+    } else {
       int start = std::stoi(cpu_range.substr(0, rangeop));
       int end = std::stoi(cpu_range.substr(rangeop + 1));
       for (int i = start; i <= end; i++)
@@ -61,14 +60,14 @@ std::vector<int> read_cpu_range(std::string path)
   return cpus;
 }
 
-std::vector<std::string> expand_wildcard_path(const std::string& path)
+std::vector<std::string> expand_wildcard_path(const std::string &path)
 {
   glob_t glob_result;
   memset(&glob_result, 0, sizeof(glob_result));
 
   if (glob(path.c_str(), GLOB_NOCHECK, nullptr, &glob_result)) {
     globfree(&glob_result);
-    throw std::runtime_error("glob() failed");
+    throw bpftrace::FatalUserException("glob() failed");
   }
 
   std::vector<std::string> matching_paths;
@@ -80,11 +79,11 @@ std::vector<std::string> expand_wildcard_path(const std::string& path)
   return matching_paths;
 }
 
-std::vector<std::string> expand_wildcard_paths(const std::vector<std::string>& paths)
+std::vector<std::string> expand_wildcard_paths(
+    const std::vector<std::string> &paths)
 {
   std::vector<std::string> expanded_paths;
-  for (const auto& p : paths)
-  {
+  for (const auto &p : paths) {
     auto ep = expand_wildcard_path(p);
     expanded_paths.insert(expanded_paths.end(), ep.begin(), ep.end());
   }
@@ -109,11 +108,10 @@ const struct vmlinux_location vmlinux_locs[] = {
   { nullptr, false },
 };
 
-constexpr std::string_view PROC_KHEADERS_PATH = "/sys/kernel/kheaders.tar.xz";
-
 static bool pid_in_different_mountns(int pid);
-static std::vector<std::string>
-resolve_binary_path(const std::string &cmd, const char *env_paths, int pid);
+static std::vector<std::string> resolve_binary_path(const std::string &cmd,
+                                                    const char *env_paths,
+                                                    int pid);
 
 void StdioSilencer::silence()
 {
@@ -121,8 +119,7 @@ void StdioSilencer::silence()
     return std::system_error(errno, std::generic_category(), msg);
   };
 
-  try
-  {
+  try {
     int fd = fileno(ofile);
     if (fd < 0)
       throw syserr("fileno()");
@@ -140,11 +137,10 @@ void StdioSilencer::silence()
       throw syserr("dup2(new_stdio_, fd)");
 
     close(new_stdio);
-  }
-  catch (const std::system_error &e)
-  {
+  } catch (const std::system_error &e) {
     if (errno == EMFILE)
-      LOG(FATAL) << e.what() << ": please raise NOFILE";
+      throw bpftrace::FatalUserException(std::string(e.what()) +
+                                         ": please raise NOFILE");
     else
       LOG(BUG) << e.what();
   }
@@ -159,8 +155,7 @@ StdioSilencer::~StdioSilencer()
     return std::system_error(errno, std::generic_category(), msg);
   };
 
-  try
-  {
+  try {
     int fd = fileno(ofile);
     if (fd < 0)
       throw syserr("fileno()");
@@ -170,9 +165,7 @@ StdioSilencer::~StdioSilencer()
       throw syserr("dup2(old_stdio_)");
     close(old_stdio_);
     old_stdio_ = -1;
-  }
-  catch (const std::system_error &e)
-  {
+  } catch (const std::system_error &e) {
     LOG(BUG) << e.what();
   }
 }
@@ -186,8 +179,7 @@ KConfig::KConfig()
   const char *path_env = std::getenv("BPFTRACE_KCONFIG_TEST");
   if (path_env)
     config_locs = { std::string(path_env) };
-  else
-  {
+  else {
     struct utsname utsname;
     if (uname(&utsname) < 0)
       return;
@@ -197,19 +189,16 @@ KConfig::KConfig()
     };
   }
 
-  for (auto &path : config_locs)
-  {
+  for (auto &path : config_locs) {
     // gzopen/gzgets handle both uncompressed and compressed files
     gzFile file = gzopen(path.c_str(), "r");
     if (!file)
       continue;
 
     char buf[4096];
-    while (gzgets(file, buf, sizeof(buf)))
-    {
+    while (gzgets(file, buf, sizeof(buf))) {
       std::string option(buf);
-      if (option.find("CONFIG_") == 0)
-      {
+      if (option.find("CONFIG_") == 0) {
         // trim trailing '\n'
         if (option[option.length() - 1] == '\n')
           option = option.substr(0, option.length() - 1);
@@ -226,48 +215,42 @@ KConfig::KConfig()
   }
 }
 
-bool get_uint64_env_var(const ::std::string &str,
+void get_uint64_env_var(const ::std::string &str,
                         const std::function<void(uint64_t)> &cb)
 {
   uint64_t dest;
-  if (const char *env_p = std::getenv(str.c_str()))
-  {
+  if (const char *env_p = std::getenv(str.c_str())) {
     std::istringstream stringstream(env_p);
-    if (!(stringstream >> dest))
-    {
-      LOG(ERROR) << "Env var '" << str
-                 << "' did not contain a valid uint64_t, or was zero-valued.";
-      return false;
+    if (!(stringstream >> dest)) {
+      throw bpftrace::FatalUserException(
+          "Env var '" + str +
+          "' did not contain a valid uint64_t, or was zero-valued.");
+      return;
     }
     cb(dest);
   }
-  return true;
 }
 
-bool get_bool_env_var(const ::std::string &str,
+void get_bool_env_var(const ::std::string &str,
                       const std::function<void(bool)> &cb)
 {
-  if (const char *env_p = std::getenv(str.c_str()))
-  {
+  if (const char *env_p = std::getenv(str.c_str())) {
     bool dest;
     std::string s(env_p);
     if (s == "1")
       dest = true;
     else if (s == "0")
       dest = false;
-    else
-    {
-      LOG(ERROR) << "Env var '" << str
-                 << "' did not contain a "
-                    "valid value (0 or 1).";
-      return false;
+    else {
+      throw bpftrace::FatalUserException(
+          "Env var '" + str + "' did not contain a valid value (0 or 1).");
     }
     cb(dest);
   }
-  return true;
+  return;
 }
 
-std::optional<std_filesystem::path> find_in_path(const std::string &name)
+std::optional<std_filesystem::path> find_in_path(std::string_view name)
 {
   std::error_code ec;
 
@@ -276,8 +259,7 @@ std::optional<std_filesystem::path> find_in_path(const std::string &name)
     return std::nullopt;
 
   auto paths = split_string(path_env, ':', true);
-  for (const auto &path : paths)
-  {
+  for (const auto &path : paths) {
     auto fpath = std_filesystem::path(path) / name;
     if (std_filesystem::exists(fpath, ec))
       return fpath;
@@ -286,6 +268,26 @@ std::optional<std_filesystem::path> find_in_path(const std::string &name)
   return std::nullopt;
 }
 
+std::optional<std_filesystem::path> find_near_self(std::string_view filename)
+{
+  std::error_code ec;
+  auto exe = std_filesystem::read_symlink("/proc/self/exe", ec);
+  if (ec) {
+    LOG(WARNING) << "Failed to resolve /proc/self/exe: " << ec;
+    return std::nullopt;
+  }
+
+  exe.replace_filename(filename);
+  bool exists = std_filesystem::exists(exe, ec);
+  if (!exists) {
+    if (ec)
+      LOG(WARNING) << "Failed to resolve stat " << exe << ": " << ec;
+    return std::nullopt;
+  }
+
+  return exe;
+}
+
 std::string get_pid_exe(const std::string &pid)
 {
   std::error_code ec;
@@ -293,12 +295,9 @@ std::string get_pid_exe(const std::string &pid)
   proc_path /= pid;
   proc_path /= "exe";
 
-  try
-  {
+  try {
     return std_filesystem::read_symlink(proc_path).string();
-  }
-  catch (const std_filesystem::filesystem_error &e)
-  {
+  } catch (const std_filesystem::filesystem_error &e) {
     auto err = e.code().value();
     if (err == ENOENT || err == EINVAL)
       return {};
@@ -335,8 +334,7 @@ std::vector<std::string> get_mapped_paths_for_pid(pid_t pid)
   static std::map<pid_t, std::vector<std::string>> paths_cache;
 
   auto it = paths_cache.find(pid);
-  if (it != paths_cache.end())
-  {
+  if (it != paths_cache.end()) {
     return it->second;
   }
 
@@ -349,15 +347,13 @@ std::vector<std::string> get_mapped_paths_for_pid(pid_t pid)
 
   // get all the mapped libraries
   std::string maps_path = get_proc_maps(pid);
-  if (maps_path.empty())
-  {
+  if (maps_path.empty()) {
     LOG(WARNING) << "Maps path is empty";
     return paths;
   }
 
   std::fstream fs(maps_path, std::ios_base::in);
-  if (!fs.is_open())
-  {
+  if (!fs.is_open()) {
     LOG(WARNING) << "Unable to open procfs mapfile: " << maps_path;
     return paths;
   }
@@ -367,18 +363,15 @@ std::vector<std::string> get_mapped_paths_for_pid(pid_t pid)
   std::string line;
   // Example mapping:
   // 7fc8ee4fa000-7fc8ee4fb000 r--p 00000000 00:1f 27168296 /usr/libc.so.6
-  while (std::getline(fs, line))
-  {
+  while (std::getline(fs, line)) {
     char buf[PATH_MAX + 1];
     buf[0] = '\0';
     auto res = std::sscanf(line.c_str(), "%*s %*s %*x %*s %*u %[^\n]", buf);
     // skip [heap], [vdso], and non file paths etc...
-    if (res == 1 && buf[0] == '/')
-    {
+    if (res == 1 && buf[0] == '/') {
       std::string name = buf;
       if (name.find("(deleted)") == std::string::npos &&
-          seen_mappings.count(name) == 0)
-      {
+          seen_mappings.count(name) == 0) {
         seen_mappings.emplace(name);
         paths.push_back(std::move(name));
       }
@@ -392,16 +385,13 @@ std::vector<std::string> get_mapped_paths_for_pid(pid_t pid)
 std::vector<std::string> get_mapped_paths_for_running_pids()
 {
   std::unordered_set<std::string> unique_paths;
-  for (auto pid : get_all_running_pids())
-  {
-    for (auto &path : get_mapped_paths_for_pid(pid))
-    {
+  for (auto pid : get_all_running_pids()) {
+    for (auto &path : get_mapped_paths_for_pid(pid)) {
       unique_paths.insert(std::move(path));
     }
   }
   std::vector<std::string> paths;
-  for (auto &path : unique_paths)
-  {
+  for (auto &path : unique_paths) {
     paths.emplace_back(std::move(path));
   }
   return paths;
@@ -421,7 +411,7 @@ std::vector<std::string> split_string(const std::string &str,
   std::vector<std::string> elems;
   std::stringstream ss(str);
   std::string value;
-  while(std::getline(ss, value, delimiter)) {
+  while (std::getline(ss, value, delimiter)) {
     if (remove_empty && value.empty())
       continue;
 
@@ -438,7 +428,11 @@ std::string erase_prefix(std::string &str)
   return prefix;
 }
 
-bool wildcard_match(const std::string &str, std::vector<std::string> &tokens, bool start_wildcard, bool end_wildcard) {
+bool wildcard_match(const std::string &str,
+                    std::vector<std::string> &tokens,
+                    bool start_wildcard,
+                    bool end_wildcard)
+{
   size_t next = 0;
 
   if (!start_wildcard)
@@ -518,9 +512,7 @@ std::vector<std::string> get_kernel_cflags(const char *uname_machine,
     arch = "sh";
   } else if (!strncmp(uname_machine, "aarch64", 7)) {
     arch = "arm64";
-  }
-  else if (!strncmp(uname_machine, "loongarch", 9))
-  {
+  } else if (!strncmp(uname_machine, "loongarch", 9)) {
     arch = "loongarch";
   }
 
@@ -534,12 +526,12 @@ std::vector<std::string> get_kernel_cflags(const char *uname_machine,
   cflags.push_back("/virtual/lib/clang/include");
 
   // see linux/Makefile for $(LINUXINCLUDE) + $(USERINCLUDE)
-  cflags.push_back("-I" + ksrc + "/arch/"+arch+"/include");
-  cflags.push_back("-I" + kobj + "/arch/"+arch+"/include/generated");
+  cflags.push_back("-I" + ksrc + "/arch/" + arch + "/include");
+  cflags.push_back("-I" + kobj + "/arch/" + arch + "/include/generated");
   cflags.push_back("-I" + ksrc + "/include");
   cflags.push_back("-I" + kobj + "/include");
-  cflags.push_back("-I" + ksrc + "/arch/"+arch+"/include/uapi");
-  cflags.push_back("-I" + kobj + "/arch/"+arch+"/include/generated/uapi");
+  cflags.push_back("-I" + ksrc + "/arch/" + arch + "/include/uapi");
+  cflags.push_back("-I" + kobj + "/arch/" + arch + "/include/generated/uapi");
   cflags.push_back("-I" + ksrc + "/include/uapi");
   cflags.push_back("-I" + kobj + "/include/generated/uapi");
 
@@ -556,19 +548,16 @@ std::vector<std::string> get_kernel_cflags(const char *uname_machine,
   if (archenv)
     cflags.push_back("-D__TARGET_ARCH_" + arch);
 
-  if (arch == "arm")
-  {
+  if (arch == "arm") {
     // Required by several header files in arch/arm/include
     cflags.push_back("-D__LINUX_ARM_ARCH__=7");
   }
 
-  if (arch == "arm64")
-  {
+  if (arch == "arm64") {
     // arm64 defines KASAN_SHADOW_SCALE_SHIFT in a Makefile instead of defining
     // it in a header file. Since we're not executing make, we need to set the
     // value manually (values are taken from arch/arm64/Makefile).
-    if (kconfig.has_value("CONFIG_KASAN", "y"))
-    {
+    if (kconfig.has_value("CONFIG_KASAN", "y")) {
       if (kconfig.has_value("CONFIG_KASAN_SW_TAGS", "y"))
         cflags.push_back("-DKASAN_SHADOW_SCALE_SHIFT=4");
       else
@@ -593,19 +582,16 @@ std::string get_cgroup_path_in_hierarchy(uint64_t cgroupid,
 
   // Check for root cgroup path separately, since recursive_directory_iterator
   // does not iterate over base directory
-  if (stat(base_path.c_str(), &path_st) >= 0 && path_st.st_ino == cgroupid)
-  {
+  if (stat(base_path.c_str(), &path_st) >= 0 && path_st.st_ino == cgroupid) {
     path_cache[{ cgroupid, base_path }] = "/";
     return "/";
   }
 
   for (auto &path_iter :
-       std_filesystem::recursive_directory_iterator(base_path))
-  {
+       std_filesystem::recursive_directory_iterator(base_path)) {
     if (stat(path_iter.path().c_str(), &path_st) < 0)
       return "";
-    if (path_st.st_ino == cgroupid)
-    {
+    if (path_st.st_ino == cgroupid) {
       // Base directory is not a part of cgroup path
       path_cache[{ cgroupid, base_path }] = path_iter.path().string().substr(
           base_path.length());
@@ -623,11 +609,9 @@ std::vector<std::pair<std::string, std::string>> get_cgroup_hierarchy_roots()
   std::vector<std::pair<std::string, std::string>> result;
 
   const std::regex cgroup_mount_regex("(cgroup[2]?) (\\S*)[ ]?.*");
-  for (std::string line; std::getline(mounts_file, line);)
-  {
+  for (std::string line; std::getline(mounts_file, line);) {
     std::smatch match;
-    if (std::regex_match(line, match, cgroup_mount_regex))
-    {
+    if (std::regex_match(line, match, cgroup_mount_regex)) {
       result.push_back({ match[1].str(), match[2].str() });
     }
   }
@@ -645,15 +629,11 @@ std::vector<std::pair<std::string, std::string>> get_cgroup_paths(
 
   // Replace cgroup version with cgroup mount point directory name for cgroupv1
   // roots and "unified" for cgroupv2 roots
-  for (auto &root : roots)
-  {
-    if (root.first == "cgroup")
-    {
+  for (auto &root : roots) {
+    if (root.first == "cgroup") {
       root = { std_filesystem::path(root.second).filename().string(),
                root.second };
-    }
-    else if (root.first == "cgroup2")
-    {
+    } else if (root.first == "cgroup2") {
       root = { "unified", root.second };
     }
   }
@@ -695,114 +675,33 @@ std::vector<std::pair<std::string, std::string>> get_cgroup_paths(
   return result;
 }
 
-bool is_dir(const std::string& path)
+bool is_module_loaded(const std::string &module)
+{
+  if (module == "vmlinux") {
+    return true;
+  }
+
+  // This file lists all loaded modules
+  std::ifstream modules_file("/proc/modules");
+
+  for (std::string line; std::getline(modules_file, line);) {
+    if (line.compare(0, module.size() + 1, module + " ") == 0) {
+      modules_file.close();
+      return true;
+    }
+  }
+
+  modules_file.close();
+  return false;
+}
+
+bool is_dir(const std::string &path)
 {
   std::error_code ec;
   std_filesystem::path buf{ path };
   return std_filesystem::is_directory(buf, ec);
 }
 
-bool file_exists_and_ownedby_root(const char *f)
-{
-  struct stat st;
-  if (stat(f, &st) == 0)
-  {
-    if (st.st_uid != 0)
-    {
-      LOG(ERROR) << "header file ownership expected to be root: "
-                 << std::string(f);
-      return false;
-    }
-    return true;
-  }
-  return false;
-}
-
-namespace {
-  struct KernelHeaderTmpDir {
-    KernelHeaderTmpDir(const std::string& prefix) : path{prefix + "XXXXXX"}
-    {
-      if (::mkdtemp(&path[0]) == nullptr) {
-        throw std::runtime_error("creating temporary path for kheaders.tar.xz failed");
-      }
-    }
-
-    ~KernelHeaderTmpDir()
-    {
-      if (path.size() > 0) {
-        // move_to either did not succeed or did not run, so clean up after ourselves
-        exec_system(("rm -rf " + path).c_str());
-      }
-    }
-
-    void move_to(const std::string& new_path)
-    {
-      int err = ::rename(path.c_str(), new_path.c_str());
-      if (err == 0) {
-        path = "";
-      }
-    }
-
-    std::string path;
-  };
-
-  std::string unpack_kheaders_tar_xz(const struct utsname& utsname)
-  {
-    std::error_code ec;
-#if defined(__ANDROID__)
-    std_filesystem::path path_prefix{ "/data/local/tmp" };
-#else
-    std_filesystem::path path_prefix{ "/tmp" };
-#endif
-    std_filesystem::path path_kheaders{ PROC_KHEADERS_PATH };
-    if (const char* tmpdir = ::getenv("TMPDIR")) {
-      path_prefix = tmpdir;
-    }
-    path_prefix /= "kheaders-";
-    std_filesystem::path shared_path{ path_prefix.string() + utsname.release };
-
-    if (file_exists_and_ownedby_root(shared_path.c_str()))
-    {
-      // already unpacked
-      return shared_path.string();
-    }
-
-    if (!std_filesystem::exists(path_kheaders, ec))
-    {
-      StderrSilencer silencer;
-      silencer.silence();
-
-      FILE* modprobe = ::popen("modprobe kheaders", "w");
-      if (modprobe == nullptr || pclose(modprobe) != 0) {
-        return "";
-      }
-
-      if (!std_filesystem::exists(path_kheaders, ec))
-      {
-        return "";
-      }
-    }
-
-    KernelHeaderTmpDir tmpdir{path_prefix};
-
-    FILE *tar = ::popen(("tar xf " + std::string(PROC_KHEADERS_PATH) + " -C " +
-                         tmpdir.path)
-                            .c_str(),
-                        "w");
-    if (!tar) {
-      return "";
-    }
-
-    int rc = ::pclose(tar);
-    if (rc == 0) {
-      tmpdir.move_to(shared_path);
-      return shared_path;
-    }
-
-    return "";
-  }
-} // namespace
-
 // get_kernel_dirs returns {ksrc, kobj} - directories for pristine and
 // generated kernel sources.
 //
@@ -818,21 +717,19 @@ namespace {
 //   /lib/modules/`uname -r`/build/
 //
 // {"", ""} is returned if no trace of kernel headers was found at all.
-// Both ksrc and kobj are guaranteed to be != "", if at least some trace of kernel sources was found.
+// Both ksrc and kobj are guaranteed to be != "", if at least some trace of
+// kernel sources was found.
 std::tuple<std::string, std::string> get_kernel_dirs(
-    const struct utsname &utsname,
-    bool unpack_kheaders)
+    const struct utsname &utsname)
 {
 #ifdef KERNEL_HEADERS_DIR
-  return {KERNEL_HEADERS_DIR, KERNEL_HEADERS_DIR};
+  return { KERNEL_HEADERS_DIR, KERNEL_HEADERS_DIR };
 #endif
 
   const char *kpath_env = ::getenv("BPFTRACE_KERNEL_SOURCE");
-  if (kpath_env)
-  {
+  if (kpath_env) {
     const char *kpath_build_env = ::getenv("BPFTRACE_KERNEL_BUILD");
-    if (!kpath_build_env)
-    {
+    if (!kpath_build_env) {
       kpath_build_env = kpath_env;
     }
     return std::make_tuple(kpath_env, kpath_build_env);
@@ -842,29 +739,24 @@ std::tuple<std::string, std::string> get_kernel_dirs(
   auto ksrc = kdir + "/source";
   auto kobj = kdir + "/build";
 
-  // if one of source/ or build/ is not present - try to use the other one for both.
-  if (!is_dir(ksrc)) {
-    ksrc = "";
-  }
-  if (!is_dir(kobj)) {
-    kobj = "";
-  }
-  if (ksrc.empty() && kobj.empty())
-  {
-    if (unpack_kheaders)
-    {
-      const auto kheaders_tar_xz_path = unpack_kheaders_tar_xz(utsname);
-      if (kheaders_tar_xz_path.size() > 0)
-        return std::make_tuple(kheaders_tar_xz_path, kheaders_tar_xz_path);
-    }
+  // if one of source/ or build/ is not present - try to use the other one for
+  // both.
+  auto has_ksrc = is_dir(ksrc);
+  auto has_kobj = is_dir(kobj);
+  if (!has_ksrc && !has_kobj) {
+    LOG(WARNING) << "Could not find kernel headers in " << ksrc << " or "
+                 << kobj
+                 << ". To specify a particular path to kernel headers, set the "
+                    "env variables BPFTRACE_KERNEL_SOURCE and, optionally, "
+                    "BPFTRACE_KERNEL_BUILD if the kernel was built in a "
+                    "different directory than its source. To create kernel "
+                    "headers run 'modprobe kheaders', which will create a tar "
+                    "file at /sys/kernel/kheaders.tar.xz";
     return std::make_tuple("", "");
   }
-  if (ksrc.empty())
-  {
+  if (!has_ksrc) {
     ksrc = kobj;
-  }
-  else if (kobj.empty())
-  {
+  } else if (!has_kobj) {
     kobj = ksrc;
   }
 
@@ -873,22 +765,22 @@ std::tuple<std::string, std::string> get_kernel_dirs(
 
 const std::string &is_deprecated(const std::string &str)
 {
+  for (auto &item : DEPRECATED_LIST) {
+    if (!item.matches(str)) {
+      continue;
+    }
 
-  std::vector<DeprecatedName>::iterator item;
+    if (item.show_warning) {
+      LOG(WARNING) << item.old_name
+                   << " is deprecated and will be removed in the future. Use "
+                   << item.new_name << " instead.";
+      item.show_warning = false;
+    }
 
-  for (item = DEPRECATED_LIST.begin(); item != DEPRECATED_LIST.end(); item++)
-  {
-    if (str == item->old_name)
-    {
-      if (item->show_warning)
-      {
-        LOG(WARNING) << item->old_name
-                     << " is deprecated and will be removed in the future. Use "
-                     << item->new_name << " instead.";
-        item->show_warning = false;
-      }
-
-      return item->new_name;
+    if (item.replace_by_new_name) {
+      return item.new_name;
+    } else {
+      return str;
     }
   }
 
@@ -897,12 +789,9 @@ const std::string &is_deprecated(const std::string &str)
 
 bool is_unsafe_func(const std::string &func_name)
 {
-  return std::any_of(
-      UNSAFE_BUILTIN_FUNCS.begin(),
-      UNSAFE_BUILTIN_FUNCS.end(),
-      [&](const auto& cand) {
-        return func_name == cand;
-      });
+  return std::any_of(UNSAFE_BUILTIN_FUNCS.begin(),
+                     UNSAFE_BUILTIN_FUNCS.end(),
+                     [&](const auto &cand) { return func_name == cand; });
 }
 
 bool is_compile_time_func(const std::string &func_name)
@@ -919,12 +808,19 @@ bool is_supported_lang(const std::string &lang)
                      [&](const auto &cand) { return lang == cand; });
 }
 
+bool is_type_name(std::string_view str)
+{
+  return str.find("struct ") == 0 || str.find("union ") == 0 ||
+         str.find("enum ") == 0;
+}
+
 std::string exec_system(const char *cmd)
 {
   std::array<char, 128> buffer;
   std::string result;
   std::shared_ptr<FILE> pipe(popen(cmd, "r"), pclose);
-  if (!pipe) throw std::runtime_error("popen() failed!");
+  if (!pipe)
+    throw bpftrace::FatalUserException("popen() failed!");
   while (!feof(pipe.get())) {
     if (fgets(buffer.data(), 128, pipe.get()) != nullptr)
       result += buffer.data();
@@ -935,7 +831,7 @@ std::string exec_system(const char *cmd)
 /*
 Original resolve_binary_path API defaulting to bpftrace's mount namespace
 */
-std::vector<std::string> resolve_binary_path(const std::string& cmd)
+std::vector<std::string> resolve_binary_path(const std::string &cmd)
 {
   const char *env_paths = getenv("PATH");
   return resolve_binary_path(cmd, env_paths, -1);
@@ -951,28 +847,22 @@ std::vector<std::string> resolve_binary_path(const std::string &cmd, int pid)
   std::string env_paths = "";
   std::ostringstream pid_environ_path;
 
-  if (pid > 0 && pid_in_different_mountns(pid))
-  {
+  if (pid > 0 && pid_in_different_mountns(pid)) {
     pid_environ_path << "/proc/" << pid << "/environ";
     std::ifstream environ(pid_environ_path.str());
 
-    if (environ)
-    {
+    if (environ) {
       std::string env_var;
       std::string pathstr = ("PATH=");
-      while (std::getline(environ, env_var, '\0'))
-      {
-        if (env_var.find(pathstr) != std::string::npos)
-        {
+      while (std::getline(environ, env_var, '\0')) {
+        if (env_var.find(pathstr) != std::string::npos) {
           env_paths = env_var.substr(pathstr.length());
           break;
         }
       }
     }
     return resolve_binary_path(cmd, env_paths.c_str(), pid);
-  }
-  else
-  {
+  } else {
     return resolve_binary_path(cmd, getenv("PATH"), pid);
   }
 }
@@ -981,21 +871,21 @@ std::vector<std::string> resolve_binary_path(const std::string &cmd, int pid)
 Private interface to resolve_binary_path, used for the exposed variants above,
 allowing for a PID whose mount namespace should be optionally considered.
 */
-static std::vector<std::string>
-resolve_binary_path(const std::string &cmd, const char *env_paths, int pid)
+static std::vector<std::string> resolve_binary_path(const std::string &cmd,
+                                                    const char *env_paths,
+                                                    int pid)
 {
   std::vector<std::string> candidate_paths = { cmd };
 
   if (env_paths != nullptr && cmd.find("/") == std::string::npos)
-    for (const auto& path : split_string(env_paths, ':'))
+    for (const auto &path : split_string(env_paths, ':'))
       candidate_paths.push_back(path + "/" + cmd);
 
   if (cmd.find("*") != std::string::npos)
     candidate_paths = ::expand_wildcard_paths(candidate_paths);
 
   std::vector<std::string> valid_executable_paths;
-  for (const auto &path : candidate_paths)
-  {
+  for (const auto &path : candidate_paths) {
     std::string rel_path;
     if (pid > 0 && pid_in_different_mountns(pid))
       rel_path = path_for_pid_mountns(pid, path);
@@ -1016,13 +906,10 @@ std::string path_for_pid_mountns(int pid, const std::string &path)
 
   snprintf(pid_root, sizeof(pid_root), "/proc/%d/root", pid);
 
-  if (path.find(pid_root) != 0)
-  {
+  if (path.find(pid_root) != 0) {
     std::string sep = (path.length() >= 1 && path.at(0) == '/') ? "" : "/";
     pid_relative_path << pid_root << sep << path;
-  }
-  else
-  {
+  } else {
     // The path is already relative to the pid's root
     pid_relative_path << path;
   }
@@ -1053,15 +940,13 @@ static bool pid_in_different_mountns(int pid)
   target_path /= std::to_string(pid);
   target_path /= "ns/mnt";
 
-  if (!std_filesystem::exists(self_path, ec))
-  {
+  if (!std_filesystem::exists(self_path, ec)) {
     throw MountNSException(
         "Failed to compare mount ns with PID " + std::to_string(pid) +
         ". The error was open (/proc/self/ns/mnt): " + ec.message());
   }
 
-  if (!std_filesystem::exists(target_path, ec))
-  {
+  if (!std_filesystem::exists(target_path, ec)) {
     throw MountNSException(
         "Failed to compare mount ns with PID " + std::to_string(pid) +
         ". The error was open (/proc/<pid>/ns/mnt): " + ec.message());
@@ -1069,8 +954,7 @@ static bool pid_in_different_mountns(int pid)
 
   bool result = !std_filesystem::equivalent(self_path, target_path, ec);
 
-  if (ec)
-  {
+  if (ec) {
     throw MountNSException("Failed to compare mount ns with PID " +
                            std::to_string(pid) +
                            ". The error was (fstat): " + ec.message());
@@ -1084,7 +968,7 @@ void cat_file(const char *filename, size_t max_bytes, std::ostream &out)
   std::ifstream file(filename);
   const size_t BUFSIZE = 4096;
 
-  if (file.fail()){
+  if (file.fail()) {
     LOG(ERROR) << "failed to open file '" << filename
                << "': " << strerror(errno);
     return;
@@ -1110,12 +994,12 @@ void cat_file(const char *filename, size_t max_bytes, std::ostream &out)
   }
 }
 
-std::string str_join(const std::vector<std::string> &list, const std::string &delim)
+std::string str_join(const std::vector<std::string> &list,
+                     const std::string &delim)
 {
   std::string str;
   bool first = true;
-  for (const auto &elem : list)
-  {
+  for (const auto &elem : list) {
     if (first)
       first = false;
     else
@@ -1129,12 +1013,9 @@ std::string str_join(const std::vector<std::string> &list, const std::string &de
 bool is_numeric(const std::string &s)
 {
   std::size_t idx;
-  try
-  {
+  try {
     std::stoll(s, &idx, 0);
-  }
-  catch (...)
-  {
+  } catch (...) {
     return false;
   }
   return idx == s.size();
@@ -1148,30 +1029,40 @@ bool symbol_has_cpp_mangled_signature(const std::string &sym_name)
     return false;
 }
 
-pid_t parse_pid(const std::string &str)
+static std::string get_invalid_pid_message(const std::string &pid,
+                                           const std::string &msg)
 {
-  try
-  {
-    constexpr ssize_t pid_max = 4 * 1024 * 1024;
-    std::size_t idx = 0;
-    auto pid = std::stol(str, &idx, 10);
-    // Detect cases like `13ABC`
-    if (idx < str.size())
-      throw InvalidPIDException(str, "is not a valid decimal number");
-    if (pid < 1 || pid > pid_max)
-      throw InvalidPIDException(str,
-                                "out of valid pid range [1," +
-                                    std::to_string(pid_max) + "]");
-    return pid;
+  return "pid '" + pid + "' " + msg;
+}
+
+std::optional<pid_t> parse_pid(const std::string &str, std::string &err)
+{
+  std::size_t idx = 0;
+  pid_t pid;
+  constexpr ssize_t pid_max = 4 * 1024 * 1024;
+  try {
+    pid = std::stol(str, &idx, 10);
+  } catch (const std::out_of_range &e) {
+    err = get_invalid_pid_message(str, "outside of integer range");
+    return std::nullopt;
+  } catch (const std::invalid_argument &e) {
+    err = get_invalid_pid_message(str, "is not a valid decimal number");
+    return std::nullopt;
   }
-  catch (const std::out_of_range &e)
-  {
-    throw InvalidPIDException(str, "outside of integer range");
+  // Detect cases like `13ABC`
+  if (idx < str.size()) {
+    err = get_invalid_pid_message(str, "is not a valid decimal number");
+    return std::nullopt;
   }
-  catch (const std::invalid_argument &e)
-  {
-    throw InvalidPIDException(str, "is not a valid decimal number");
+
+  if (pid < 1 || pid > pid_max) {
+    err = get_invalid_pid_message(str,
+                                  "out of valid pid range [1," +
+                                      std::to_string(pid_max) + "]");
+    return std::nullopt;
   }
+
+  return pid;
 }
 
 std::string hex_format_buffer(const char *buf,
@@ -1198,6 +1089,44 @@ std::string hex_format_buffer(const char *buf,
   return std::string(s);
 }
 
+/*
+ * Attaching to these kernel functions with kfunc/fentry or kretfunc/fexit
+ * could lead to a recursive loop and kernel crash so we need additional
+ * generated BPF code to protect against this if one of these are being
+ * attached to.
+ */
+bool is_recursive_func(const std::string &func_name)
+{
+  return RECURSIVE_KERNEL_FUNCS.find(func_name) != RECURSIVE_KERNEL_FUNCS.end();
+}
+
+static bool is_bad_func(std::string &func)
+{
+  /*
+   * Certain kernel functions are known to cause system stability issues if
+   * traced (but not marked "notrace" in the kernel) so they should be filtered
+   * out as the list is built. The list of functions have been taken from the
+   * bpf kernel selftests (bpf/prog_tests/kprobe_multi_test.c).
+   */
+  static const std::unordered_set<std::string> bad_funcs = {
+    "arch_cpu_idle", "default_idle", "bpf_dispatcher_xdp_func"
+  };
+
+  static const std::vector<std::string> bad_funcs_partial = {
+    "__ftrace_invalid_address__", "rcu_"
+  };
+
+  if (bad_funcs.find(func) != bad_funcs.end())
+    return true;
+
+  for (const auto &s : bad_funcs_partial) {
+    if (!std::strncmp(func.c_str(), s.c_str(), s.length()))
+      return true;
+  }
+
+  return false;
+}
+
 FuncsModulesMap parse_traceable_funcs()
 {
 #ifdef FUZZ
@@ -1210,34 +1139,29 @@ FuncsModulesMap parse_traceable_funcs()
                                       : tracefs::available_filter_functions();
 
   std::ifstream available_funs(kprobe_path);
-  if (available_funs.fail())
-  {
-    if (bt_debug != DebugLevel::kNone)
-    {
-      std::cerr << "Error while reading traceable functions from "
-                << kprobe_path << ": " << strerror(errno);
-    }
+  if (available_funs.fail()) {
+    LOG(V1) << "Error while reading traceable functions from " << kprobe_path
+            << ": " << strerror(errno);
     return {};
   }
 
   FuncsModulesMap result;
   std::string line;
-  while (std::getline(available_funs, line))
-  {
+  while (std::getline(available_funs, line)) {
     auto func_mod = split_symbol_module(line);
     if (func_mod.second.empty())
       func_mod.second = "vmlinux";
-    result[func_mod.first].insert(func_mod.second);
+
+    if (!is_bad_func(func_mod.first))
+      result[func_mod.first].insert(func_mod.second);
   }
 
   // Filter out functions from the kprobe blacklist.
   const std::string kprobes_blacklist_path = debugfs::kprobes_blacklist();
   std::ifstream kprobes_blacklist_funs(kprobes_blacklist_path);
-  while (std::getline(kprobes_blacklist_funs, line))
-  {
+  while (std::getline(kprobes_blacklist_funs, line)) {
     auto addr_func_mod = split_addrrange_symbol_module(line);
-    if (result.find(std::get<1>(addr_func_mod)) != result.end())
-    {
+    if (result.find(std::get<1>(addr_func_mod)) != result.end()) {
       result.erase(std::get<1>(addr_func_mod));
     }
   }
@@ -1253,18 +1177,15 @@ static uint32_t _find_version_note(unsigned long base)
 {
   auto ehdr = reinterpret_cast<const ElfW(Ehdr) *>(base);
 
-  for (int i = 0; i < ehdr->e_shnum; i++)
-  {
+  for (int i = 0; i < ehdr->e_shnum; i++) {
     auto shdr = reinterpret_cast<const ElfW(Shdr) *>(base + ehdr->e_shoff +
                                                      (i * ehdr->e_shentsize));
 
-    if (shdr->sh_type == SHT_NOTE)
-    {
+    if (shdr->sh_type == SHT_NOTE) {
       auto ptr = reinterpret_cast<const char *>(base + shdr->sh_offset);
       auto end = ptr + shdr->sh_size;
 
-      while (ptr < end)
-      {
+      while (ptr < end) {
         auto nhdr = reinterpret_cast<const ElfW(Nhdr) *>(ptr);
         ptr += sizeof *nhdr;
 
@@ -1330,85 +1251,51 @@ static uint32_t kernel_version_from_khdr(void)
  * may not match if bpftrace is compiled on a different Linux version than it's
  * used on, e.g. if built with Docker.
  */
-uint32_t kernel_version(int attempt)
+uint32_t kernel_version(KernelVersionMethod method)
 {
   static std::optional<uint32_t> a0, a1, a2;
-  switch (attempt)
-  {
-    case 0:
-    {
+  switch (method) {
+    case vDSO: {
       if (!a0)
         a0 = kernel_version_from_vdso();
       return *a0;
     }
-    case 1:
-    {
+    case UTS: {
       if (!a1)
         a1 = kernel_version_from_uts();
       return *a1;
     }
-    case 2:
-    {
+    case File: {
       if (!a2)
         a2 = kernel_version_from_khdr();
       return *a2;
     }
-    default:
-      LOG(BUG) << "kernel_version(): Invalid attempt: "
-               << std::to_string(attempt);
+    case None:
+      return 0;
   }
-  // lgtm[cpp/missing-return]
+
+  // Unreachable
+  return 0;
 }
 
 std::optional<std::string> abs_path(const std::string &rel_path)
 {
   // filesystem::canonical does not work very well with /proc/<pid>/root paths
   // of processes in a different mount namespace (than the one bpftrace is
-  // running in), failing during canonicalization. See iovisor:bpftrace#1595
+  // running in), failing during canonicalization. See bpftrace:bpftrace#1595
   static auto re = std::regex("^/proc/\\d+/root/.*");
-  if (!std::regex_match(rel_path, re))
-  {
-    try
-    {
+  if (!std::regex_match(rel_path, re)) {
+    try {
       auto p = std_filesystem::path(rel_path);
       return std_filesystem::canonical(std_filesystem::absolute(p)).string();
-    }
-    catch (std_filesystem::filesystem_error &)
-    {
+    } catch (std_filesystem::filesystem_error &) {
       return {};
     }
-  }
-  else
-  {
+  } else {
     return rel_path;
   }
 }
 
-int64_t min_value(const std::vector<uint8_t> &value, int nvalues)
-{
-  int64_t val, max = 0;
-  for (int i = 0; i < nvalues; i++)
-  {
-    val = read_data<int64_t>(value.data() + i * sizeof(int64_t));
-    if (val > max)
-      max = val;
-  }
-
-  return max;
-}
-
-uint64_t max_value(const std::vector<uint8_t> &value, int nvalues)
-{
-  uint64_t val, max = 0;
-  for (int i = 0; i < nvalues; i++)
-  {
-    val = read_data<uint64_t>(value.data() + i * sizeof(uint64_t));
-    if (val > max)
-      max = val;
-  }
-  return max;
-}
-
 bool symbol_has_module(const std::string &symbol)
 {
   return !symbol.empty() && symbol[symbol.size() - 1] == ']';
@@ -1473,7 +1360,7 @@ std::map<uintptr_t, elf_symbol, std::greater<>> get_symbol_table_for_elf(
   };
   struct bcc_symbol_option option;
   memset(&option, 0, sizeof(option));
-  option.use_symbol_type = BCC_SYM_ALL_TYPES;
+  option.use_symbol_type = BCC_SYM_ALL_TYPES ^ (1 << STT_NOTYPE);
   bcc_elf_foreach_sym(
       elf_file.c_str(), sym_resolve_callback, &option, &symbol_table);
 
@@ -1484,8 +1371,7 @@ std::vector<int> get_pids_for_program(const std::string &program)
 {
   std::error_code ec;
   auto program_abs = std_filesystem::canonical(program, ec);
-  if (ec)
-  {
+  if (ec) {
     // std::filesystem::canonical will fail if we are attaching to a uprobe that
     // lives in another filesystem namespace. For example,
     // uprobe:/proc/12345/root/my_program:function1
@@ -1497,8 +1383,7 @@ std::vector<int> get_pids_for_program(const std::string &program)
   }
 
   std::vector<int> pids;
-  for (const auto &process : std_filesystem::directory_iterator("/proc"))
-  {
+  for (const auto &process : std_filesystem::directory_iterator("/proc")) {
     std::string filename = process.path().filename().string();
     if (!std::all_of(filename.begin(), filename.end(), ::isdigit))
       continue;
@@ -1514,8 +1399,7 @@ std::vector<int> get_pids_for_program(const std::string &program)
 std::vector<int> get_all_running_pids()
 {
   std::vector<int> pids;
-  for (const auto &process : std_filesystem::directory_iterator("/proc"))
-  {
+  for (const auto &process : std_filesystem::directory_iterator("/proc")) {
     std::string filename = process.path().filename().string();
     if (!std::all_of(filename.begin(), filename.end(), ::isdigit))
       continue;
@@ -1537,8 +1421,7 @@ std::string sanitise_bpf_program_name(const std::string &name)
 
   // Kernel KSYM_NAME_LEN is 128 until 6.1
   // If we'll exceed the limit, hash the string and cap at 127 (+ null byte).
-  if (sanitised_name.size() > 127)
-  {
+  if (sanitised_name.size() > 127) {
     size_t hash = std::hash<std::string>{}(sanitised_name);
 
     // std::hash returns size_t, so we reserve 2*sizeof(size_t)+1 characters
