@@ -5,6 +5,7 @@
 #include "struct.h"
 #include "gtest/gtest.h"
 #include <iostream>
+#include <llvm/Config/llvm-config.h>
 
 namespace bpftrace {
 namespace test {
@@ -12,8 +13,10 @@ namespace clang_parser {
 
 #include "btf_common.h"
 
-static void parse(const std::string &input, BPFtrace &bpftrace, bool result = true,
-                  const std::string& probe = "kprobe:sys_read { 1 }")
+static void parse(const std::string &input,
+                  BPFtrace &bpftrace,
+                  bool result = true,
+                  const std::string &probe = "kprobe:sys_read { 1 }")
 {
   auto extended_input = input + probe;
   Driver driver(bpftrace);
@@ -40,15 +43,15 @@ TEST(clang_parser, integers)
   ASSERT_EQ(foo->HasField("y"), true);
   ASSERT_EQ(foo->HasField("z"), true);
 
-  EXPECT_EQ(foo->GetField("x").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("x").type.IsIntTy());
   EXPECT_EQ(foo->GetField("x").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("x").offset, 0);
 
-  EXPECT_EQ(foo->GetField("y").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("y").type.IsIntTy());
   EXPECT_EQ(foo->GetField("y").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("y").offset, 4);
 
-  EXPECT_EQ(foo->GetField("z").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("z").type.IsIntTy());
   EXPECT_EQ(foo->GetField("z").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("z").offset, 8);
 }
@@ -68,19 +71,19 @@ TEST(clang_parser, c_union)
   ASSERT_TRUE(foo->HasField("i"));
   ASSERT_TRUE(foo->HasField("l"));
 
-  EXPECT_EQ(foo->GetField("c").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("c").type.IsIntTy());
   EXPECT_EQ(foo->GetField("c").type.GetSize(), 1U);
   EXPECT_EQ(foo->GetField("c").offset, 0);
 
-  EXPECT_EQ(foo->GetField("s").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("s").type.IsIntTy());
   EXPECT_EQ(foo->GetField("s").type.GetSize(), 2U);
   EXPECT_EQ(foo->GetField("s").offset, 0);
 
-  EXPECT_EQ(foo->GetField("i").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("i").type.IsIntTy());
   EXPECT_EQ(foo->GetField("i").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("i").offset, 0);
 
-  EXPECT_EQ(foo->GetField("l").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("l").type.IsIntTy());
   EXPECT_EQ(foo->GetField("l").type.GetSize(), 8U);
   EXPECT_EQ(foo->GetField("l").offset, 0);
 }
@@ -97,7 +100,7 @@ TEST(clang_parser, c_enum)
   ASSERT_EQ(foo->fields.size(), 1U);
   ASSERT_TRUE(foo->HasField("e"));
 
-  EXPECT_EQ(foo->GetField("e").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("e").type.IsIntTy());
   EXPECT_EQ(foo->GetField("e").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("e").offset, 0);
 }
@@ -151,7 +154,7 @@ TEST(clang_parser, string_array)
   ASSERT_EQ(foo->fields.size(), 1U);
   ASSERT_TRUE(foo->HasField("str"));
 
-  EXPECT_EQ(foo->GetField("str").type.type, Type::string);
+  EXPECT_TRUE(foo->GetField("str").type.IsStringTy());
   EXPECT_EQ(foo->GetField("str").type.GetSize(), 32U);
   EXPECT_EQ(foo->GetField("str").offset, 0);
 }
@@ -202,7 +205,8 @@ TEST(clang_parser, nested_struct_no_type)
   BPFtrace bpftrace;
   // bar and baz's struct/union do not have type names, but are not anonymous
   // since they are called bar and baz
-  parse("struct Foo { struct { int x; } bar; union { int y; } baz; }", bpftrace);
+  parse("struct Foo { struct { int x; } bar; union { int y; } baz; }",
+        bpftrace);
 
 #if LLVM_VERSION_MAJOR >= 13
   std::string bar_name = "struct Foo::(unnamed at definitions.h:2:14)";
@@ -228,7 +232,7 @@ TEST(clang_parser, nested_struct_no_type)
   ASSERT_EQ(bar->fields.size(), 1U);
   ASSERT_TRUE(bar->HasField("x"));
 
-  EXPECT_EQ(bar->GetField("x").type.type, Type::integer);
+  EXPECT_TRUE(bar->GetField("x").type.IsIntTy());
   EXPECT_EQ(bar->GetField("x").type.GetSize(), 4U);
   EXPECT_EQ(bar->GetField("x").offset, 0);
 
@@ -236,7 +240,7 @@ TEST(clang_parser, nested_struct_no_type)
   ASSERT_EQ(baz->fields.size(), 1U);
   ASSERT_TRUE(baz->HasField("y"));
 
-  EXPECT_EQ(baz->GetField("y").type.type, Type::integer);
+  EXPECT_TRUE(baz->GetField("y").type.IsIntTy());
   EXPECT_EQ(baz->GetField("y").type.GetSize(), 4U);
   EXPECT_EQ(baz->GetField("y").offset, 0);
 
@@ -275,13 +279,13 @@ TEST(clang_parser, nested_struct_unnamed_fields)
   ASSERT_TRUE(foo->HasField("y"));
   ASSERT_TRUE(foo->HasField("a"));
 
-  EXPECT_EQ(foo->GetField("x").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("x").type.IsIntTy());
   EXPECT_EQ(foo->GetField("x").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("x").offset, 0);
-  EXPECT_EQ(foo->GetField("y").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("y").type.IsIntTy());
   EXPECT_EQ(foo->GetField("y").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("y").offset, 4);
-  EXPECT_EQ(foo->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 8);
 
@@ -289,7 +293,7 @@ TEST(clang_parser, nested_struct_unnamed_fields)
   EXPECT_EQ(bar->fields.size(), 1U);
   EXPECT_TRUE(bar->HasField("z"));
 
-  EXPECT_EQ(bar->GetField("z").type.type, Type::integer);
+  EXPECT_TRUE(bar->GetField("z").type.IsIntTy());
   EXPECT_EQ(bar->GetField("z").type.GetSize(), 4U);
   EXPECT_EQ(bar->GetField("z").offset, 0);
 }
@@ -320,23 +324,23 @@ TEST(clang_parser, nested_struct_anon_union_struct)
   ASSERT_TRUE(foo->HasField("a"));
   ASSERT_TRUE(foo->HasField("z"));
 
-  EXPECT_EQ(foo->GetField("_xy").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("_xy").type.IsIntTy());
   EXPECT_EQ(foo->GetField("_xy").type.GetSize(), 8U);
   EXPECT_EQ(foo->GetField("_xy").offset, 0);
 
-  EXPECT_EQ(foo->GetField("x").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("x").type.IsIntTy());
   EXPECT_EQ(foo->GetField("x").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("x").offset, 0);
 
-  EXPECT_EQ(foo->GetField("y").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("y").type.IsIntTy());
   EXPECT_EQ(foo->GetField("y").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("y").offset, 4);
 
-  EXPECT_EQ(foo->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 8);
 
-  EXPECT_EQ(foo->GetField("z").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("z").type.IsIntTy());
   EXPECT_EQ(foo->GetField("z").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("z").offset, 12);
 }
@@ -355,7 +359,7 @@ TEST(clang_parser, bitfields)
   ASSERT_TRUE(foo->HasField("b"));
   ASSERT_TRUE(foo->HasField("c"));
 
-  EXPECT_EQ(foo->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 0);
   EXPECT_TRUE(foo->GetField("a").bitfield.has_value());
@@ -363,7 +367,7 @@ TEST(clang_parser, bitfields)
   EXPECT_EQ(foo->GetField("a").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("a").bitfield->mask, 0xFFU);
 
-  EXPECT_EQ(foo->GetField("b").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("b").type.IsIntTy());
   EXPECT_EQ(foo->GetField("b").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("b").offset, 1);
   EXPECT_TRUE(foo->GetField("b").bitfield.has_value());
@@ -371,7 +375,7 @@ TEST(clang_parser, bitfields)
   EXPECT_EQ(foo->GetField("b").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("b").bitfield->mask, 0xFFU);
 
-  EXPECT_EQ(foo->GetField("c").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("c").type.IsIntTy());
   EXPECT_EQ(foo->GetField("c").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("c").offset, 2);
   EXPECT_TRUE(foo->GetField("c").bitfield.has_value());
@@ -396,7 +400,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   ASSERT_TRUE(foo->HasField("d"));
   ASSERT_TRUE(foo->HasField("e"));
 
-  EXPECT_EQ(foo->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 0);
   EXPECT_TRUE(foo->GetField("a").bitfield.has_value());
@@ -404,7 +408,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_EQ(foo->GetField("a").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("a").bitfield->mask, 0x1U);
 
-  EXPECT_EQ(foo->GetField("b").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("b").type.IsIntTy());
   EXPECT_EQ(foo->GetField("b").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("b").offset, 0);
   EXPECT_TRUE(foo->GetField("b").bitfield.has_value());
@@ -412,7 +416,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_EQ(foo->GetField("b").bitfield->access_rshift, 1U);
   EXPECT_EQ(foo->GetField("b").bitfield->mask, 0x1U);
 
-  EXPECT_EQ(foo->GetField("c").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("c").type.IsIntTy());
   EXPECT_EQ(foo->GetField("c").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("c").offset, 0);
   EXPECT_TRUE(foo->GetField("c").bitfield.has_value());
@@ -420,7 +424,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_EQ(foo->GetField("c").bitfield->access_rshift, 2U);
   EXPECT_EQ(foo->GetField("c").bitfield->mask, 0x7U);
 
-  EXPECT_EQ(foo->GetField("d").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("d").type.IsIntTy());
   EXPECT_EQ(foo->GetField("d").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("d").offset, 0);
   EXPECT_TRUE(foo->GetField("d").bitfield.has_value());
@@ -428,7 +432,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_EQ(foo->GetField("d").bitfield->access_rshift, 5U);
   EXPECT_EQ(foo->GetField("d").bitfield->mask, 0xFFFFFU);
 
-  EXPECT_EQ(foo->GetField("e").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("e").type.IsIntTy());
   EXPECT_EQ(foo->GetField("e").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("e").offset, 3);
   EXPECT_TRUE(foo->GetField("e").bitfield.has_value());
@@ -452,7 +456,7 @@ TEST(clang_parser, bitfields_with_padding)
   ASSERT_TRUE(foo->HasField("b"));
   ASSERT_TRUE(foo->HasField("end"));
 
-  EXPECT_EQ(foo->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 4);
   EXPECT_TRUE(foo->GetField("a").bitfield.has_value());
@@ -460,7 +464,7 @@ TEST(clang_parser, bitfields_with_padding)
   EXPECT_EQ(foo->GetField("a").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("a").bitfield->mask, 0xFFFFFFFU);
 
-  EXPECT_EQ(foo->GetField("b").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("b").type.IsIntTy());
   EXPECT_EQ(foo->GetField("b").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("b").offset, 7);
   EXPECT_TRUE(foo->GetField("b").bitfield.has_value());
@@ -484,15 +488,15 @@ TEST(clang_parser, builtin_headers)
   ASSERT_TRUE(foo->HasField("y"));
   ASSERT_TRUE(foo->HasField("z"));
 
-  EXPECT_EQ(foo->GetField("x").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("x").type.IsIntTy());
   EXPECT_EQ(foo->GetField("x").type.GetSize(), 8U);
   EXPECT_EQ(foo->GetField("x").offset, 0);
 
-  EXPECT_EQ(foo->GetField("y").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("y").type.IsIntTy());
   EXPECT_EQ(foo->GetField("y").type.GetSize(), 8U);
   EXPECT_EQ(foo->GetField("y").offset, 8);
 
-  EXPECT_EQ(foo->GetField("z").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("z").type.IsIntTy());
   EXPECT_EQ(foo->GetField("z").type.GetSize(), 8U);
   EXPECT_EQ(foo->GetField("z").offset, 16);
 }
@@ -518,9 +522,7 @@ TEST(clang_parser, parse_fail)
   parse("struct a { int a; struct b b; };", bpftrace, false);
 }
 
-class clang_parser_btf : public test_btf
-{
-};
+class clang_parser_btf : public test_btf {};
 
 TEST_F(clang_parser_btf, btf)
 {
@@ -549,15 +551,15 @@ TEST_F(clang_parser_btf, btf)
   ASSERT_TRUE(foo1->HasField("b"));
   ASSERT_TRUE(foo1->HasField("c"));
 
-  EXPECT_EQ(foo1->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo1->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo1->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo1->GetField("a").offset, 0);
 
-  EXPECT_EQ(foo1->GetField("b").type.type, Type::integer);
+  EXPECT_TRUE(foo1->GetField("b").type.IsIntTy());
   EXPECT_EQ(foo1->GetField("b").type.GetSize(), 1U);
   EXPECT_EQ(foo1->GetField("b").offset, 4);
 
-  EXPECT_EQ(foo1->GetField("c").type.type, Type::integer);
+  EXPECT_TRUE(foo1->GetField("c").type.IsIntTy());
   EXPECT_EQ(foo1->GetField("c").type.GetSize(), 8U);
   EXPECT_EQ(foo1->GetField("c").offset, 8);
 
@@ -567,15 +569,15 @@ TEST_F(clang_parser_btf, btf)
   ASSERT_TRUE(foo2->HasField("f"));
   ASSERT_TRUE(foo2->HasField("g"));
 
-  EXPECT_EQ(foo2->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo2->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo2->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo2->GetField("a").offset, 0);
 
-  EXPECT_EQ(foo2->GetField("f").type.type, Type::record);
+  EXPECT_TRUE(foo2->GetField("f").type.IsRecordTy());
   EXPECT_EQ(foo2->GetField("f").type.GetSize(), 16U);
   EXPECT_EQ(foo2->GetField("f").offset, 8);
 
-  EXPECT_EQ(foo2->GetField("g").type.type, Type::integer);
+  EXPECT_TRUE(foo2->GetField("g").type.IsIntTy());
   EXPECT_EQ(foo2->GetField("g").type.GetSize(), 1U);
   EXPECT_EQ(foo2->GetField("g").offset, 8);
 
@@ -604,6 +606,39 @@ TEST_F(clang_parser_btf, btf)
   EXPECT_EQ(foo2_field.offset, 8);
 }
 
+TEST_F(clang_parser_btf, btf_arrays_multi_dim)
+{
+  GTEST_SKIP() << "BTF flattens multi-dimensional arrays #3082";
+
+  BPFtrace bpftrace;
+  bpftrace.parse_btf({});
+  parse("struct Foo { struct Arrays a; };", bpftrace);
+
+  ASSERT_TRUE(bpftrace.structs.Has("struct Arrays"));
+  auto arrs = bpftrace.structs.Lookup("struct Arrays").lock();
+
+  ASSERT_TRUE(arrs->HasField("multi_dim"));
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.IsArrayTy());
+  EXPECT_EQ(arrs->GetField("multi_dim").offset, 40);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetSize(), 24U);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetNumElements(), 3);
+
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.GetElementTy()->IsArrayTy());
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetElementTy()->GetSize(), 8U);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetElementTy()->GetNumElements(),
+            2);
+
+  EXPECT_TRUE(arrs->GetField("multi_dim")
+                  .type.GetElementTy()
+                  ->GetElementTy()
+                  ->IsIntTy());
+  EXPECT_EQ(arrs->GetField("multi_dim")
+                .type.GetElementTy()
+                ->GetElementTy()
+                ->GetSize(),
+            4U);
+}
+
 TEST(clang_parser, btf_unresolved_typedef)
 {
   // size_t is defined in stddef.h, but if we have BTF, it should be possible to
@@ -622,7 +657,7 @@ TEST(clang_parser, btf_unresolved_typedef)
   ASSERT_EQ(foo->fields.size(), 1U);
   ASSERT_TRUE(foo->HasField("x"));
 
-  EXPECT_EQ(foo->GetField("x").type.type, Type::integer);
+  EXPECT_TRUE(foo->GetField("x").type.IsIntTy());
   EXPECT_EQ(foo->GetField("x").type.GetSize(), 8U);
   EXPECT_EQ(foo->GetField("x").offset, 0);
 }
@@ -665,7 +700,8 @@ TEST(clang_parser, struct_typedef)
   // "typedef struct {} max_align_t"
   BPFtrace bpftrace;
   parse("#include <__stddef_max_align_t.h>\n"
-        "struct max_align_t { int x; };", bpftrace);
+        "struct max_align_t { int x; };",
+        bpftrace);
 
   ASSERT_TRUE(bpftrace.structs.Has("struct max_align_t"));
   ASSERT_TRUE(bpftrace.structs.Has("max_align_t"));
@@ -677,7 +713,7 @@ TEST(clang_parser, struct_typedef)
   ASSERT_EQ(max_align_struct->fields.size(), 1U);
   ASSERT_TRUE(max_align_struct->HasField("x"));
 
-  EXPECT_EQ(max_align_struct->GetField("x").type.type, Type::integer);
+  EXPECT_TRUE(max_align_struct->GetField("x").type.IsIntTy());
   EXPECT_EQ(max_align_struct->GetField("x").type.GetSize(), 4U);
   EXPECT_EQ(max_align_struct->GetField("x").offset, 0);
 
@@ -687,16 +723,16 @@ TEST(clang_parser, struct_typedef)
   ASSERT_TRUE(max_align_typedef->HasField("__clang_max_align_nonce1"));
   ASSERT_TRUE(max_align_typedef->HasField("__clang_max_align_nonce2"));
 
-  EXPECT_EQ(max_align_typedef->GetField("__clang_max_align_nonce1").type.type,
-            Type::integer);
+  EXPECT_TRUE(
+      max_align_typedef->GetField("__clang_max_align_nonce1").type.IsIntTy());
   EXPECT_EQ(
       max_align_typedef->GetField("__clang_max_align_nonce1").type.GetSize(),
       8U);
   EXPECT_EQ(max_align_typedef->GetField("__clang_max_align_nonce1").offset, 0);
 
   // double are not parsed correctly yet so these fields are junk for now
-  EXPECT_EQ(max_align_typedef->GetField("__clang_max_align_nonce2").type.type,
-            Type::none);
+  EXPECT_TRUE(
+      max_align_typedef->GetField("__clang_max_align_nonce2").type.IsNoneTy());
   EXPECT_EQ(
       max_align_typedef->GetField("__clang_max_align_nonce2").type.GetSize(),
       0U);
