@@ -4,16 +4,26 @@ namespace bpftrace {
 namespace test {
 namespace codegen {
 
-TEST(codegen, builtin_func)
+TEST(codegen, builtin_func_kprobe)
 {
   test("kprobe:f { @x = func }", NAME);
 }
 
+TEST(codegen, builtin_func_kretprobe)
+{
+  test("kretprobe:f { @x = func }", NAME);
+}
+
 TEST(codegen, builtin_func_uprobe)
 {
   test("uprobe:/bin/sh:f { @x = func }", NAME);
 }
 
+TEST(codegen, builtin_func_uretprobe)
+{
+  test("uretprobe:/bin/sh:f { @x = func }", NAME);
+}
+
 TEST(codegen, builtin_func_kfunc)
 {
   test("kfunc:f { @x = func }", NAME);
