@@ -10,12 +10,7 @@ TEST(codegen, call_kstack)
 {
   auto result = NAME;
 
-  // Mode doesn't directly affect codegen, so the programs below should
-  // generate the same IR
-  test("kprobe:f { @x = kstack(); @y = kstack(6) }", result);
-  test("kprobe:f { @x = kstack(perf); @y = kstack(perf, 6) }", result);
-  test("kprobe:f { @x = kstack(raw); @y = kstack(raw, 6) }", result);
-  test("kprobe:f { @x = kstack(perf); @y = kstack(bpftrace) }", result);
+  test("kprobe:f { @x = kstack(); @y = kstack(6); @z = kstack(perf) }", result);
 }
 
 TEST(codegen, call_kstack_mapids)
@@ -38,21 +33,19 @@ TEST(codegen, call_kstack_mapids)
   ast::ResourceAnalyser resource_analyser(driver.root.get());
   auto resources_optional = resource_analyser.analyse();
   ASSERT_TRUE(resources_optional.has_value());
-  auto resources = resources_optional.value();
-  ASSERT_EQ(resources.create_maps(*bpftrace, true), 0);
-  bpftrace->resources = resources;
+  bpftrace->resources = resources_optional.value();
 
   ast::CodegenLLVM codegen(driver.root.get(), *bpftrace);
-  codegen.compile();
+  bpftrace->bytecode_ = codegen.compile();
 
-  ASSERT_EQ(std::distance(bpftrace->maps.begin(), bpftrace->maps.end()), 7);
-  ASSERT_EQ(bpftrace->maps.CountStackTypes(), 2U);
+  ASSERT_EQ(bpftrace->bytecode_.maps().size(), 8);
+  ASSERT_EQ(bpftrace->bytecode_.countStackMaps(), 3U);
 
   StackType stack_type;
   stack_type.limit = 5;
-  ASSERT_TRUE(bpftrace->maps.Has(stack_type));
+  ASSERT_TRUE(bpftrace->bytecode_.hasMap(stack_type));
   stack_type.limit = 6;
-  ASSERT_TRUE(bpftrace->maps.Has(stack_type));
+  ASSERT_TRUE(bpftrace->bytecode_.hasMap(stack_type));
 }
 
 TEST(codegen, call_kstack_modes_mapids)
@@ -76,23 +69,21 @@ TEST(codegen, call_kstack_modes_mapids)
   ast::ResourceAnalyser resource_analyser(driver.root.get());
   auto resources_optional = resource_analyser.analyse();
   ASSERT_TRUE(resources_optional.has_value());
-  auto resources = resources_optional.value();
-  ASSERT_EQ(resources.create_maps(*bpftrace, true), 0);
-  bpftrace->resources = resources;
+  bpftrace->resources = resources_optional.value();
 
   ast::CodegenLLVM codegen(driver.root.get(), *bpftrace);
-  codegen.compile();
+  bpftrace->bytecode_ = codegen.compile();
 
-  ASSERT_EQ(std::distance(bpftrace->maps.begin(), bpftrace->maps.end()), 9);
-  ASSERT_EQ(bpftrace->maps.CountStackTypes(), 3U);
+  ASSERT_EQ(bpftrace->bytecode_.maps().size(), 10);
+  ASSERT_EQ(bpftrace->bytecode_.countStackMaps(), 4U);
 
   StackType stack_type;
   stack_type.mode = StackMode::perf;
-  ASSERT_TRUE(bpftrace->maps.Has(stack_type));
+  ASSERT_TRUE(bpftrace->bytecode_.hasMap(stack_type));
   stack_type.mode = StackMode::bpftrace;
-  ASSERT_TRUE(bpftrace->maps.Has(stack_type));
+  ASSERT_TRUE(bpftrace->bytecode_.hasMap(stack_type));
   stack_type.mode = StackMode::raw;
-  ASSERT_TRUE(bpftrace->maps.Has(stack_type));
+  ASSERT_TRUE(bpftrace->bytecode_.hasMap(stack_type));
 }
 
 } // namespace codegen
