@@ -6,7 +6,7 @@ namespace codegen {
 
 TEST(codegen, call_printf)
 {
-  test("struct Foo { char c; long l; } kprobe:f { $foo = (struct Foo*)0; "
+  test("struct Foo { char c; long l; } kprobe:f { $foo = (struct Foo*)arg0; "
        "printf(\"%c %lu\\n\", $foo->c, $foo->l) }",
 
        NAME);
