@@ -16,7 +16,6 @@
 #include "bpftrace.h"
 #include "clang_parser.h"
 #include "driver.h"
-#include "fake_map.h"
 #include "tracepoint_format_parser.h"
 
 namespace bpftrace {
@@ -64,9 +63,7 @@ static void test(BPFtrace &bpftrace,
   ast::ResourceAnalyser resource_analyser(driver.root.get());
   auto resources_optional = resource_analyser.analyse();
   ASSERT_TRUE(resources_optional.has_value());
-  auto resources = resources_optional.value();
-  ASSERT_EQ(resources.create_maps(bpftrace, true), 0);
-  bpftrace.resources = resources;
+  bpftrace.resources = resources_optional.value();
 
   std::stringstream out;
   ast::CodegenLLVM codegen(driver.root.get(), bpftrace);
@@ -77,10 +74,9 @@ static void test(BPFtrace &bpftrace,
   codegen.emit();
 
   uint64_t update_tests = 0;
-  if (get_uint64_env_var("BPFTRACE_UPDATE_TESTS",
-                         [&](uint64_t x) { update_tests = x; }) &&
-      update_tests >= 1)
-  {
+  get_uint64_env_var("BPFTRACE_UPDATE_TESTS",
+                     [&](uint64_t x) { update_tests = x; });
+  if (update_tests >= 1) {
     std::cerr << "Running in update mode, test is skipped" << std::endl;
     std::ofstream file(TEST_CODEGEN_LOCATION + name + ".ll");
     file << out.str();
