@@ -1,32 +1,30 @@
-struct Foo1
-{
+struct Foo1 {
   int a;
   char b;
   long c;
 };
 
-struct Foo2
-{
+struct Foo2 {
   int a;
-  union
-  {
+  union {
     struct Foo1 f;
-    struct
-    {
+    struct {
       char g;
     };
   };
 };
 
-struct Foo3
-{
+struct Foo3 {
   struct Foo1 *foo1;
   const volatile struct Foo2 *restrict foo2;
 };
 
 struct Foo3 foo3;
 
-struct Foo3 *func_1(int a, struct Foo1 *foo1, struct Foo2 *foo2)
+struct Foo3 *func_1(int a,
+                    struct Foo1 *foo1,
+                    struct Foo2 *foo2,
+                    struct Foo3 *foo3)
 {
   return 0;
 }
@@ -36,13 +34,23 @@ struct Foo3 *func_2(int a, int *b, struct Foo1 *foo1)
   return 0;
 }
 
-struct Foo3 *func_3(int a, int *b, struct Foo1 *foo1)
+// __attribute__((noinline)) is needed due to a LLDB/GCC compatibility bug
+struct Foo3 *__attribute__((noinline)) func_3(int a, int *b, struct Foo1 *foo1)
 {
   return 0;
 }
 
-struct Arrays
-{
+struct FirstFieldsAreAnonUnion {
+  union {
+    int a;
+    int b;
+  };
+  int c;
+};
+
+struct FirstFieldsAreAnonUnion first_fields_anon_union;
+
+struct Arrays {
   int int_arr[4];
   char char_arr[8];
   void *ptr_arr[2];
@@ -52,8 +60,12 @@ struct Arrays
 };
 struct Arrays arrays;
 
-struct task_struct
+struct Arrays *func_arrays(struct Arrays *arr)
 {
+  return 0;
+}
+
+struct task_struct {
   int pid;
   int pgid;
   int : 12; // padding
@@ -63,30 +75,25 @@ struct task_struct
   int d : 20;
 };
 
-struct file
-{
+struct file {
   int ino;
 };
 
-struct vm_area_struct
-{
+struct vm_area_struct {
   unsigned long vm_start;
   unsigned long vm_end;
 };
 
-struct bpf_iter__task
-{
+struct bpf_iter__task {
   struct task_struct *task;
 };
 
-struct bpf_iter__task_file
-{
+struct bpf_iter__task_file {
   struct task_struct *task;
   struct file *file;
 };
 
-struct bpf_iter__task_vma
-{
+struct bpf_iter__task_vma {
   struct task_struct *task;
   struct vm_area_struct *vma;
 };
@@ -112,7 +119,7 @@ int main(void)
   struct bpf_iter__task_file iter_task_file;
   struct bpf_iter__task_vma iter_task_vma;
 
-  func_1(0, 0, 0);
+  func_1(0, 0, 0, 0);
 
   bpf_iter_task();
   bpf_iter_task_file();
