@@ -30,9 +30,7 @@ void test(const std::string &input, int expected_result = 0)
   test(*bpftrace, input, expected_result);
 }
 
-class field_analyser_btf : public test_btf
-{
-};
+class field_analyser_btf : public test_btf {};
 
 TEST_F(field_analyser_btf, kfunc_args)
 {
@@ -81,15 +79,15 @@ TEST_F(field_analyser_btf, btf_types)
   ASSERT_TRUE(foo1->HasField("b"));
   ASSERT_TRUE(foo1->HasField("c"));
 
-  EXPECT_EQ(foo1->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo1->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo1->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo1->GetField("a").offset, 0);
 
-  EXPECT_EQ(foo1->GetField("b").type.type, Type::integer);
+  EXPECT_TRUE(foo1->GetField("b").type.IsIntTy());
   EXPECT_EQ(foo1->GetField("b").type.GetSize(), 1U);
   EXPECT_EQ(foo1->GetField("b").offset, 4);
 
-  EXPECT_EQ(foo1->GetField("c").type.type, Type::integer);
+  EXPECT_TRUE(foo1->GetField("c").type.IsIntTy());
   EXPECT_EQ(foo1->GetField("c").type.GetSize(), 8U);
   EXPECT_EQ(foo1->GetField("c").offset, 8);
 
@@ -99,15 +97,15 @@ TEST_F(field_analyser_btf, btf_types)
   ASSERT_TRUE(foo2->HasField("f"));
   ASSERT_TRUE(foo2->HasField("g"));
 
-  EXPECT_EQ(foo2->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(foo2->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo2->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo2->GetField("a").offset, 0);
 
-  EXPECT_EQ(foo2->GetField("f").type.type, Type::record);
+  EXPECT_TRUE(foo2->GetField("f").type.IsRecordTy());
   EXPECT_EQ(foo2->GetField("f").type.GetSize(), 16U);
   EXPECT_EQ(foo2->GetField("f").offset, 8);
 
-  EXPECT_EQ(foo2->GetField("g").type.type, Type::integer);
+  EXPECT_TRUE(foo2->GetField("g").type.IsIntTy());
   EXPECT_EQ(foo2->GetField("g").type.GetSize(), 1U);
   EXPECT_EQ(foo2->GetField("g").offset, 8);
 
@@ -149,43 +147,81 @@ TEST_F(field_analyser_btf, btf_arrays)
   ASSERT_TRUE(arrs->HasField("zero"));
   ASSERT_TRUE(arrs->HasField("flexible"));
 
-  EXPECT_EQ(arrs->GetField("int_arr").type.type, Type::array);
+  EXPECT_TRUE(arrs->GetField("int_arr").type.IsArrayTy());
   EXPECT_EQ(arrs->GetField("int_arr").type.GetNumElements(), 4);
-  EXPECT_EQ(arrs->GetField("int_arr").type.GetElementTy()->type, Type::integer);
+  EXPECT_TRUE(arrs->GetField("int_arr").type.GetElementTy()->IsIntTy());
   EXPECT_EQ(arrs->GetField("int_arr").type.GetSize(), 16U);
   EXPECT_EQ(arrs->GetField("int_arr").offset, 0);
 
-  EXPECT_EQ(arrs->GetField("char_arr").type.type, Type::string);
+  EXPECT_TRUE(arrs->GetField("char_arr").type.IsStringTy());
   EXPECT_EQ(arrs->GetField("char_arr").type.GetSize(), 8U);
   EXPECT_EQ(arrs->GetField("char_arr").offset, 16);
 
-  EXPECT_EQ(arrs->GetField("ptr_arr").type.type, Type::array);
+  EXPECT_TRUE(arrs->GetField("ptr_arr").type.IsArrayTy());
   EXPECT_EQ(arrs->GetField("ptr_arr").type.GetNumElements(), 2);
-  EXPECT_EQ(arrs->GetField("ptr_arr").type.GetElementTy()->type, Type::pointer);
+  EXPECT_TRUE(arrs->GetField("ptr_arr").type.GetElementTy()->IsPtrTy());
   EXPECT_EQ(arrs->GetField("ptr_arr").type.GetSize(), 2 * sizeof(uintptr_t));
   EXPECT_EQ(arrs->GetField("ptr_arr").offset, 24);
 
-  EXPECT_EQ(arrs->GetField("multi_dim").type.type, Type::array);
+  // BTF flattens multi-dimensional arrays, so this test doesn't
+  // check the correct number of elements. The correct values are
+  // below in 'field_analyser_btf.btf_arrays_multi_dim'.
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.IsArrayTy());
   EXPECT_EQ(arrs->GetField("multi_dim").type.GetNumElements(), 6);
-  EXPECT_EQ(arrs->GetField("multi_dim").type.GetElementTy()->type,
-            Type::integer);
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.GetElementTy()->IsIntTy());
   EXPECT_EQ(arrs->GetField("multi_dim").type.GetSize(), 24U);
   EXPECT_EQ(arrs->GetField("multi_dim").offset, 40);
 
-  EXPECT_EQ(arrs->GetField("zero").type.type, Type::array);
+  EXPECT_TRUE(arrs->GetField("zero").type.IsArrayTy());
   EXPECT_EQ(arrs->GetField("zero").type.GetNumElements(), 0);
-  EXPECT_EQ(arrs->GetField("zero").type.GetElementTy()->type, Type::integer);
+  EXPECT_TRUE(arrs->GetField("zero").type.GetElementTy()->IsIntTy());
   EXPECT_EQ(arrs->GetField("zero").type.GetSize(), 0U);
   EXPECT_EQ(arrs->GetField("zero").offset, 64);
 
-  EXPECT_EQ(arrs->GetField("flexible").type.type, Type::array);
+  EXPECT_TRUE(arrs->GetField("flexible").type.IsArrayTy());
   EXPECT_EQ(arrs->GetField("flexible").type.GetNumElements(), 0);
-  EXPECT_EQ(arrs->GetField("flexible").type.GetElementTy()->type,
-            Type::integer);
+  EXPECT_TRUE(arrs->GetField("flexible").type.GetElementTy()->IsIntTy());
   EXPECT_EQ(arrs->GetField("flexible").type.GetSize(), 0U);
   EXPECT_EQ(arrs->GetField("flexible").offset, 64);
 }
 
+TEST_F(field_analyser_btf, btf_arrays_multi_dim)
+{
+  GTEST_SKIP() << "BTF flattens multi-dimensional arrays #3082";
+
+  BPFtrace bpftrace;
+  bpftrace.parse_btf({});
+  test(bpftrace,
+       "BEGIN {\n"
+       "  @ = (struct Arrays *) 0;\n"
+       "}",
+       0);
+
+  ASSERT_TRUE(bpftrace.structs.Has("struct Arrays"));
+  auto arrs = bpftrace.structs.Lookup("struct Arrays").lock();
+
+  ASSERT_TRUE(arrs->HasField("multi_dim"));
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.IsArrayTy());
+  EXPECT_EQ(arrs->GetField("multi_dim").offset, 40);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetSize(), 24U);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetNumElements(), 3);
+
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.GetElementTy()->IsArrayTy());
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetElementTy()->GetSize(), 8U);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetElementTy()->GetNumElements(),
+            2);
+
+  EXPECT_TRUE(arrs->GetField("multi_dim")
+                  .type.GetElementTy()
+                  ->GetElementTy()
+                  ->IsIntTy());
+  EXPECT_EQ(arrs->GetField("multi_dim")
+                .type.GetElementTy()
+                ->GetElementTy()
+                ->GetSize(),
+            4U);
+}
+
 TEST_F(field_analyser_btf, btf_types_struct_ptr)
 {
   BPFtrace bpftrace;
@@ -212,6 +248,31 @@ TEST_F(field_analyser_btf, btf_types_struct_ptr)
   ASSERT_EQ(foo3->fields.size(), 2U); // fields are resolved
 }
 
+TEST_F(field_analyser_btf, btf_types_arr_access)
+{
+  BPFtrace bpftrace;
+  bpftrace.parse_btf({});
+  test(bpftrace,
+       "kfunc:func_1 {\n"
+       "  @foo2 = args.foo3[0].foo2;\n"
+       "}",
+       0);
+
+  // args.foo3[0].foo2 should do 2 things:
+  // - add struct Foo2 (without resolving its fields)
+  // - resolve fields of struct Foo3
+
+  ASSERT_TRUE(bpftrace.structs.Has("struct Foo2"));
+  ASSERT_TRUE(bpftrace.structs.Has("struct Foo3"));
+  auto foo2 = bpftrace.structs.Lookup("struct Foo2").lock();
+  auto foo3 = bpftrace.structs.Lookup("struct Foo3").lock();
+
+  EXPECT_EQ(foo2->size, 24);
+  ASSERT_EQ(foo2->fields.size(), 0U); // fields are not resolved
+  EXPECT_EQ(foo3->size, 16);
+  ASSERT_EQ(foo3->fields.size(), 2U); // fields are resolved
+}
+
 TEST_F(field_analyser_btf, btf_types_bitfields)
 {
   BPFtrace bpftrace;
@@ -222,7 +283,7 @@ TEST_F(field_analyser_btf, btf_types_bitfields)
   auto task_struct = bpftrace.structs.Lookup("struct task_struct").lock();
 
   ASSERT_TRUE(task_struct->HasField("a"));
-  EXPECT_EQ(task_struct->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("a").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(task_struct->GetField("a").offset, 9);
   EXPECT_TRUE(task_struct->GetField("a").bitfield.has_value());
@@ -231,7 +292,7 @@ TEST_F(field_analyser_btf, btf_types_bitfields)
   EXPECT_EQ(task_struct->GetField("a").bitfield->mask, 0xFFU);
 
   ASSERT_TRUE(task_struct->HasField("b"));
-  EXPECT_EQ(task_struct->GetField("b").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("b").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("b").type.GetSize(), 4U);
   EXPECT_EQ(task_struct->GetField("b").offset, 10);
   EXPECT_TRUE(task_struct->GetField("b").bitfield.has_value());
@@ -240,7 +301,7 @@ TEST_F(field_analyser_btf, btf_types_bitfields)
   EXPECT_EQ(task_struct->GetField("b").bitfield->mask, 0x1U);
 
   ASSERT_TRUE(task_struct->HasField("c"));
-  EXPECT_EQ(task_struct->GetField("c").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("c").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("c").type.GetSize(), 4U);
   EXPECT_EQ(task_struct->GetField("c").offset, 10);
   EXPECT_TRUE(task_struct->GetField("c").bitfield.has_value());
@@ -249,7 +310,7 @@ TEST_F(field_analyser_btf, btf_types_bitfields)
   EXPECT_EQ(task_struct->GetField("c").bitfield->mask, 0x7U);
 
   ASSERT_TRUE(task_struct->HasField("d"));
-  EXPECT_EQ(task_struct->GetField("d").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("d").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("d").type.GetSize(), 4U);
   EXPECT_EQ(task_struct->GetField("d").offset, 12);
   EXPECT_TRUE(task_struct->GetField("d").bitfield.has_value());
@@ -258,13 +319,37 @@ TEST_F(field_analyser_btf, btf_types_bitfields)
   EXPECT_EQ(task_struct->GetField("d").bitfield->mask, 0xFFFFFU);
 }
 
-#ifdef HAVE_LIBDW
+TEST_F(field_analyser_btf, btf_anon_union_first_in_struct)
+{
+  BPFtrace bpftrace;
+  bpftrace.parse_btf({});
+  test(bpftrace, "BEGIN { @ = (struct FirstFieldsAreAnonUnion *)0; }");
+
+  ASSERT_TRUE(bpftrace.structs.Has("struct FirstFieldsAreAnonUnion"));
+  auto record =
+      bpftrace.structs.Lookup("struct FirstFieldsAreAnonUnion").lock();
+
+  ASSERT_TRUE(record->HasField("a"));
+  EXPECT_TRUE(record->GetField("a").type.IsIntTy());
+  EXPECT_EQ(record->GetField("a").type.GetSize(), 4U);
+  EXPECT_EQ(record->GetField("a").offset, 0);
+
+  ASSERT_TRUE(record->HasField("b"));
+  EXPECT_TRUE(record->GetField("b").type.IsIntTy());
+  EXPECT_EQ(record->GetField("b").type.GetSize(), 4U);
+  EXPECT_EQ(record->GetField("b").offset, 0);
+
+  ASSERT_TRUE(record->HasField("c"));
+  EXPECT_TRUE(record->GetField("c").type.IsIntTy());
+  EXPECT_EQ(record->GetField("c").type.GetSize(), 4U);
+  EXPECT_EQ(record->GetField("c").offset, 4);
+}
+
+#ifdef HAVE_LIBLLDB
 
 #include "dwarf_common.h"
 
-class field_analyser_dwarf : public test_dwarf
-{
-};
+class field_analyser_dwarf : public test_dwarf {};
 
 TEST_F(field_analyser_dwarf, uprobe_args)
 {
@@ -322,6 +407,108 @@ TEST_F(field_analyser_dwarf, parse_struct)
   ASSERT_EQ(str->GetField("c").type.GetSize(), 8);
 }
 
+TEST_F(field_analyser_dwarf, parse_arrays)
+{
+  BPFtrace bpftrace;
+  std::string uprobe = "uprobe:" + std::string(bin_);
+  test(bpftrace,
+       uprobe + ":func_arrays {\n"
+                "  @ = (struct Arrays *) args.arr;\n"
+                "}");
+
+  ASSERT_TRUE(bpftrace.structs.Has("struct Arrays"));
+  auto arrs = bpftrace.structs.Lookup("struct Arrays").lock();
+
+  EXPECT_EQ(arrs->size, 64);
+  ASSERT_EQ(arrs->fields.size(), 6U);
+  ASSERT_TRUE(arrs->HasField("int_arr"));
+  ASSERT_TRUE(arrs->HasField("char_arr"));
+  ASSERT_TRUE(arrs->HasField("ptr_arr"));
+  ASSERT_TRUE(arrs->HasField("multi_dim"));
+  ASSERT_TRUE(arrs->HasField("zero"));
+  ASSERT_TRUE(arrs->HasField("flexible"));
+
+  EXPECT_TRUE(arrs->GetField("int_arr").type.IsArrayTy());
+  EXPECT_EQ(arrs->GetField("int_arr").type.GetNumElements(), 4);
+  EXPECT_TRUE(arrs->GetField("int_arr").type.GetElementTy()->IsIntTy());
+  EXPECT_EQ(arrs->GetField("int_arr").type.GetSize(), 16U);
+  EXPECT_EQ(arrs->GetField("int_arr").offset, 0);
+
+  EXPECT_TRUE(arrs->GetField("char_arr").type.IsStringTy());
+  EXPECT_EQ(arrs->GetField("char_arr").type.GetSize(), 8U);
+  EXPECT_EQ(arrs->GetField("char_arr").offset, 16);
+
+  EXPECT_TRUE(arrs->GetField("ptr_arr").type.IsArrayTy());
+  EXPECT_EQ(arrs->GetField("ptr_arr").type.GetNumElements(), 2);
+  EXPECT_TRUE(arrs->GetField("ptr_arr").type.GetElementTy()->IsPtrTy());
+  EXPECT_EQ(arrs->GetField("ptr_arr").type.GetSize(), 2 * sizeof(uintptr_t));
+  EXPECT_EQ(arrs->GetField("ptr_arr").offset, 24);
+
+  ASSERT_TRUE(arrs->HasField("multi_dim"));
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.IsArrayTy());
+  EXPECT_EQ(arrs->GetField("multi_dim").offset, 40);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetSize(), 24U);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetNumElements(), 3);
+
+  EXPECT_TRUE(arrs->GetField("multi_dim").type.GetElementTy()->IsArrayTy());
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetElementTy()->GetSize(), 8U);
+  EXPECT_EQ(arrs->GetField("multi_dim").type.GetElementTy()->GetNumElements(),
+            2);
+
+  EXPECT_TRUE(arrs->GetField("multi_dim")
+                  .type.GetElementTy()
+                  ->GetElementTy()
+                  ->IsIntTy());
+  EXPECT_EQ(arrs->GetField("multi_dim")
+                .type.GetElementTy()
+                ->GetElementTy()
+                ->GetSize(),
+            4U);
+
+  EXPECT_TRUE(arrs->GetField("zero").type.IsArrayTy());
+  EXPECT_EQ(arrs->GetField("zero").type.GetNumElements(), 0);
+  EXPECT_TRUE(arrs->GetField("zero").type.GetElementTy()->IsIntTy());
+  EXPECT_EQ(arrs->GetField("zero").type.GetSize(), 0U);
+  EXPECT_EQ(arrs->GetField("zero").offset, 64);
+
+  EXPECT_TRUE(arrs->GetField("flexible").type.IsArrayTy());
+  EXPECT_EQ(arrs->GetField("flexible").type.GetNumElements(), 0);
+  EXPECT_TRUE(arrs->GetField("flexible").type.GetElementTy()->IsIntTy());
+  EXPECT_EQ(arrs->GetField("flexible").type.GetSize(), 0U);
+  EXPECT_EQ(arrs->GetField("flexible").offset, 64);
+}
+
+TEST_F(field_analyser_dwarf, parse_struct_anonymous_fields)
+{
+  GTEST_SKIP() << "Anonymous fields not supported #3084";
+
+  BPFtrace bpftrace;
+  std::string uprobe = "uprobe:" + std::string(bin_);
+  test(bpftrace, uprobe + ":func_1 { $x = args.foo2->g; }", 0);
+
+  ASSERT_TRUE(bpftrace.structs.Has("struct Foo2"));
+  auto str = bpftrace.structs.Lookup("struct Foo2").lock();
+
+  ASSERT_TRUE(str->HasFields());
+  ASSERT_EQ(str->fields.size(), 3);
+  ASSERT_EQ(str->size, 72);
+
+  ASSERT_TRUE(str->HasField("a"));
+  ASSERT_TRUE(str->GetField("a").type.IsIntTy());
+  ASSERT_EQ(str->GetField("a").type.GetSize(), 4);
+  ASSERT_EQ(str->GetField("a").offset, 0);
+
+  ASSERT_TRUE(str->HasField("f"));
+  ASSERT_TRUE(str->GetField("f").type.IsRecordTy());
+  ASSERT_EQ(str->GetField("f").type.GetSize(), 64);
+  ASSERT_EQ(str->GetField("f").offset, 8);
+
+  ASSERT_TRUE(str->HasField("g"));
+  ASSERT_TRUE(str->GetField("g").type.IsIntTy());
+  ASSERT_EQ(str->GetField("g").type.GetSize(), 1);
+  ASSERT_EQ(str->GetField("g").offset, 8);
+}
+
 TEST_F(field_analyser_dwarf, dwarf_types_bitfields)
 {
   BPFtrace bpftrace;
@@ -334,68 +521,59 @@ TEST_F(field_analyser_dwarf, dwarf_types_bitfields)
   auto task_struct = bpftrace.structs.Lookup("struct task_struct").lock();
 
   ASSERT_TRUE(task_struct->HasField("a"));
-  EXPECT_EQ(task_struct->GetField("a").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("a").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("a").type.GetSize(), 4U);
   EXPECT_TRUE(task_struct->GetField("a").bitfield.has_value());
 
   EXPECT_TRUE(task_struct->GetField("a").offset == 8 ||
               task_struct->GetField("a").offset == 9);
-  if (task_struct->GetField("a").offset == 8)
-  { // DWARF < 4
+  if (task_struct->GetField("a").offset == 8) { // DWARF < 4
     EXPECT_EQ(task_struct->GetField("a").bitfield->read_bytes, 0x3U);
     EXPECT_EQ(task_struct->GetField("a").bitfield->access_rshift, 12U);
     EXPECT_EQ(task_struct->GetField("a").bitfield->mask, 0xFFU);
-  }
-  else
-  { // DWARF >= 4
+  } else { // DWARF >= 4
     EXPECT_EQ(task_struct->GetField("a").bitfield->read_bytes, 0x2U);
     EXPECT_EQ(task_struct->GetField("a").bitfield->access_rshift, 4U);
     EXPECT_EQ(task_struct->GetField("a").bitfield->mask, 0xFFU);
   }
 
   ASSERT_TRUE(task_struct->HasField("b"));
-  EXPECT_EQ(task_struct->GetField("b").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("b").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("b").type.GetSize(), 4U);
   EXPECT_TRUE(task_struct->GetField("b").bitfield.has_value());
 
   EXPECT_TRUE(task_struct->GetField("b").offset == 8 ||
               task_struct->GetField("b").offset == 10);
-  if (task_struct->GetField("b").offset == 8)
-  { // DWARF < 4
+  if (task_struct->GetField("b").offset == 8) { // DWARF < 4
     EXPECT_EQ(task_struct->GetField("b").bitfield->read_bytes, 0x3U);
     EXPECT_EQ(task_struct->GetField("b").bitfield->access_rshift, 20U);
     EXPECT_EQ(task_struct->GetField("b").bitfield->mask, 0x1U);
-  }
-  else
-  { // DWARF >= 4
+  } else { // DWARF >= 4
     EXPECT_EQ(task_struct->GetField("b").bitfield->read_bytes, 0x1U);
     EXPECT_EQ(task_struct->GetField("b").bitfield->access_rshift, 4U);
     EXPECT_EQ(task_struct->GetField("b").bitfield->mask, 0x1U);
   }
 
   ASSERT_TRUE(task_struct->HasField("c"));
-  EXPECT_EQ(task_struct->GetField("c").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("c").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("c").type.GetSize(), 4U);
   EXPECT_TRUE(task_struct->GetField("c").bitfield.has_value());
 
   EXPECT_TRUE(task_struct->GetField("c").offset == 8 ||
               task_struct->GetField("c").offset == 10);
 
-  if (task_struct->GetField("c").offset == 8)
-  { // DWARF < 4
+  if (task_struct->GetField("c").offset == 8) { // DWARF < 4
     EXPECT_EQ(task_struct->GetField("c").bitfield->read_bytes, 0x3U);
     EXPECT_EQ(task_struct->GetField("c").bitfield->access_rshift, 21U);
     EXPECT_EQ(task_struct->GetField("c").bitfield->mask, 0x7U);
-  }
-  else
-  { // DWARF >= 4
+  } else { // DWARF >= 4
     EXPECT_EQ(task_struct->GetField("c").bitfield->read_bytes, 0x1U);
     EXPECT_EQ(task_struct->GetField("c").bitfield->access_rshift, 5U);
     EXPECT_EQ(task_struct->GetField("c").bitfield->mask, 0x7U);
   }
 
   ASSERT_TRUE(task_struct->HasField("d"));
-  EXPECT_EQ(task_struct->GetField("d").type.type, Type::integer);
+  EXPECT_TRUE(task_struct->GetField("d").type.IsIntTy());
   EXPECT_EQ(task_struct->GetField("d").type.GetSize(), 4U);
   EXPECT_EQ(task_struct->GetField("d").offset, 12);
   EXPECT_TRUE(task_struct->GetField("d").bitfield.has_value());
@@ -404,7 +582,12 @@ TEST_F(field_analyser_dwarf, dwarf_types_bitfields)
   EXPECT_EQ(task_struct->GetField("d").bitfield->mask, 0xFFFFFU);
 }
 
-#endif // HAVE_LIBDW
+TEST(field_analyser_subprog, struct_cast)
+{
+  test("struct x { int a; } fn f(): void { $s = (struct x *)0; }", 0);
+}
+
+#endif // HAVE_LIBLLDB
 
 } // namespace field_analyser
 } // namespace test
