@@ -23,6 +23,16 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
         return myval;
       });
 
+  ON_CALL(matcher, get_symbols_from_traceable_funcs(true))
+      .WillByDefault([](void) {
+        std::string ksyms = "kernel_mod:func_in_mod\n"
+                            "kernel_mod:other_func_in_mod\n"
+                            "other_kernel_mod:func_in_mod\n";
+        auto myval = std::unique_ptr<std::istream>(
+            new std::istringstream(ksyms));
+        return myval;
+      });
+
   ON_CALL(matcher, get_symbols_from_file(tracefs::available_events()))
       .WillByDefault([](const std::string &) {
         std::string tracepoints = "sched:sched_one\n"
@@ -31,8 +41,10 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
                                   "sched_extra:sched_extra\n"
                                   "notsched:bar\n"
                                   "file:filename\n"
-                                  "tcp:some_tcp_tp\n";
-        return std::unique_ptr<std::istream>(new std::istringstream(tracepoints));
+                                  "tcp:some_tcp_tp\n"
+                                  "btf:tag\n";
+        return std::unique_ptr<std::istream>(
+            new std::istringstream(tracepoints));
       });
 
   std::string sh_usyms = "/bin/sh:first_open\n"
@@ -44,7 +56,6 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
                          "/bin/sh:_Z11cpp_mangledv\n"
                          "/bin/sh:_Z18cpp_mangled_suffixv\n";
   std::string bash_usyms = "/bin/bash:first_open\n";
-  std::string proc_usyms = "/proc/1234/exe:third_open\n";
   ON_CALL(matcher, get_func_symbols_from_file(_, "/bin/sh"))
       .WillByDefault([sh_usyms](int, const std::string &) {
         return std::unique_ptr<std::istream>(new std::istringstream(sh_usyms));
@@ -55,12 +66,6 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
         return std::unique_ptr<std::istream>(
             new std::istringstream(sh_usyms + bash_usyms));
       });
-  ON_CALL(matcher, get_func_symbols_from_file(_, "*"))
-      .WillByDefault(
-          [sh_usyms, bash_usyms, proc_usyms](int, const std::string &) {
-            return std::unique_ptr<std::istream>(
-                new std::istringstream(sh_usyms + bash_usyms + proc_usyms));
-          });
 
   std::string sh_usdts = "/bin/sh:prov1:tp1\n"
                          "/bin/sh:prov1:tp2\n"
@@ -68,7 +73,6 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
                          "/bin/sh:prov2:notatp\n"
                          "/bin/sh:nahprov:tp\n";
   std::string bash_usdts = "/bin/bash:prov1:tp3\n";
-  std::string proc_usdts = "/proc/1234/exe:prov2:tp4\n";
   ON_CALL(matcher, get_symbols_from_usdt(_, "/bin/sh"))
       .WillByDefault([sh_usdts](int, const std::string &) {
         return std::unique_ptr<std::istream>(new std::istringstream(sh_usdts));
@@ -78,12 +82,6 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
         return std::unique_ptr<std::istream>(
             new std::istringstream(sh_usdts + bash_usdts));
       });
-  ON_CALL(matcher, get_symbols_from_usdt(_, "*"))
-      .WillByDefault(
-          [sh_usdts, bash_usdts, proc_usdts](int, const std::string &) {
-            return std::unique_ptr<std::istream>(
-                new std::istringstream(sh_usdts + bash_usdts + proc_usdts));
-          });
 }
 
 void setup_mock_bpftrace(MockBPFtrace &bpftrace)
@@ -127,6 +125,17 @@ void setup_mock_bpftrace(MockBPFtrace &bpftrace)
   bpftrace.structs.Lookup("struct _tracepoint_file_filename")
       .lock()
       ->AddField("filename", ptr_type, 8, std::nullopt, false);
+  bpftrace.structs.Add("struct _tracepoint_btf_tag", 16);
+  auto ptr_type_w_tag = CreatePointer(CreateInt8());
+  ptr_type_w_tag.SetBtfTypeTags({ "rcu" });
+  auto ptr_type_w_bad_tag = CreatePointer(CreateInt8());
+  ptr_type_w_bad_tag.SetBtfTypeTags({ "rcu", "percpu" });
+  bpftrace.structs.Lookup("struct _tracepoint_btf_tag")
+      .lock()
+      ->AddField("parent", ptr_type_w_tag, 8, std::nullopt, false);
+  bpftrace.structs.Lookup("struct _tracepoint_btf_tag")
+      .lock()
+      ->AddField("real_parent", ptr_type_w_bad_tag, 16, std::nullopt, false);
 }
 
 std::unique_ptr<MockBPFtrace> get_mock_bpftrace()
@@ -155,5 +164,26 @@ std::unique_ptr<MockBPFtrace> get_strict_mock_bpftrace()
   return bpftrace;
 }
 
+std::unique_ptr<MockUSDTHelper> get_mock_usdt_helper(int num_locations)
+{
+  auto usdt_helper = std::make_unique<NiceMock<MockUSDTHelper>>();
+
+  ON_CALL(*usdt_helper, find(_, _, _, _))
+      .WillByDefault([num_locations](int,
+                                     const std::string &,
+                                     const std::string &,
+                                     const std::string &) {
+        return usdt_probe_entry{
+          .path = "",
+          .provider = "",
+          .name = "",
+          .semaphore_offset = 0,
+          .num_locations = num_locations,
+        };
+      });
+
+  return usdt_helper;
+}
+
 } // namespace test
 } // namespace bpftrace
