@@ -7,7 +7,6 @@
 #include "bpftrace.h"
 #include "clang_parser.h"
 #include "driver.h"
-#include "fake_map.h"
 #include "mocks.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
@@ -43,9 +42,7 @@ void gen_bytecode(const std::string &input, std::stringstream &out)
   ast::ResourceAnalyser resource_analyser(driver.root.get());
   auto resources_optional = resource_analyser.analyse();
   ASSERT_TRUE(resources_optional.has_value());
-  auto resources = resources_optional.value();
-  ASSERT_EQ(resources.create_maps(*bpftrace, true), 0);
-  bpftrace->resources = resources;
+  bpftrace->resources = resources_optional.value();
 
   ast::CodegenLLVM codegen(driver.root.get(), *bpftrace);
   codegen.generate_ir();
@@ -70,14 +67,13 @@ TEST(probe, short_name)
   compare_bytecode("kretprobe:f { pid }", "kr:f { pid }");
   compare_bytecode("uprobe:sh:f { 1 }", "u:sh:f { 1 }");
   compare_bytecode("profile:hz:997 { 1 }", "p:hz:997 { 1 }");
-  compare_bytecode("hardware:cache-references:1000000 { 1 }", "h:cache-references:1000000 { 1 }");
+  compare_bytecode("hardware:cache-references:1000000 { 1 }",
+                   "h:cache-references:1000000 { 1 }");
   compare_bytecode("software:faults:1000 { 1 }", "s:faults:1000 { 1 }");
   compare_bytecode("interval:s:1 { 1 }", "i:s:1 { 1 }");
 }
 
-class probe_btf : public test_btf
-{
-};
+class probe_btf : public test_btf {};
 
 TEST_F(probe_btf, short_name)
 {
