@@ -13,7 +13,7 @@ from functools import lru_cache
 import cmake_vars
 
 BPFTRACE_BIN = os.environ["BPFTRACE_RUNTIME_TEST_EXECUTABLE"]
-AOT_BIN = os.environ["BPFTRACE_AOT_RUNTIME_TEST_EXECUTABLE"]
+COLOR_SETTING = os.environ.get("RUNTIME_TEST_COLOR", "auto")
 ATTACH_TIMEOUT = 10
 DEFAULT_TIMEOUT = 5
 
@@ -23,9 +23,17 @@ WARN_COLOR = '\033[94m'
 ERROR_COLOR = '\033[91m'
 NO_COLOR = '\033[0m'
 
-# TODO(mmarchini) only add colors if terminal supports it
 def colorify(s, color):
-    return "%s%s%s" % (color, s, NO_COLOR) if sys.stdout.isatty() else s
+    if COLOR_SETTING == "yes":
+        use_color = True
+    elif COLOR_SETTING == "auto":
+        use_color = sys.stdout.isatty()
+    elif COLOR_SETTING == "no":
+        use_color = False
+    else:
+        raise ValueError("Invalid setting for RUNTIME_TEST_COLOR")
+
+    return f"{color}{s}{NO_COLOR}" if use_color else s
 
 def ok(s):
     return colorify(s, OK_COLOR)
@@ -94,15 +102,15 @@ class Runner(object):
 
         if test.run:
             ret = re.sub("{{BPFTRACE}}", BPFTRACE_BIN, test.run)
-            ret = re.sub("{{BPFTRACE_AOTRT}}", AOT_BIN, ret)
 
             return nsenter_prefix + ret
         else:  # PROG
-            use_json = "-q -f json" if test.expect_mode == "json" else ""
-            cmd = nsenter_prefix + "{} {} -e '{}'".format(BPFTRACE_BIN, use_json, test.prog)
+            use_json = "-q -f json" if test.expects[0].mode == "json" else ""
+            escaped_prog = test.prog.replace("'", "'\\''")
+            cmd = nsenter_prefix + "{} {} -e '{}'".format(BPFTRACE_BIN, use_json, escaped_prog)
             # We're only reusing PROG-directive tests for AOT tests
             if test.suite == 'aot':
-                return cmd + " --aot /tmp/tmpprog.btaot && {} /tmp/tmpprog.btaot".format(AOT_BIN)
+                return cmd + " --aot /tmp/tmpprog.btaot && /tmp/tmpprog.btaot"
             else:
                 return cmd
 
@@ -130,11 +138,9 @@ class Runner(object):
         bpffeature["uprobe_refcount"] = \
             output.find("uprobe refcount (depends on Build:bcc bpf_attach_uprobe refcount): yes") != -1
         bpffeature["signal"] = output.find("send_signal: yes") != -1
-        bpffeature["iter:task"] = output.find("iter:task: yes") != -1
-        bpffeature["iter:task_file"] = output.find("iter:task_file: yes") != -1
-        bpffeature["iter:task_vma"] = output.find("iter:task_vma: yes") != -1
+        bpffeature["iter"] = output.find("iter: yes") != -1
         bpffeature["libpath_resolv"] = output.find("bcc library path resolution: yes") != -1
-        bpffeature["dwarf"] = output.find("libdw (DWARF support): yes") != -1
+        bpffeature["dwarf"] = output.find("liblldb (DWARF support): yes") != -1
         bpffeature["kprobe_multi"] = output.find("kprobe_multi: yes") != -1
         bpffeature["uprobe_multi"] = output.find("uprobe_multi: yes") != -1
         bpffeature["aot"] = cmake_vars.LIBBCC_BPF_CONTAINS_RUNTIME
@@ -178,8 +184,7 @@ class Runner(object):
             print(warn("[   SKIP   ] ") + "%s.%s" % (test.suite, test.name))
             return Runner.SKIP_KERNEL_VERSION_MAX
 
-        full_test_name = test.suite + "." + test.name
-        if full_test_name in os.getenv("RUNTIME_TEST_DISABLE", "").split(","):
+        if test.skip_if_env_has and os.environ.get(test.skip_if_env_has[0], '') == test.skip_if_env_has[1]:
             print(warn("[   SKIP   ] ") + "%s.%s" % (test.suite, test.name))
             return Runner.SKIP_ENVIRONMENT_DISABLED
 
@@ -197,23 +202,65 @@ class Runner(object):
         # the primary RUN or PROG command, and the AFTER.
         nsenter = []
         bpf_call = "[unknown]"
+        failed_expects = []
 
         def get_pid_ns_cmd(cmd):
             return nsenter + [os.path.abspath(x) for x in cmd.split()]
 
-        def check_result(output):
+        def check_expect(expect, output):
             try:
-                if test.expect_mode == "regex":
-                    return re.search(test.expect, output, re.M)
-                elif test.expect_mode == "file":
-                    # remove leading and trailing empty lines
-                    return output.strip() == open(test.expect).read().strip()
+                if expect.mode == "text":
+                    # Raw text match on an entire line, ignoring leading/trailing whitespace
+                    return re.search(f"^\\s*{re.escape(expect.expect)}\\s*$", output, re.M)
+                elif expect.mode == "text_none":
+                    return not re.search(f"^\\s*{re.escape(expect.expect)}\\s*$", output, re.M)
+                elif expect.mode == "regex":
+                    return re.search(expect.expect, output, re.M)
+                elif expect.mode == "regex_none":
+                    return not re.search(expect.expect, output, re.M)
+                elif expect.mode == "file":
+                    with open(expect.expect) as expect_file:
+                        # remove leading and trailing empty lines
+                        return output.strip() == expect_file.read().strip()
                 else:
-                    return json.loads(output) == json.load(open(test.expect))
+                    with open(expect.expect) as expect_file:
+                        _, file_extension = os.path.splitext(expect.expect)
+                        stripped_output = output.strip()
+                        output_lines = stripped_output.splitlines()
+
+                        # ndjson files are new line delimited blocks of json
+                        # https://github.com/ndjson/ndjson-spec
+                        if file_extension == ".ndjson":
+                            stripped_file = expect_file.read().strip()
+                            file_lines = stripped_file.splitlines()
+
+                            if len(file_lines) != len(output_lines):
+                                return False
+
+                            for x in range(len(file_lines)):
+                                if json.loads(output_lines[x]) != json.loads(file_lines[x]):
+                                    return False
+
+                            return True
+
+                        if len(output_lines) != 1:
+                            print(f"Expected a single line of json ouput. Got {len(output_lines)} lines")
+                            return False
+                        return json.loads(stripped_output) == json.load(expect_file)
+
+
             except Exception as err:
                 print("ERROR in check_result: ", err)
                 return False
 
+        def check_result(output):
+            all_passed = True
+            for expect in test.expects:
+                if not check_expect(expect, output):
+                    all_passed = False
+                    failed_expects.append(expect)
+            return all_passed
+
         try:
             result = None
             timeout = False
@@ -324,6 +371,7 @@ class Runner(object):
                 shell=True,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT,
+                errors='backslashreplace',
                 env=env,
                 start_new_session=True,
                 universal_newlines=True,
@@ -338,7 +386,7 @@ class Runner(object):
                 output += nextline
                 if not attached and nextline == "__BPFTRACE_NOTIFY_PROBES_ATTACHED\n":
                     attached = True
-                    if test.expect_mode != "regex":
+                    if test.has_exact_expect:
                         output = ""  # ignore earlier ouput
                     signal.alarm(test.timeout or DEFAULT_TIMEOUT)
                     if test.after:
@@ -438,22 +486,57 @@ class Runner(object):
         else:
             print(fail("[  FAILED  ] ") + "%s.%s" % (test.suite, test.name))
             print('\tCommand: ' + bpf_call)
-            if test.expect_mode == "regex":
-                print('\tExpected REGEX: ' + test.expect)
-                print('\tFound:\n' + to_utf8(output))
-            elif test.expect_mode == "json":
-                try:
-                    expected = json.dumps(json.loads(open(test.expect).read()), indent=2)
-                except json.decoder.JSONDecodeError as err:
-                    expected = "Could not parse JSON: " + str(err)
-                try:
-                    found = json.dumps(json.loads(output), indent=2)
-                except json.decoder.JSONDecodeError as err:
-                    found = "Could not parse JSON: " + str(err)
-                print('\tExpected JSON:\n' + expected)
-                print('\tFound:\n' + found)
-            else:
-                print('\tExpected FILE:\n\t\t' + to_utf8(open(test.expect).read()))
-                print('\tFound:\n\t\t' + to_utf8(output))
+            for failed_expect in failed_expects:
+                if failed_expect.mode == "text":
+                    print('\tExpected: ' + failed_expect.expect)
+                    print('\tFound:\n' + to_utf8(output))
+                elif failed_expect.mode == "text_none":
+                    print('\tExpected no: ' + failed_expect.expect)
+                    print('\tFound:\n' + to_utf8(output))
+                elif failed_expect.mode == "regex":
+                    print('\tExpected REGEX: ' + failed_expect.expect)
+                    print('\tFound:\n' + to_utf8(output))
+                elif failed_expect.mode == "regex_none":
+                    print('\tExpected no REGEX: ' + failed_expect.expect)
+                    print('\tFound:\n' + to_utf8(output))
+                elif failed_expect.mode == "json":
+                    _, file_extension = os.path.splitext(failed_expect.expect)
+                    # ndjson files are new line delimited blocks of json
+                    # https://github.com/ndjson/ndjson-spec
+                    if file_extension == ".ndjson":
+                        with open(failed_expect.expect) as expect_file:
+                            stripped_file = expect_file.read().strip()
+                            file_lines = stripped_file.splitlines()
+
+                            print('\tExpected JSON:\n')
+                            for x in file_lines:
+                                try:
+                                    print(json.dumps(json.loads(x), indent=2))
+                                except json.decoder.JSONDecodeError as err:
+                                    print("Could not parse JSON: " + str(err)  + '\n' + "Raw Line: " + x)
+
+                        stripped_output = output.strip()
+                        output_lines = stripped_output.splitlines()
+
+                        print('\tFound:\n')
+                        for x in output_lines:
+                            try:
+                                print(json.dumps(json.loads(x), indent=2))
+                            except json.decoder.JSONDecodeError as err:
+                                print("Could not parse JSON: " + str(err) + '\n' + "Raw Output: " + x)
+                    else:
+                        try:
+                            expected = json.dumps(json.loads(open(failed_expect.expect).read()), indent=2)
+                        except json.decoder.JSONDecodeError as err:
+                            expected = "Could not parse JSON: " + str(err) + '\n' + "Raw File: " + expected
+                        try:
+                            found = json.dumps(json.loads(output), indent=2)
+                        except json.decoder.JSONDecodeError as err:
+                            found = "Could not parse JSON: " + str(err) + '\n' + "Raw Output: " + output
+                        print('\tExpected JSON:\n' + expected)
+                        print('\tFound:\n' + found)
+                else:
+                    print('\tExpected FILE:\n\t\t' + to_utf8(open(failed_expect.expect).read()))
+                    print('\tFound:\n\t\t' + to_utf8(output))
             print_befores_and_after_output()
             return Runner.FAIL
