@@ -4,14 +4,14 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-int main() {
-  volatile void* addr = mmap(
-      (void*)0x10000000,
-      2 << 20,
-      PROT_READ | PROT_WRITE,
-      MAP_PRIVATE | MAP_ANONYMOUS,
-      -1,
-      0);
+int main()
+{
+  volatile void* addr = mmap((void*)0x10000000,
+                             2 << 20,
+                             PROT_READ | PROT_WRITE,
+                             MAP_PRIVATE | MAP_ANONYMOUS,
+                             -1,
+                             0);
 
   if ((long)addr < 0) {
     perror("mmap");
@@ -28,8 +28,7 @@ int main() {
   fclose(addr_fp);
 
   uint8_t i = 0;
-  while (i < 10)
-  {
+  while (i < 10) {
     *((volatile uint8_t*)addr) = i++;
     // 250ms*10 sleep, enough for watchpoint trigger
     usleep(250 * 1000);
